@@ -1,5 +1,7 @@
 /*
- * Copyright 2013-2019 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2013-2021 the Pacemaker project contributors
+ *
+ * The version control history for this file may have further details.
  *
  * This source code is licensed under the GNU General Public License version 2
  * or later (GPLv2+) WITHOUT ANY WARRANTY.
@@ -8,7 +10,7 @@
 #include <crm_internal.h>
 #include <crm/msg_xml.h>
 #include <crm/cluster.h>
-#include <crm/cluster/election.h>
+#include <crm/cluster/election_internal.h>
 
 #include "pacemaker-attrd.h"
 
@@ -149,6 +151,14 @@ attrd_remove_voter(const crm_node_t *peer)
          * attributes.
          */
         attrd_start_election_if_needed();
+
+    /* If an election is in progress, we need to call election_check(), in case
+     * this lost peer is the only one that hasn't voted, otherwise the election
+     * would be pending until it's timed out.
+     */
+    } else if (election_state(writer) == election_in_progress) {
+       crm_debug("Checking election status upon loss of voter %s", peer->uname);
+       election_check(writer);
     }
 }
 
