@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2020 the Pacemaker project contributors
+ * Copyright 2004-2021 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -11,6 +11,7 @@
 #include <crm/crm.h>
 #include <crm/msg_xml.h>
 #include <crm/common/xml.h>
+#include <crm/stonith-ng.h>
 #include <crm/fencing/internal.h>
 
 #include <pacemaker-controld.h>
@@ -34,14 +35,12 @@ static bool fence_reaction_panic = FALSE;
 static unsigned long int stonith_max_attempts = 10;
 static GHashTable *stonith_failures = NULL;
 
-// crmd_opts defines default for stonith-max-attempts, so value is never NULL
 void
 update_stonith_max_attempts(const char *value)
 {
-    if (pcmk__str_eq(value, CRM_INFINITY_S, pcmk__str_casei)) {
-       stonith_max_attempts = CRM_SCORE_INFINITY;
-    } else {
-       stonith_max_attempts = (unsigned long int) crm_parse_ll(value, NULL);
+    stonith_max_attempts = char2score(value);
+    if (stonith_max_attempts < 1UL) {
+        stonith_max_attempts = 10UL;
     }
 }
 
@@ -134,7 +133,7 @@ st_fail_count_increment(const char *target)
     struct st_fail_rec *rec = NULL;
 
     if (stonith_failures == NULL) {
-        stonith_failures = crm_str_table_new();
+        stonith_failures = pcmk__strkey_table(free, free);
     }
 
     rec = g_hash_table_lookup(stonith_failures, target);
@@ -212,8 +211,8 @@ send_stonith_update(crm_action_t *action, const char *target, const char *uuid)
 
     /* we have to mark whether or not remote nodes have already been fenced */
     if (peer->flags & crm_remote_node) {
-        time_t now = time(NULL);
-        char *now_s = crm_itoa(now);
+        char *now_s = pcmk__ttoa(time(NULL));
+
         crm_xml_add(node_state, XML_NODE_IS_FENCED, now_s);
         free(now_s);
     }
@@ -267,7 +266,7 @@ abort_for_stonith_failure(enum transition_action abort_action,
  * notifications once a new DC is elected.
  */
 
-static GListPtr stonith_cleanup_list = NULL;
+static GList *stonith_cleanup_list = NULL;
 
 /*!
  * \internal
@@ -289,10 +288,10 @@ add_stonith_cleanup(const char *target) {
 void
 remove_stonith_cleanup(const char *target)
 {
-    GListPtr iter = stonith_cleanup_list;
+    GList *iter = stonith_cleanup_list;
 
     while (iter != NULL) {
-        GListPtr tmp = iter;
+        GList *tmp = iter;
         char *iter_name = tmp->data;
 
         iter = iter->next;
@@ -312,7 +311,7 @@ void
 purge_stonith_cleanup()
 {
     if (stonith_cleanup_list) {
-        GListPtr iter = NULL;
+        GList *iter = NULL;
 
         for (iter = stonith_cleanup_list; iter != NULL; iter = iter->next) {
             char *target = iter->data;
@@ -332,7 +331,7 @@ purge_stonith_cleanup()
 void
 execute_stonith_cleanup()
 {
-    GListPtr iter;
+    GList *iter;
 
     for (iter = stonith_cleanup_list; iter != NULL; iter = iter->next) {
         char *target = iter->data;
@@ -362,7 +361,7 @@ static char *te_client_id = NULL;
 static gboolean
 fail_incompletable_stonith(crm_graph_t *graph)
 {
-    GListPtr lpc = NULL;
+    GList *lpc = NULL;
     const char *task = NULL;
     xmlNode *last_action = NULL;
 
@@ -371,25 +370,25 @@ fail_incompletable_stonith(crm_graph_t *graph)
     }
 
     for (lpc = graph->synapses; lpc != NULL; lpc = lpc->next) {
-        GListPtr lpc2 = NULL;
+        GList *lpc2 = NULL;
         synapse_t *synapse = (synapse_t *) lpc->data;
 
-        if (synapse->confirmed) {
+        if (pcmk_is_set(synapse->flags, pcmk__synapse_confirmed)) {
             continue;
         }
 
         for (lpc2 = synapse->actions; lpc2 != NULL; lpc2 = lpc2->next) {
             crm_action_t *action = (crm_action_t *) lpc2->data;
 
-            if (action->type != action_type_crm || action->confirmed) {
+            if (action->type != action_type_crm || pcmk_is_set(action->flags, pcmk__graph_action_confirmed)) {
                 continue;
             }
 
             task = crm_element_value(action->xml, XML_LRM_ATTR_TASK);
             if (task && pcmk__str_eq(task, CRM_OP_FENCE, pcmk__str_casei)) {
-                action->failed = TRUE;
+                crm__set_graph_action_flags(action, pcmk__graph_action_failed);
                 last_action = action->xml;
-                update_graph(graph, action);
+                pcmk__update_graph(graph, action);
                 crm_notice("Failing action %d (%s): fencer terminated",
                            action->id, ID(action->xml));
             }
@@ -511,7 +510,8 @@ tengine_stonith_notify(stonith_t *st, stonith_event_t *st_event)
                st_event->origin, st_event->id);
 
     if (st_event->result == pcmk_ok) {
-        crm_node_t *peer = crm_find_known_peer_full(0, st_event->target, CRM_GET_PEER_ANY);
+        crm_node_t *peer = pcmk__search_known_node_cache(0, st_event->target,
+                                                         CRM_GET_PEER_ANY);
         const char *uuid = NULL;
         gboolean we_are_executioner = pcmk__str_eq(st_event->executioner,
                                                    fsa_our_uname,
@@ -753,24 +753,39 @@ tengine_stonith_callback(stonith_t *stonith, stonith_callback_data_t *data)
         const char *op = crm_meta_value(action->params, "stonith_action");
 
         crm_info("Stonith operation %d for %s passed", call_id, target);
-        if (action->confirmed == FALSE) {
+        if (!(pcmk_is_set(action->flags, pcmk__graph_action_confirmed))) {
             te_action_confirmed(action, NULL);
             if (pcmk__str_eq("on", op, pcmk__str_casei)) {
                 const char *value = NULL;
-                char *now = crm_ttoa(time(NULL));
+                char *now = pcmk__ttoa(time(NULL));
+                gboolean is_remote_node = FALSE;
 
-                update_attrd(target, CRM_ATTR_UNFENCED, now, NULL, FALSE);
+                /* This check is not 100% reliable, since this node is not
+                 * guaranteed to have the remote node cached. However, it
+                 * doesn't have to be reliable, since the attribute manager can
+                 * learn a node's "remoteness" by other means sooner or later.
+                 * This allows it to learn more quickly if this node does have
+                 * the information.
+                 */
+                if (g_hash_table_lookup(crm_remote_peer_cache, uuid) != NULL) {
+                    is_remote_node = TRUE;
+                }
+
+                update_attrd(target, CRM_ATTR_UNFENCED, now, NULL,
+                             is_remote_node);
                 free(now);
 
                 value = crm_meta_value(action->params, XML_OP_ATTR_DIGESTS_ALL);
-                update_attrd(target, CRM_ATTR_DIGESTS_ALL, value, NULL, FALSE);
+                update_attrd(target, CRM_ATTR_DIGESTS_ALL, value, NULL,
+                             is_remote_node);
 
                 value = crm_meta_value(action->params, XML_OP_ATTR_DIGESTS_SECURE);
-                update_attrd(target, CRM_ATTR_DIGESTS_SECURE, value, NULL, FALSE);
+                update_attrd(target, CRM_ATTR_DIGESTS_SECURE, value, NULL,
+                             is_remote_node);
 
-            } else if (action->sent_update == FALSE) {
+            } else if (!(pcmk_is_set(action->flags, pcmk__graph_action_sent_update))) {
                 send_stonith_update(action, target, uuid);
-                action->sent_update = TRUE;
+                crm__set_graph_action_flags(action, pcmk__graph_action_sent_update);
             }
         }
         st_fail_count_reset(target);
@@ -779,7 +794,7 @@ tengine_stonith_callback(stonith_t *stonith, stonith_callback_data_t *data)
         const char *target = crm_element_value(action->xml, XML_LRM_ATTR_TARGET);
         enum transition_action abort_action = tg_restart;
 
-        action->failed = TRUE;
+        crm__set_graph_action_flags(action, pcmk__graph_action_failed);
         crm_notice("Stonith operation %d for %s failed (%s): aborting transition.",
                    call_id, target, pcmk_strerror(rc));
 
@@ -799,7 +814,7 @@ tengine_stonith_callback(stonith_t *stonith, stonith_callback_data_t *data)
         abort_for_stonith_failure(abort_action, target, NULL);
     }
 
-    update_graph(transition_graph, action);
+    pcmk__update_graph(transition_graph, action);
     trigger_graph();
 
   bail:
@@ -813,13 +828,14 @@ fence_with_delay(const char *target, const char *type, const char *delay)
 {
     uint32_t options = st_opt_none; // Group of enum stonith_call_options
     int timeout_sec = (int) (transition_graph->stonith_timeout / 1000);
+    int delay_i;
 
     if (crmd_join_phase_count(crm_join_confirmed) == 1) {
         stonith__set_call_options(options, target, st_opt_allow_suicide);
     }
+    pcmk__scan_min_int(delay, &delay_i, 0);
     return stonith_api->cmds->fence_with_delay(stonith_api, options, target,
-                                               type, timeout_sec, 0,
-                                               crm_atoi(delay, "0"));
+                                               type, timeout_sec, 0, delay_i);
 }
 
 gboolean
@@ -871,6 +887,19 @@ te_fence_node(crm_graph_t *graph, crm_action_t *action)
     return TRUE;
 }
 
+bool
+controld_verify_stonith_watchdog_timeout(const char *value)
+{
+    gboolean rv = TRUE;
+
+    if (stonith_api && (stonith_api->state != stonith_disconnected) &&
+        stonith__watchdog_fencing_enabled_for_node_api(stonith_api,
+                                                       fsa_our_uname)) {
+        rv = pcmk__valid_sbd_timeout(value);
+    }
+    return rv;
+}
+
 /* end stonith API client functions */
 
 
