@@ -1,14 +1,15 @@
 /*
- * Copyright 2004-2019 the Pacemaker project contributors
+ * Copyright 2004-2021 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
  * This source code is licensed under the GNU Lesser General Public License
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
+#ifndef CONTROLD_LRM__H
+#  define CONTROLD_LRM__H
 
 #include <controld_messages.h>
-#include <controld_metadata.h>
 
 extern gboolean verify_stopped(enum crmd_fsa_state cur_state, int log_level);
 void lrm_clear_last_failure(const char *rsc_id, const char *node_name,
@@ -33,19 +34,36 @@ typedef struct resource_history_s {
 
 void history_free(gpointer data);
 
-/* TODO - Replace this with lrmd_event_data_t */
-struct recurring_op_s {
+enum active_op_e {
+    active_op_remove    = (1 << 0),
+    active_op_cancelled = (1 << 1),
+};
+
+// In-flight action (recurring or pending)
+typedef struct active_op_s {
     guint interval_ms;
     int call_id;
-    gboolean remove;
-    gboolean cancelled;
+    uint32_t flags; // bitmask of active_op_e
     time_t start_time;
+    time_t lock_time;
     char *rsc_id;
     char *op_type;
     char *op_key;
     char *user_data;
     GHashTable *params;
-};
+} active_op_t;
+
+#define controld_set_active_op_flags(active_op, flags_to_set) do {          \
+        (active_op)->flags = pcmk__set_flags_as(__func__, __LINE__,         \
+            LOG_TRACE, "Active operation", (active_op)->op_key,             \
+            (active_op)->flags, (flags_to_set), #flags_to_set);             \
+    } while (0)
+
+#define controld_clear_active_op_flags(active_op, flags_to_clear) do {      \
+        (active_op)->flags = pcmk__clear_flags_as(__func__, __LINE__,       \
+            LOG_TRACE, "Active operation", (active_op)->op_key,             \
+            (active_op)->flags, (flags_to_clear), #flags_to_clear);         \
+    } while (0)
 
 typedef struct lrm_state_s {
     const char *node_name;
@@ -119,9 +137,9 @@ lrm_state_t *lrm_state_find_or_create(const char *node_name);
  */
 void lrm_state_disconnect_only(lrm_state_t * lrm_state);
 void lrm_state_disconnect(lrm_state_t * lrm_state);
-int lrm_state_ipc_connect(lrm_state_t * lrm_state);
-int lrm_state_remote_connect_async(lrm_state_t * lrm_state, const char *server, int port,
-                                   int timeout);
+int controld_connect_local_executor(lrm_state_t *lrm_state);
+int controld_connect_remote_executor(lrm_state_t *lrm_state, const char *server,
+                                     int port, int timeout);
 int lrm_state_is_connected(lrm_state_t * lrm_state);
 int lrm_state_poke_connection(lrm_state_t * lrm_state);
 
@@ -131,11 +149,11 @@ int lrm_state_get_metadata(lrm_state_t * lrm_state,
                            const char *agent, char **output, enum lrmd_call_options options);
 int lrm_state_cancel(lrm_state_t *lrm_state, const char *rsc_id,
                      const char *action, guint interval_ms);
-int lrm_state_exec(lrm_state_t *lrm_state, const char *rsc_id,
-                   const char *action, const char *userdata, guint interval_ms,
-                   int timeout, /* ms */
-                   int start_delay,     /* ms */
-                   lrmd_key_value_t * params);
+int controld_execute_resource_agent(lrm_state_t *lrm_state, const char *rsc_id,
+                                    const char *action, const char *userdata,
+                                    guint interval_ms, int timeout_ms,
+                                    int start_delay_ms,
+                                    lrmd_key_value_t *params, int *call_id);
 lrmd_rsc_info_t *lrm_state_get_rsc_info(lrm_state_t * lrm_state,
                                         const char *rsc_id, enum lrmd_call_options options);
 int lrm_state_register_rsc(lrm_state_t * lrm_state,
@@ -151,11 +169,11 @@ gboolean is_remote_lrmd_ra(const char *agent, const char *provider, const char *
 lrmd_rsc_info_t *remote_ra_get_rsc_info(lrm_state_t * lrm_state, const char *rsc_id);
 int remote_ra_cancel(lrm_state_t *lrm_state, const char *rsc_id,
                      const char *action, guint interval_ms);
-int remote_ra_exec(lrm_state_t *lrm_state, const char *rsc_id,
-                   const char *action, const char *userdata, guint interval_ms,
-                   int timeout, /* ms */
-                   int start_delay,     /* ms */
-                   lrmd_key_value_t * params);
+int controld_execute_remote_agent(lrm_state_t *lrm_state, const char *rsc_id,
+                                  const char *action, const char *userdata,
+                                  guint interval_ms, int timeout_ms,
+                                  int start_delay_ms, lrmd_key_value_t *params,
+                                  int *call_id);
 void remote_ra_cleanup(lrm_state_t * lrm_state);
 void remote_ra_fail(const char *node_name);
 void remote_ra_process_pseudo(xmlNode *xml);
@@ -164,4 +182,11 @@ void remote_ra_process_maintenance_nodes(xmlNode *xml);
 gboolean remote_ra_controlling_guest(lrm_state_t * lrm_state);
 
 void process_lrm_event(lrm_state_t *lrm_state, lrmd_event_data_t *op,
-                       struct recurring_op_s *pending, xmlNode *action_xml);
+                       active_op_t *pending, xmlNode *action_xml);
+void controld_ack_event_directly(const char *to_host, const char *to_sys,
+                                 lrmd_rsc_info_t *rsc, lrmd_event_data_t *op,
+                                 const char *rsc_id);
+void controld_rc2event(lrmd_event_data_t *event, int rc);
+void controld_trigger_delete_refresh(const char *from_sys, const char *rsc_id);
+
+#endif
