@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2020 the Pacemaker project contributors
+ * Copyright 2004-2021 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -241,13 +241,14 @@ process_resource_updates(const char *node, xmlNode *xml, xmlNode *change,
 
     if (xml == NULL) {
         return;
-
-    } else if (strcmp((const char*)xml->name, XML_CIB_TAG_LRM) == 0) {
-        xml = first_named_child(xml, XML_LRM_TAG_RESOURCES);
-        crm_trace("Got %p in %s", xml, XML_CIB_TAG_LRM);
     }
 
-    CRM_ASSERT(strcmp((const char*)xml->name, XML_LRM_TAG_RESOURCES) == 0);
+    if (strcmp(TYPE(xml), XML_CIB_TAG_LRM) == 0) {
+        xml = first_named_child(xml, XML_LRM_TAG_RESOURCES);
+        CRM_CHECK(xml != NULL, return);
+    }
+
+    CRM_CHECK(strcmp(TYPE(xml), XML_LRM_TAG_RESOURCES) == 0, return);
 
     /*
      * Updates by, or in response to, TE actions will never contain updates
@@ -276,24 +277,6 @@ process_resource_updates(const char *node, xmlNode *xml, xmlNode *change,
     }
 }
 
-#define NODE_PATT "/lrm[@id="
-static char *get_node_from_xpath(const char *xpath) 
-{
-    char *nodeid = NULL;
-    char *tmp = strstr(xpath, NODE_PATT);
-
-    if(tmp) {
-        tmp += strlen(NODE_PATT);
-        tmp += 1;
-
-        nodeid = strdup(tmp);
-        tmp = strstr(nodeid, "\'");
-        CRM_ASSERT(tmp);
-        tmp[0] = 0;
-    }
-    return nodeid;
-}
-
 static char *extract_node_uuid(const char *xpath) 
 {
     char *mutable_path = strdup(xpath);
@@ -522,19 +505,19 @@ te_update_diff_v2(xmlNode *diff)
             process_resource_updates(ID(match), match, change, op, xpath);
 
         } else if (strcmp(name, XML_LRM_TAG_RESOURCES) == 0) {
-            char *local_node = get_node_from_xpath(xpath);
+            char *local_node = pcmk__xpath_node_id(xpath, "lrm");
 
             process_resource_updates(local_node, match, change, op, xpath);
             free(local_node);
 
         } else if (strcmp(name, XML_LRM_TAG_RESOURCE) == 0) {
-            char *local_node = get_node_from_xpath(xpath);
+            char *local_node = pcmk__xpath_node_id(xpath, "lrm");
 
             process_lrm_resource_diff(match, local_node);
             free(local_node);
 
         } else if (strcmp(name, XML_LRM_TAG_RSC_OP) == 0) {
-            char *local_node = get_node_from_xpath(xpath);
+            char *local_node = pcmk__xpath_node_id(xpath, "lrm");
 
             process_graph_event(match, local_node);
             free(local_node);
@@ -701,9 +684,10 @@ action_timer_callback(gpointer data)
                 (on_node? on_node : ""), (task? task : "unknown action"),
                 (via_node? via_node : "controller"),
                 timer->timeout + transition_graph->network_delay);
-        print_action(LOG_ERR, "Aborting transition, action lost: ", timer->action);
+        pcmk__log_graph_action(LOG_ERR, timer->action);
+
+        crm__set_graph_action_flags(timer->action, pcmk__graph_action_failed);
 
-        timer->action->failed = TRUE;
         te_action_confirmed(timer->action, transition_graph);
         abort_transition(INFINITY, tg_restart, "Action lost", NULL);
 
