@@ -1,5 +1,5 @@
 /*
- * Copyright 2012-2020 the Pacemaker project contributors
+ * Copyright 2012-2021 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -19,6 +19,7 @@
 #include <crm/pengine/status.h>
 #include <crm/pengine/internal.h>
 #include <crm/cib.h>
+#include <crm/cib/internal.h>
 #include <crm/lrmd.h>
 
 static pcmk__cli_option_t long_options[] = {
@@ -121,7 +122,6 @@ static pcmk__cli_option_t long_options[] = {
     { 0, 0, 0, 0 }
 };
 
-static cib_t *cib_conn = NULL;
 static int exec_call_id = 0;
 static int exec_call_opts = 0;
 static gboolean start_test(gpointer user_data);
@@ -169,8 +169,8 @@ test_exit(crm_exit_t exit_code)
              lrmd_event_type2str(event->type),                          \
              event->rsc_id,                                             \
              event->op_type ? event->op_type : "none",                  \
-             services_ocf_exitcode_str(event->rc),                              \
-             services_lrm_status_str(event->op_status));                \
+             services_ocf_exitcode_str(event->rc),                      \
+             pcmk_exec_status_str(event->op_status));                   \
     crm_info("%s", event_buf_v0);
 
 static void
@@ -196,7 +196,8 @@ read_events(lrmd_event_data_t * event)
             print_result(printf("API-CALL SUCCESSFUL for 'exec'\n"));
         } else {
             print_result(printf("API-CALL FAILURE for 'exec', rc:%d lrmd_op_status:%s\n",
-                                event->rc, services_lrm_status_str(event->op_status)));
+                                event->rc,
+                                pcmk_exec_status_str(event->op_status)));
             test_exit(CRM_EX_ERROR);
         }
 
@@ -244,7 +245,7 @@ try_connect(void)
 
     lrmd_conn->cmds->set_callback(lrmd_conn, connection_events);
     for (; num_tries < tries; num_tries++) {
-        rc = lrmd_conn->cmds->connect_async(lrmd_conn, "pacemaker-execd", 3000);
+        rc = lrmd_conn->cmds->connect_async(lrmd_conn, crm_system_name, 3000);
 
         if (!rc) {
             return;             /* we'll hear back in async callback */
@@ -442,22 +443,10 @@ generate_params(void)
     }
     pe__set_working_set_flags(data_set, pe_flag_no_counts|pe_flag_no_compat);
 
-    cib_conn = cib_new();
-    rc = cib_conn->cmds->signon(cib_conn, "cts-exec-helper", cib_query);
-    if (rc != pcmk_ok) {
-        crm_err("Could not connect to the CIB manager: %s", pcmk_strerror(rc));
-        rc = -1;
-        goto param_gen_bail;
-    }
+    rc = cib__signon_query(NULL, &cib_xml_copy);
 
-    rc = cib_conn->cmds->query(cib_conn, NULL, &cib_xml_copy, cib_scope_local | cib_sync_call);
-    if (rc != pcmk_ok) {
-        crm_err("Error retrieving cib copy: %s (%d)", pcmk_strerror(rc), rc);
-        goto param_gen_bail;
-
-    } else if (cib_xml_copy == NULL) {
-        rc = -ENODATA;
-        crm_err("Error retrieving cib copy: %s (%d)", pcmk_strerror(rc), rc);
+    if (rc != pcmk_rc_ok) {
+        crm_err("CIB query failed: %s", pcmk_rc_str(rc));
         goto param_gen_bail;
     }
 
@@ -482,13 +471,12 @@ generate_params(void)
         goto param_gen_bail;
     }
 
-    params = crm_str_table_new();
-    meta = crm_str_table_new();
+    params = pe_rsc_params(rsc, NULL, data_set);
+    meta = pcmk__strkey_table(free, free);
 
-    get_rsc_attributes(params, rsc, NULL, data_set);
     get_meta_attributes(meta, rsc, NULL, data_set);
 
-    if (params) {
+    if (params != NULL) {
         char *key = NULL;
         char *value = NULL;
 
@@ -496,7 +484,6 @@ generate_params(void)
         while (g_hash_table_iter_next(&iter, (gpointer *) & key, (gpointer *) & value)) {
             options.params = lrmd_key_value_add(options.params, key, value);
         }
-        g_hash_table_destroy(params);
     }
 
     if (meta) {
@@ -529,7 +516,7 @@ main(int argc, char **argv)
     gboolean use_tls = FALSE;
     crm_trigger_t *trig;
 
-    crm_log_cli_init("cts-exec-helper");
+    pcmk__cli_init_logging("cts-exec-helper", 0);
     pcmk__set_cli_options(NULL, "<mode> [options]", long_options,
                           "inject commands into the Pacemaker executor, "
                           "and watch for events");
@@ -682,11 +669,6 @@ main(int argc, char **argv)
     mainloop = g_main_loop_new(NULL, FALSE);
     g_main_loop_run(mainloop);
 
-    if (cib_conn != NULL) {
-        cib_conn->cmds->signoff(cib_conn);
-        cib_delete(cib_conn);
-    }
-
     test_exit(CRM_EX_OK);
     return CRM_EX_OK;
 }
