@@ -1,5 +1,5 @@
 /*
- * Copyright 2012-2020 the Pacemaker project contributors
+ * Copyright 2012-2021 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -22,6 +22,7 @@
 
 #include <crm/crm.h>
 #include <crm/services.h>
+#include <crm/services_internal.h>
 #include <crm/common/mainloop.h>
 #include <crm/common/ipc.h>
 #include <crm/common/ipc_internal.h>
@@ -29,8 +30,6 @@
 
 #include "pacemaker-execd.h"
 
-#define EXIT_REASON_MAX_LEN 128
-
 GHashTable *rsc_list = NULL;
 
 typedef struct lrmd_cmd_s {
@@ -40,8 +39,6 @@ typedef struct lrmd_cmd_s {
     int timeout_orig;
 
     int call_id;
-    int exec_rc;
-    int lrmd_op_status;
 
     int call_opts;
     /* Timer ids, must be removed on cmd destruction. */
@@ -57,10 +54,10 @@ typedef struct lrmd_cmd_s {
     char *rsc_id;
     char *action;
     char *real_action;
-    char *exit_reason;
-    char *output;
     char *userdata_str;
 
+    pcmk__action_result_t result;
+
     /* We can track operation queue time and run time, to be saved with the CIB
      * resource history (and displayed in cluster status). We need
      * high-resolution monotonic time for this purpose, so we use
@@ -90,7 +87,7 @@ typedef struct lrmd_cmd_s {
     time_t epoch_last_run;          // Epoch timestamp of when op last ran
     time_t epoch_rcchange;          // Epoch timestamp of when rc last changed
 
-    int first_notify_sent;
+    bool first_notify_sent;
     int last_notify_rc;
     int last_notify_op_status;
     int last_pid;
@@ -193,30 +190,59 @@ cmd_original_times(lrmd_cmd_t * cmd)
 }
 #endif
 
-static void
-log_finished(lrmd_cmd_t * cmd, int exec_time, int queue_time)
+static inline bool
+action_matches(lrmd_cmd_t *cmd, const char *action, guint interval_ms)
 {
-    char pid_str[32] = { 0, };
-    int log_level = LOG_INFO;
+    return (cmd->interval_ms == interval_ms)
+           && pcmk__str_eq(cmd->action, action, pcmk__str_casei);
+}
 
-    if (cmd->last_pid) {
-        snprintf(pid_str, 32, "%d", cmd->last_pid);
-    }
+/*!
+ * \internal
+ * \brief Log the result of an asynchronous command
+ *
+ * \param[in] cmd            Command to log result for
+ * \param[in] exec_time_ms   Execution time in milliseconds, if known
+ * \param[in] queue_time_ms  Queue time in milliseconds, if known
+ */
+static void
+log_finished(lrmd_cmd_t *cmd, int exec_time_ms, int queue_time_ms)
+{
+    int log_level = LOG_INFO;
+    GString *str = g_string_sized_new(100); // reasonable starting size
 
     if (pcmk__str_eq(cmd->action, "monitor", pcmk__str_casei)) {
         log_level = LOG_DEBUG;
     }
+
+    g_string_printf(str, "%s %s (call %d",
+                    cmd->rsc_id, cmd->action, cmd->call_id);
+    if (cmd->last_pid != 0) {
+        g_string_append_printf(str, ", PID %d", cmd->last_pid);
+    }
+    if (cmd->result.execution_status == PCMK_EXEC_DONE) {
+        g_string_append_printf(str, ") exited with status %d",
+                               cmd->result.exit_status);
+    } else {
+        g_string_append_printf(str, ") could not be executed: %s",
+                               pcmk_exec_status_str(cmd->result.execution_status));
+    }
+    if (cmd->result.exit_reason != NULL) {
+        g_string_append_printf(str, " (%s)", cmd->result.exit_reason);
+    }
+
 #ifdef PCMK__TIME_USE_CGT
-    do_crm_log(log_level, "%s %s (call %d%s%s) exited with status %d"
-               " (execution time %dms, queue time %dms)",
-               cmd->rsc_id, cmd->action, cmd->call_id,
-               (cmd->last_pid? ", PID " : ""), pid_str, cmd->exec_rc,
-               exec_time, queue_time);
-#else
-    do_crm_log(log_level, "%s %s (call %d%s%s) exited with status %d"
-               cmd->rsc_id, cmd->action, cmd->call_id,
-               (cmd->last_pid? ", PID " : ""), pid_str, cmd->exec_rc);
+    g_string_append_printf(str, " (execution time %s",
+                           pcmk__readable_interval(exec_time_ms));
+    if (queue_time_ms > 0) {
+        g_string_append_printf(str, " after being queued %s",
+                               pcmk__readable_interval(queue_time_ms));
+    }
+    g_string_append(str, ")");
 #endif
+
+    do_crm_log(log_level, "%s", str->str);
+    g_string_free(str, TRUE);
 }
 
 static void
@@ -321,13 +347,12 @@ free_lrmd_cmd(lrmd_cmd_t * cmd)
     if (cmd->params) {
         g_hash_table_destroy(cmd->params);
     }
+    pcmk__reset_result(&(cmd->result));
     free(cmd->origin);
     free(cmd->action);
     free(cmd->real_action);
     free(cmd->userdata_str);
     free(cmd->rsc_id);
-    free(cmd->output);
-    free(cmd->exit_reason);
     free(cmd->client_id);
     free(cmd);
 }
@@ -385,40 +410,52 @@ start_delay_helper(gpointer data)
     return FALSE;
 }
 
-static gboolean
+/*!
+ * \internal
+ * \brief Check whether a list already contains the equivalent of a given action
+ */
+static lrmd_cmd_t *
+find_duplicate_action(GList *action_list, lrmd_cmd_t *cmd)
+{
+    for (GList *item = action_list; item != NULL; item = item->next) {
+        lrmd_cmd_t *dup = item->data;
+
+        if (action_matches(cmd, dup->action, dup->interval_ms)) {
+            return dup;
+        }
+    }
+    return NULL;
+}
+
+static bool
 merge_recurring_duplicate(lrmd_rsc_t * rsc, lrmd_cmd_t * cmd)
 {
-    GListPtr gIter = NULL;
     lrmd_cmd_t * dup = NULL;
-    gboolean dup_pending = FALSE;
+    bool dup_pending = true;
 
     if (cmd->interval_ms == 0) {
-        return 0;
+        return false;
     }
 
-    for (gIter = rsc->pending_ops; gIter != NULL; gIter = gIter->next) {
-        dup = gIter->data;
-        if (pcmk__str_eq(cmd->action, dup->action, pcmk__str_casei)
-            && (cmd->interval_ms == dup->interval_ms)) {
-            dup_pending = TRUE;
-            goto merge_dup;
+    // Search for a duplicate of this action (in-flight or not)
+    dup = find_duplicate_action(rsc->pending_ops, cmd);
+    if (dup == NULL) {
+        dup_pending = false;
+        dup = find_duplicate_action(rsc->recurring_ops, cmd);
+        if (dup == NULL) {
+            return false;
         }
     }
 
-    /* if dup is in recurring_ops list, that means it has already executed
-     * and is in the interval loop. we can't just remove it in this case. */
-    for (gIter = rsc->recurring_ops; gIter != NULL; gIter = gIter->next) {
-        dup = gIter->data;
-        if (pcmk__str_eq(cmd->action, dup->action, pcmk__str_casei)
-            && (cmd->interval_ms == dup->interval_ms)) {
-            goto merge_dup;
-        }
+    /* Do not merge fencing monitors marked for cancellation, so we can reply to
+     * the cancellation separately.
+     */
+    if (pcmk__str_eq(rsc->class, PCMK_RESOURCE_CLASS_STONITH,
+                     pcmk__str_casei)
+        && (dup->result.execution_status == PCMK_EXEC_CANCELLED)) {
+        return false;
     }
 
-    return FALSE;
-merge_dup:
-
-
     /* This should not occur. If it does, we need to investigate how something
      * like this is possible in the controller.
      */
@@ -427,43 +464,43 @@ merge_dup:
              rsc->rsc_id, normalize_action_name(rsc, dup->action),
              dup->interval_ms);
 
-    /* merge */
-    dup->first_notify_sent = 0;
+    // Merge new action's call ID and user data into existing action
+    dup->first_notify_sent = false;
     free(dup->userdata_str);
     dup->userdata_str = cmd->userdata_str;
     cmd->userdata_str = NULL;
     dup->call_id = cmd->call_id;
-
-    if (pcmk__str_eq(rsc->class, PCMK_RESOURCE_CLASS_STONITH, pcmk__str_casei)) {
-        /* if we are waiting for the next interval, kick it off now */
-        if (dup_pending == TRUE) {
-            stop_recurring_timer(cmd);
-            stonith_recurring_op_helper(cmd);
-        }
-
-    } else if (dup_pending == FALSE) {
-        /* if we've already handed this to the service lib, kick off an early execution */
-        services_action_kick(rsc->rsc_id,
-                             normalize_action_name(rsc, dup->action),
-                             dup->interval_ms);
-    }
     free_lrmd_cmd(cmd);
+    cmd = NULL;
 
-    return TRUE;
+    /* If dup is not pending, that means it has already executed at least once
+     * and is waiting in the interval. In that case, stop waiting and initiate
+     * a new instance now.
+     */
+    if (!dup_pending) {
+        if (pcmk__str_eq(rsc->class, PCMK_RESOURCE_CLASS_STONITH,
+                         pcmk__str_casei)) {
+            stop_recurring_timer(dup);
+            stonith_recurring_op_helper(dup);
+        } else {
+            services_action_kick(rsc->rsc_id,
+                                 normalize_action_name(rsc, dup->action),
+                                 dup->interval_ms);
+        }
+    }
+    return true;
 }
 
 static void
 schedule_lrmd_cmd(lrmd_rsc_t * rsc, lrmd_cmd_t * cmd)
 {
-    gboolean dup_processed = FALSE;
     CRM_CHECK(cmd != NULL, return);
     CRM_CHECK(rsc != NULL, return);
 
     crm_trace("Scheduling %s on %s", cmd->action, rsc->rsc_id);
 
-    dup_processed = merge_recurring_duplicate(rsc, cmd);
-    if (dup_processed) {
-        /* duplicate recurring cmd found, cmds merged */
+    if (merge_recurring_duplicate(rsc, cmd)) {
+        // Equivalent of cmd has already been scheduled
         return;
     }
 
@@ -515,8 +552,8 @@ send_client_notify(gpointer key, gpointer value, gpointer user_data)
          * running as Pacemaker Remote, we may have clients proxied to other
          * IPC services in the cluster, so skip those.
          */
-        crm_trace("Skipping executor API notification to %s IPC client",
-                  client->name);
+        crm_trace("Skipping executor API notification to client %s",
+                  pcmk__client_name(client));
         return;
     }
 
@@ -536,31 +573,29 @@ send_client_notify(gpointer key, gpointer value, gpointer user_data)
             msg = pcmk_rc_str(rc);
             break;
     }
-    do_crm_log(log_level,
-               "Could not notify client %s/%s: %s " CRM_XS " rc=%d",
-               client->name, client->id, msg, rc);
+    do_crm_log(log_level, "Could not notify client %s: %s " CRM_XS " rc=%d",
+               pcmk__client_name(client), msg, rc);
 }
 
 static void
 send_cmd_complete_notify(lrmd_cmd_t * cmd)
 {
     xmlNode *notify = NULL;
+    int exec_time = 0;
+    int queue_time = 0;
 
 #ifdef PCMK__TIME_USE_CGT
-    int exec_time = time_diff_ms(NULL, &(cmd->t_run));
-    int queue_time = time_diff_ms(&cmd->t_run, &(cmd->t_queue));
-
-    log_finished(cmd, exec_time, queue_time);
-#else
-    log_finished(cmd, 0, 0);
+    exec_time = time_diff_ms(NULL, &(cmd->t_run));
+    queue_time = time_diff_ms(&cmd->t_run, &(cmd->t_queue));
 #endif
+    log_finished(cmd, exec_time, queue_time);
 
     /* if the first notify result for a cmd has already been sent earlier, and the
      * the option to only send notifies on result changes is set. Check to see
      * if the last result is the same as the new one. If so, suppress this update */
     if (cmd->first_notify_sent && (cmd->call_opts & lrmd_opt_notify_changes_only)) {
-        if (cmd->last_notify_rc == cmd->exec_rc &&
-            cmd->last_notify_op_status == cmd->lrmd_op_status) {
+        if ((cmd->last_notify_rc == cmd->result.exit_status) &&
+            (cmd->last_notify_op_status == cmd->result.execution_status)) {
 
             /* only send changes */
             return;
@@ -568,9 +603,9 @@ send_cmd_complete_notify(lrmd_cmd_t * cmd)
 
     }
 
-    cmd->first_notify_sent = 1;
-    cmd->last_notify_rc = cmd->exec_rc;
-    cmd->last_notify_op_status = cmd->lrmd_op_status;
+    cmd->first_notify_sent = true;
+    cmd->last_notify_rc = cmd->result.exit_status;
+    cmd->last_notify_op_status = cmd->result.execution_status;
 
     notify = create_xml_node(NULL, T_LRMD_NOTIFY);
 
@@ -578,8 +613,8 @@ send_cmd_complete_notify(lrmd_cmd_t * cmd)
     crm_xml_add_int(notify, F_LRMD_TIMEOUT, cmd->timeout);
     crm_xml_add_ms(notify, F_LRMD_RSC_INTERVAL, cmd->interval_ms);
     crm_xml_add_int(notify, F_LRMD_RSC_START_DELAY, cmd->start_delay);
-    crm_xml_add_int(notify, F_LRMD_EXEC_RC, cmd->exec_rc);
-    crm_xml_add_int(notify, F_LRMD_OP_STATUS, cmd->lrmd_op_status);
+    crm_xml_add_int(notify, F_LRMD_EXEC_RC, cmd->result.exit_status);
+    crm_xml_add_int(notify, F_LRMD_OP_STATUS, cmd->result.execution_status);
     crm_xml_add_int(notify, F_LRMD_CALLID, cmd->call_id);
     crm_xml_add_int(notify, F_LRMD_RSC_DELETED, cmd->rsc_deleted);
 
@@ -600,8 +635,14 @@ send_cmd_complete_notify(lrmd_cmd_t * cmd)
         crm_xml_add(notify, F_LRMD_RSC_ACTION, cmd->action);
     }
     crm_xml_add(notify, F_LRMD_RSC_USERDATA_STR, cmd->userdata_str);
-    crm_xml_add(notify, F_LRMD_RSC_OUTPUT, cmd->output);
-    crm_xml_add(notify, F_LRMD_RSC_EXIT_REASON, cmd->exit_reason);
+    crm_xml_add(notify, F_LRMD_RSC_EXIT_REASON, cmd->result.exit_reason);
+
+    if (cmd->result.action_stderr != NULL) {
+        crm_xml_add(notify, F_LRMD_RSC_OUTPUT, cmd->result.action_stderr);
+
+    } else if (cmd->result.action_stdout != NULL) {
+        crm_xml_add(notify, F_LRMD_RSC_OUTPUT, cmd->result.action_stdout);
+    }
 
     if (cmd->params) {
         char *key = NULL;
@@ -656,17 +697,15 @@ send_generic_notify(int rc, xmlNode * request)
 static void
 cmd_reset(lrmd_cmd_t * cmd)
 {
-    cmd->lrmd_op_status = 0;
     cmd->last_pid = 0;
 #ifdef PCMK__TIME_USE_CGT
     memset(&cmd->t_run, 0, sizeof(cmd->t_run));
     memset(&cmd->t_queue, 0, sizeof(cmd->t_queue));
 #endif
     cmd->epoch_last_run = 0;
-    free(cmd->exit_reason);
-    cmd->exit_reason = NULL;
-    free(cmd->output);
-    cmd->output = NULL;
+
+    pcmk__reset_result(&(cmd->result));
+    cmd->result.execution_status = PCMK_EXEC_DONE;
 }
 
 static void
@@ -689,7 +728,9 @@ cmd_finalize(lrmd_cmd_t * cmd, lrmd_rsc_t * rsc)
 
     send_cmd_complete_notify(cmd);
 
-    if (cmd->interval_ms && (cmd->lrmd_op_status == PCMK_LRM_OP_CANCELLED)) {
+    if ((cmd->interval_ms != 0)
+        && (cmd->result.execution_status == PCMK_EXEC_CANCELLED)) {
+
         if (rsc) {
             rsc->recurring_ops = g_list_remove(rsc->recurring_ops, cmd);
             rsc->pending_ops = g_list_remove(rsc->pending_ops, cmd);
@@ -706,21 +747,6 @@ cmd_finalize(lrmd_cmd_t * cmd, lrmd_rsc_t * rsc)
     }
 }
 
-static int
-ocf2uniform_rc(int rc)
-{
-    switch (rc) {
-        case PCMK_OCF_DEGRADED:
-        case PCMK_OCF_DEGRADED_MASTER:
-            break;
-        default:
-            if (rc < 0 || rc > PCMK_OCF_FAILED_MASTER)
-                return PCMK_OCF_UNKNOWN_ERROR;
-    }
-
-    return rc;
-}
-
 static int
 stonith2uniform_rc(const char *action, int rc)
 {
@@ -746,11 +772,6 @@ stonith2uniform_rc(const char *action, int rc)
             rc = PCMK_OCF_UNIMPLEMENT_FEATURE;
             break;
 
-        case -ETIME:
-        case -ETIMEDOUT:
-            rc = PCMK_OCF_TIMEOUT;
-            break;
-
         default:
             rc = PCMK_OCF_UNKNOWN_ERROR;
             break;
@@ -758,58 +779,21 @@ stonith2uniform_rc(const char *action, int rc)
     return rc;
 }
 
-#if SUPPORT_NAGIOS
 static int
-nagios2uniform_rc(const char *action, int rc)
-{
-    if (rc < 0) {
-        return PCMK_OCF_UNKNOWN_ERROR;
-    }
-
-    switch (rc) {
-        case NAGIOS_STATE_OK:
-            return PCMK_OCF_OK;
-        case NAGIOS_INSUFFICIENT_PRIV:
-            return PCMK_OCF_INSUFFICIENT_PRIV;
-        case NAGIOS_NOT_INSTALLED:
-            return PCMK_OCF_NOT_INSTALLED;
-        case NAGIOS_STATE_WARNING:
-        case NAGIOS_STATE_CRITICAL:
-        case NAGIOS_STATE_UNKNOWN:
-        case NAGIOS_STATE_DEPENDENT:
-        default:
-            return PCMK_OCF_UNKNOWN_ERROR;
-    }
-
-    return PCMK_OCF_UNKNOWN_ERROR;
-}
-#endif
-
-static int
-get_uniform_rc(const char *standard, const char *action, int rc)
-{
-    if (pcmk__str_eq(standard, PCMK_RESOURCE_CLASS_OCF, pcmk__str_casei)) {
-        return ocf2uniform_rc(rc);
-    } else if (pcmk__str_eq(standard, PCMK_RESOURCE_CLASS_STONITH, pcmk__str_casei)) {
-        return stonith2uniform_rc(action, rc);
-    } else if (pcmk__str_eq(standard, PCMK_RESOURCE_CLASS_SYSTEMD, pcmk__str_casei)) {
-        return rc;
-    } else if (pcmk__str_eq(standard, PCMK_RESOURCE_CLASS_UPSTART, pcmk__str_casei)) {
-        return rc;
-#if SUPPORT_NAGIOS
-    } else if (pcmk__str_eq(standard, PCMK_RESOURCE_CLASS_NAGIOS, pcmk__str_casei)) {
-        return nagios2uniform_rc(action, rc);
-#endif
-    } else {
-        return services_get_ocf_exitcode(action, rc);
-    }
-}
-
-static int
-action_get_uniform_rc(svc_action_t * action)
+action_get_uniform_rc(svc_action_t *action)
 {
     lrmd_cmd_t *cmd = action->cb_data;
-    return get_uniform_rc(action->standard, cmd->action, action->rc);
+
+    if (pcmk__str_eq(action->standard, PCMK_RESOURCE_CLASS_STONITH,
+                            pcmk__str_casei)) {
+        return stonith2uniform_rc(cmd->action, action->rc);
+    } else {
+        enum ocf_exitcode code = services_result2ocf(action->standard,
+                                                     cmd->action, action->rc);
+
+        // Cast variable instead of function return to keep compilers happy
+        return (int) code;
+    }
 }
 
 struct notify_new_client_data {
@@ -841,41 +825,6 @@ notify_of_new_client(pcmk__client_t *new_client)
     free_xml(data.notify);
 }
 
-static char *
-parse_exit_reason(const char *output)
-{
-    const char *cur = NULL;
-    const char *last = NULL;
-    static int cookie_len = 0;
-    char *eol = NULL;
-    size_t reason_len = EXIT_REASON_MAX_LEN;
-
-    if (output == NULL) {
-        return NULL;
-    }
-
-    if (!cookie_len) {
-        cookie_len = strlen(PCMK_OCF_REASON_PREFIX);
-    }
-
-    cur = strstr(output, PCMK_OCF_REASON_PREFIX);
-    for (; cur != NULL; cur = strstr(cur, PCMK_OCF_REASON_PREFIX)) {
-        /* skip over the cookie delimiter string */
-        cur += cookie_len;
-        last = cur;
-    }
-    if (last == NULL) {
-        return NULL;
-    }
-
-    // Truncate everything after a new line, and limit reason string size
-    eol = strchr(last, '\n');
-    if (eol) {
-        reason_len = QB_MIN(reason_len, eol - last);
-    }
-    return strndup(last, reason_len);
-}
-
 void
 client_disconnect_cleanup(const char *client_id)
 {
@@ -898,9 +847,9 @@ action_complete(svc_action_t * action)
 {
     lrmd_rsc_t *rsc;
     lrmd_cmd_t *cmd = action->cb_data;
-    const char *rclass = NULL;
 
 #ifdef PCMK__TIME_USE_CGT
+    const char *rclass = NULL;
     bool goagain = false;
 #endif
 
@@ -911,25 +860,25 @@ action_complete(svc_action_t * action)
     }
 
 #ifdef PCMK__TIME_USE_CGT
-    if (cmd->exec_rc != action->rc) {
+    if (cmd->result.exit_status != action->rc) {
         cmd->epoch_rcchange = time(NULL);
     }
 #endif
 
     cmd->last_pid = action->pid;
-    cmd->exec_rc = action_get_uniform_rc(action);
-    cmd->lrmd_op_status = action->status;
+    pcmk__set_result(&(cmd->result), action_get_uniform_rc(action),
+                     action->status, services__exit_reason(action));
     rsc = cmd->rsc_id ? g_hash_table_lookup(rsc_list, cmd->rsc_id) : NULL;
 
+#ifdef PCMK__TIME_USE_CGT
     if (rsc && pcmk__str_eq(rsc->class, PCMK_RESOURCE_CLASS_SERVICE, pcmk__str_casei)) {
         rclass = resources_find_service_class(rsc->type);
     } else if(rsc) {
         rclass = rsc->class;
     }
 
-#ifdef PCMK__TIME_USE_CGT
     if (pcmk__str_eq(rclass, PCMK_RESOURCE_CLASS_SYSTEMD, pcmk__str_casei)) {
-        if ((cmd->exec_rc == PCMK_OCF_OK)
+        if ((cmd->result.exit_status == PCMK_OCF_OK)
             && pcmk__strcase_any_of(cmd->action, "start", "stop", NULL)) {
             /* systemd returns from start and stop actions after the action
              * begins, not after it completes. We have to jump through a few
@@ -942,11 +891,10 @@ action_complete(svc_action_t * action)
 
         } else if (cmd->real_action != NULL) {
             // This is follow-up monitor to check whether start/stop completed
-            if ((cmd->lrmd_op_status == PCMK_LRM_OP_DONE)
-                && (cmd->exec_rc == PCMK_OCF_PENDING)) {
+            if (cmd->result.execution_status == PCMK_EXEC_PENDING) {
                 goagain = true;
 
-            } else if ((cmd->exec_rc == PCMK_OCF_OK)
+            } else if ((cmd->result.exit_status == PCMK_OCF_OK)
                        && pcmk__str_eq(cmd->real_action, "stop", pcmk__str_casei)) {
                 goagain = true;
 
@@ -957,18 +905,18 @@ action_complete(svc_action_t * action)
                 crm_debug("%s systemd %s is now complete (elapsed=%dms, "
                           "remaining=%dms): %s (%d)",
                           cmd->rsc_id, cmd->real_action, time_sum, timeout_left,
-                          services_ocf_exitcode_str(cmd->exec_rc),
-                          cmd->exec_rc);
+                          services_ocf_exitcode_str(cmd->result.exit_status),
+                          cmd->result.exit_status);
                 cmd_original_times(cmd);
 
                 // Monitors may return "not running", but start/stop shouldn't
-                if ((cmd->lrmd_op_status == PCMK_LRM_OP_DONE)
-                    && (cmd->exec_rc == PCMK_OCF_NOT_RUNNING)) {
+                if ((cmd->result.execution_status == PCMK_EXEC_DONE)
+                    && (cmd->result.exit_status == PCMK_OCF_NOT_RUNNING)) {
 
                     if (pcmk__str_eq(cmd->real_action, "start", pcmk__str_casei)) {
-                        cmd->exec_rc = PCMK_OCF_UNKNOWN_ERROR;
+                        cmd->result.exit_status = PCMK_OCF_UNKNOWN_ERROR;
                     } else if (pcmk__str_eq(cmd->real_action, "stop", pcmk__str_casei)) {
-                        cmd->exec_rc = PCMK_OCF_OK;
+                        cmd->result.exit_status = PCMK_OCF_OK;
                     }
                 }
             }
@@ -978,12 +926,13 @@ action_complete(svc_action_t * action)
 
 #if SUPPORT_NAGIOS
     if (rsc && pcmk__str_eq(rsc->class, PCMK_RESOURCE_CLASS_NAGIOS, pcmk__str_casei)) {
-        if (pcmk__str_eq(cmd->action, "monitor", pcmk__str_casei) &&
-            (cmd->interval_ms == 0) && cmd->exec_rc == PCMK_OCF_OK) {
+        if (action_matches(cmd, "monitor", 0)
+            && (cmd->result.exit_status == PCMK_OCF_OK)) {
             /* Successfully executed --version for the nagios plugin */
-            cmd->exec_rc = PCMK_OCF_NOT_RUNNING;
+            cmd->result.exit_status = PCMK_OCF_NOT_RUNNING;
 
-        } else if (pcmk__str_eq(cmd->action, "start", pcmk__str_casei) && cmd->exec_rc != PCMK_OCF_OK) {
+        } else if (pcmk__str_eq(cmd->action, "start", pcmk__str_casei)
+                   && (cmd->result.exit_status != PCMK_OCF_OK)) {
 #ifdef PCMK__TIME_USE_CGT
             goagain = true;
 #endif
@@ -1006,17 +955,20 @@ action_complete(svc_action_t * action)
             cmd->start_delay = delay;
             cmd->timeout = timeout_left;
 
-            if(cmd->exec_rc == PCMK_OCF_OK) {
+            if (cmd->result.exit_status == PCMK_OCF_OK) {
                 crm_debug("%s %s may still be in progress: re-scheduling (elapsed=%dms, remaining=%dms, start_delay=%dms)",
                           cmd->rsc_id, cmd->real_action, time_sum, timeout_left, delay);
 
-            } else if(cmd->exec_rc == PCMK_OCF_PENDING) {
+            } else if (cmd->result.execution_status == PCMK_EXEC_PENDING) {
                 crm_info("%s %s is still in progress: re-scheduling (elapsed=%dms, remaining=%dms, start_delay=%dms)",
                          cmd->rsc_id, cmd->action, time_sum, timeout_left, delay);
 
             } else {
                 crm_notice("%s %s failed '%s' (%d): re-scheduling (elapsed=%dms, remaining=%dms, start_delay=%dms)",
-                           cmd->rsc_id, cmd->action, services_ocf_exitcode_str(cmd->exec_rc), cmd->exec_rc, time_sum, timeout_left, delay);
+                           cmd->rsc_id, cmd->action,
+                           services_ocf_exitcode_str(cmd->result.exit_status),
+                           cmd->result.exit_status, time_sum, timeout_left,
+                           delay);
             }
 
             cmd_reset(cmd);
@@ -1030,22 +982,20 @@ action_complete(svc_action_t * action)
 
         } else {
             crm_notice("Giving up on %s %s (rc=%d): timeout (elapsed=%dms, remaining=%dms)",
-                       cmd->rsc_id, cmd->real_action?cmd->real_action:cmd->action, cmd->exec_rc, time_sum, timeout_left);
-            cmd->lrmd_op_status = PCMK_LRM_OP_TIMEOUT;
-            cmd->exec_rc = PCMK_OCF_TIMEOUT;
+                       cmd->rsc_id,
+                       (cmd->real_action? cmd->real_action : cmd->action),
+                       cmd->result.exit_status, time_sum, timeout_left);
+            pcmk__set_result(&(cmd->result), PCMK_OCF_UNKNOWN_ERROR,
+                             PCMK_EXEC_TIMEOUT,
+                             "Investigate reason for timeout, and adjust "
+                             "configured operation timeout if necessary");
             cmd_original_times(cmd);
         }
     }
 #endif
 
-    if (action->stderr_data) {
-        cmd->output = strdup(action->stderr_data);
-        cmd->exit_reason = parse_exit_reason(action->stderr_data);
-
-    } else if (action->stdout_data) {
-        cmd->output = strdup(action->stdout_data);
-    }
-
+    pcmk__set_result_output(&(cmd->result), services__grab_stdout(action),
+                            services__grab_stderr(action));
     cmd_finalize(cmd, rsc);
 }
 
@@ -1066,7 +1016,7 @@ action_complete(svc_action_t * action)
 static int
 stonith_rc2status(const char *action, guint interval_ms, int rc)
 {
-    int status = PCMK_LRM_OP_DONE;
+    int status = PCMK_EXEC_DONE;
 
     switch (rc) {
         case pcmk_ok:
@@ -1074,23 +1024,23 @@ stonith_rc2status(const char *action, guint interval_ms, int rc)
 
         case -EOPNOTSUPP:
         case -EPROTONOSUPPORT:
-            status = PCMK_LRM_OP_NOTSUPPORTED;
+            status = PCMK_EXEC_NOT_SUPPORTED;
             break;
 
         case -ETIME:
         case -ETIMEDOUT:
-            status = PCMK_LRM_OP_TIMEOUT;
+            status = PCMK_EXEC_TIMEOUT;
             break;
 
         case -ENOTCONN:
         case -ECOMM:
             // Couldn't talk to fencer
-            status = PCMK_LRM_OP_ERROR;
+            status = PCMK_EXEC_ERROR;
             break;
 
         case -ENODEV:
             // The device is not registered with the fencer
-            status = PCMK_LRM_OP_ERROR;
+            status = PCMK_EXEC_ERROR;
             break;
 
         default:
@@ -1105,18 +1055,18 @@ stonith_action_complete(lrmd_cmd_t * cmd, int rc)
     // This can be NULL if resource was removed before command completed
     lrmd_rsc_t *rsc = g_hash_table_lookup(rsc_list, cmd->rsc_id);
 
-    cmd->exec_rc = stonith2uniform_rc(cmd->action, rc);
+    cmd->result.exit_status = stonith2uniform_rc(cmd->action, rc);
 
     /* This function may be called with status already set to cancelled, if a
      * pending action was aborted. Otherwise, we need to determine status from
      * the fencer return code.
      */
-    if (cmd->lrmd_op_status != PCMK_LRM_OP_CANCELLED) {
-        cmd->lrmd_op_status = stonith_rc2status(cmd->action, cmd->interval_ms,
-                                                rc);
+    if (cmd->result.execution_status != PCMK_EXEC_CANCELLED) {
+        cmd->result.execution_status = stonith_rc2status(cmd->action,
+                                                         cmd->interval_ms, rc);
 
         // Certain successful actions change the known state of the resource
-        if (rsc && (cmd->exec_rc == PCMK_OCF_OK)) {
+        if ((rsc != NULL) && (cmd->result.exit_status == PCMK_OCF_OK)) {
             if (pcmk__str_eq(cmd->action, "start", pcmk__str_casei)) {
                 rsc->st_probe_rc = pcmk_ok; // maps to PCMK_OCF_OK
             } else if (pcmk__str_eq(cmd->action, "stop", pcmk__str_casei)) {
@@ -1125,17 +1075,22 @@ stonith_action_complete(lrmd_cmd_t * cmd, int rc)
         }
     }
 
+    // Give the user more detail than an OCF code
+    if (rc != -pcmk_err_generic) {
+        cmd->result.exit_reason = strdup(pcmk_strerror(rc));
+    }
+
     /* The recurring timer should not be running at this point in any case, but
      * as a failsafe, stop it if it is.
      */
     stop_recurring_timer(cmd);
 
     /* Reschedule this command if appropriate. If a recurring command is *not*
-     * rescheduled, its status must be PCMK_LRM_OP_CANCELLED, otherwise it will
+     * rescheduled, its status must be PCMK_EXEC_CANCELLED, otherwise it will
      * not be removed from recurring_ops by cmd_finalize().
      */
     if (rsc && (cmd->interval_ms > 0)
-        && (cmd->lrmd_op_status != PCMK_LRM_OP_CANCELLED)) {
+        && (cmd->result.execution_status != PCMK_EXEC_CANCELLED)) {
         start_recurring_timer(cmd);
     }
 
@@ -1348,43 +1303,44 @@ lrmd_rsc_execute_service_lib(lrmd_rsc_t * rsc, lrmd_cmd_t * cmd)
     if (pcmk__str_eq(rsc->class, PCMK_RESOURCE_CLASS_NAGIOS, pcmk__str_casei)
         && pcmk__str_eq(cmd->action, "stop", pcmk__str_casei)) {
 
-        cmd->exec_rc = PCMK_OCF_OK;
+        cmd->result.exit_status = PCMK_OCF_OK;
         goto exec_done;
     }
 #endif
 
-    params_copy = crm_str_table_dup(cmd->params);
+    params_copy = pcmk__str_table_dup(cmd->params);
 
-    action = resources_action_create(rsc->rsc_id, rsc->class, rsc->provider,
+    action = services__create_resource_action(rsc->rsc_id, rsc->class, rsc->provider,
                                      rsc->type,
                                      normalize_action_name(rsc, cmd->action),
                                      cmd->interval_ms, cmd->timeout,
                                      params_copy, cmd->service_flags);
 
-    if (!action) {
-        crm_err("Failed to create action, action:%s on resource %s", cmd->action, rsc->rsc_id);
-        cmd->lrmd_op_status = PCMK_LRM_OP_ERROR;
+    if (action == NULL) {
+        pcmk__set_result(&(cmd->result), PCMK_OCF_UNKNOWN_ERROR,
+                         PCMK_EXEC_ERROR, strerror(ENOMEM));
+        goto exec_done;
+    }
+
+    if (action->rc != PCMK_OCF_UNKNOWN) {
+        pcmk__set_result(&(cmd->result), action->rc, action->status,
+                         services__exit_reason(action));
+        services_action_free(action);
         goto exec_done;
     }
 
     action->cb_data = cmd;
 
-    /* 'cmd' may not be valid after this point if
-     * services_action_async() returned TRUE
-     *
-     * Upstart and systemd both synchronously determine monitor/status
-     * results and call action_complete (which may free 'cmd') if necessary.
-     */
     if (services_action_async(action, action_complete)) {
+        /* When services_action_async() returns TRUE, the callback might have
+         * been called -- in this case action_complete(), which might free cmd,
+         * so cmd cannot be used here.
+         */
         return TRUE;
     }
 
-    cmd->exec_rc = action->rc;
-    if(action->status != PCMK_LRM_OP_DONE) {
-        cmd->lrmd_op_status = action->status;
-    } else {
-        cmd->lrmd_op_status = PCMK_LRM_OP_ERROR;
-    }
+    pcmk__set_result(&(cmd->result), action->rc, action->status,
+                     services__exit_reason(action));
     services_action_free(action);
     action = NULL;
 
@@ -1454,18 +1410,18 @@ lrmd_rsc_dispatch(gpointer user_data)
 void
 free_rsc(gpointer data)
 {
-    GListPtr gIter = NULL;
+    GList *gIter = NULL;
     lrmd_rsc_t *rsc = data;
     int is_stonith = pcmk__str_eq(rsc->class, PCMK_RESOURCE_CLASS_STONITH,
                                   pcmk__str_casei);
 
     gIter = rsc->pending_ops;
     while (gIter != NULL) {
-        GListPtr next = gIter->next;
+        GList *next = gIter->next;
         lrmd_cmd_t *cmd = gIter->data;
 
         /* command was never executed */
-        cmd->lrmd_op_status = PCMK_LRM_OP_CANCELLED;
+        cmd->result.execution_status = PCMK_EXEC_CANCELLED;
         cmd_finalize(cmd, NULL);
 
         gIter = next;
@@ -1475,11 +1431,11 @@ free_rsc(gpointer data)
 
     gIter = rsc->recurring_ops;
     while (gIter != NULL) {
-        GListPtr next = gIter->next;
+        GList *next = gIter->next;
         lrmd_cmd_t *cmd = gIter->data;
 
         if (is_stonith) {
-            cmd->lrmd_op_status = PCMK_LRM_OP_CANCELLED;
+            cmd->result.execution_status = PCMK_EXEC_CANCELLED;
             /* If a stonith command is in-flight, just mark it as cancelled;
              * it is not safe to finalize/free the cmd until the stonith api
              * says it has either completed or timed out.
@@ -1526,7 +1482,7 @@ process_lrmd_signon(pcmk__client_t *client, xmlNode *request, int call_id,
     }
 
     if (crm_is_true(is_ipc_provider)) {
-#ifdef SUPPORT_REMOTE
+#ifdef PCMK__COMPILE_REMOTE
         if ((client->remote != NULL) && client->remote->tls_handshake_complete) {
             // This is a remote connection from a cluster node's controller
             ipc_proxy_add_provider(client);
@@ -1659,7 +1615,7 @@ process_lrmd_rsc_exec(pcmk__client_t *client, uint32_t id, xmlNode *request)
 static int
 cancel_op(const char *rsc_id, const char *action, guint interval_ms)
 {
-    GListPtr gIter = NULL;
+    GList *gIter = NULL;
     lrmd_rsc_t *rsc = g_hash_table_lookup(rsc_list, rsc_id);
 
     /* How to cancel an action.
@@ -1680,10 +1636,8 @@ cancel_op(const char *rsc_id, const char *action, guint interval_ms)
     for (gIter = rsc->pending_ops; gIter != NULL; gIter = gIter->next) {
         lrmd_cmd_t *cmd = gIter->data;
 
-        if (pcmk__str_eq(cmd->action, action, pcmk__str_casei)
-            && (cmd->interval_ms == interval_ms)) {
-
-            cmd->lrmd_op_status = PCMK_LRM_OP_CANCELLED;
+        if (action_matches(cmd, action, interval_ms)) {
+            cmd->result.execution_status = PCMK_EXEC_CANCELLED;
             cmd_finalize(cmd, rsc);
             return pcmk_ok;
         }
@@ -1695,10 +1649,8 @@ cancel_op(const char *rsc_id, const char *action, guint interval_ms)
         for (gIter = rsc->recurring_ops; gIter != NULL; gIter = gIter->next) {
             lrmd_cmd_t *cmd = gIter->data;
 
-            if (pcmk__str_eq(cmd->action, action, pcmk__str_casei)
-                && (cmd->interval_ms == interval_ms)) {
-
-                cmd->lrmd_op_status = PCMK_LRM_OP_CANCELLED;
+            if (action_matches(cmd, action, interval_ms)) {
+                cmd->result.execution_status = PCMK_EXEC_CANCELLED;
                 if (rsc->active != cmd) {
                     cmd_finalize(cmd, rsc);
                 }
@@ -1847,21 +1799,17 @@ process_lrmd_message(pcmk__client_t *client, uint32_t id, xmlNode *request)
     int do_notify = 0;
     xmlNode *reply = NULL;
 
-#if ENABLE_ACL
     /* Certain IPC commands may be done only by privileged users (i.e. root or
-     * hacluster) when ACLs are enabled, because they would otherwise provide a
-     * means of bypassing ACLs.
+     * hacluster), because they would otherwise provide a means of bypassing
+     * ACLs.
      */
     bool allowed = pcmk_is_set(client->flags, pcmk__client_privileged);
-#else
-    bool allowed = true;
-#endif
 
     crm_trace("Processing %s operation from %s", op, client->id);
     crm_element_value_int(request, F_LRMD_CALLID, &call_id);
 
     if (pcmk__str_eq(op, CRM_OP_IPC_FWD, pcmk__str_none)) {
-#ifdef SUPPORT_REMOTE
+#ifdef PCMK__COMPILE_REMOTE
         if (allowed) {
             ipc_proxy_forward_client(client, request);
         } else {
@@ -1943,7 +1891,8 @@ process_lrmd_message(pcmk__client_t *client, uint32_t id, xmlNode *request)
     } else {
         rc = -EOPNOTSUPP;
         do_reply = 1;
-        crm_err("Unknown IPC request '%s' from %s", op, client->name);
+        crm_err("Unknown IPC request '%s' from client %s",
+                op, pcmk__client_name(client));
     }
 
     if (rc == -EACCES) {
@@ -1963,8 +1912,8 @@ process_lrmd_message(pcmk__client_t *client, uint32_t id, xmlNode *request)
         send_rc = lrmd_server_send_reply(client, id, reply);
         free_xml(reply);
         if (send_rc != pcmk_rc_ok) {
-            crm_warn("Reply to client %s failed: %s " CRM_XS " %d",
-                     client->name, pcmk_rc_str(send_rc), send_rc);
+            crm_warn("Reply to client %s failed: %s " CRM_XS " rc=%d",
+                     pcmk__client_name(client), pcmk_rc_str(send_rc), send_rc);
         }
     }
 
