@@ -1,5 +1,5 @@
 /*
- * Copyright 2009-2019 the Pacemaker project contributors
+ * Copyright 2009-2020 the Pacemaker project contributors
  *
  * This source code is licensed under the GNU General Public License version 2
  * or later (GPLv2+) WITHOUT ANY WARRANTY.
@@ -26,6 +26,7 @@
 
 #include <crm/stonith-ng.h>
 #include <crm/fencing/internal.h>
+#include <crm/common/agents.h>
 #include <crm/common/xml.h>
 
 #include <crm/common/mainloop.h>
@@ -40,17 +41,16 @@ typedef void (*mainloop_test_iteration_cb) (int check_event);
 
 #define mainloop_test_done(pass) \
     if (pass) { \
-        crm_info("SUCCESS - %s", __FUNCTION__); \
+        crm_info("SUCCESS - %s", __func__); \
         mainloop_iter++;   \
         mainloop_set_trigger(trig);  \
     } else { \
-        crm_err("FAILURE = %s async_callback %d", __FUNCTION__, callback_rc); \
+        crm_err("FAILURE = %s async_callback %d", __func__, callback_rc); \
         crm_exit(CRM_EX_ERROR); \
     } \
     callback_rc = 0; \
 
 
-/* *INDENT-OFF* */
 enum test_modes {
     test_standard = 0,  // test using a specific developer environment
     test_passive,       // watch notifications only
@@ -58,21 +58,38 @@ enum test_modes {
     test_api_mainloop,  // sanity-test mainloop code with async responses
 };
 
-static struct crm_option long_options[] = {
-    {"verbose",     0, 0, 'V'},
-    {"version",     0, 0, '$'},
-    {"help",        0, 0, '?'},
-    {"passive",     0, 0, 'p'},
-    {"api_test",    0, 0, 't'},
-    {"mainloop_api_test",    0, 0, 'm'},
-
-    {0, 0, 0, 0}
+static pcmk__cli_option_t long_options[] = {
+    // long option, argument type, storage, short option, description, flags
+    {
+        "verbose", no_argument, NULL, 'V',
+        NULL, pcmk__option_default
+    },
+    {
+        "version", no_argument, NULL, '$',
+        NULL, pcmk__option_default
+    },
+    {
+        "help", no_argument, NULL, '?',
+        NULL, pcmk__option_default
+    },
+    {
+        "passive", no_argument, NULL, 'p',
+        NULL, pcmk__option_default
+    },
+    {
+        "api_test", no_argument, NULL, 't',
+        NULL, pcmk__option_default
+    },
+    {
+        "mainloop_api_test", no_argument, NULL, 'm',
+        NULL, pcmk__option_default
+    },
+    { 0, 0, 0, 0 }
 };
-/* *INDENT-ON* */
 
 static stonith_t *st = NULL;
 static struct pollfd pollfd;
-static int st_opts = st_opt_sync_call;
+static const int st_opts = st_opt_sync_call;
 static int expected_notifications = 0;
 static int verbose = 0;
 
@@ -168,7 +185,8 @@ run_fence_failure_test(void)
 {
     stonith_key_value_t *params = NULL;
 
-    params = stonith_key_value_add(params, "pcmk_host_map", "false_1_node1=1,2 false_1_node2=3,4");
+    params = stonith_key_value_add(params, PCMK_STONITH_HOST_MAP,
+                                   "false_1_node1=1,2 false_1_node2=3,4");
     params = stonith_key_value_add(params, "mode", "fail");
 
     single_test(st->
@@ -192,7 +210,8 @@ run_fence_failure_rollover_test(void)
 {
     stonith_key_value_t *params = NULL;
 
-    params = stonith_key_value_add(params, "pcmk_host_map", "false_1_node1=1,2 false_1_node2=3,4");
+    params = stonith_key_value_add(params, PCMK_STONITH_HOST_MAP,
+                                   "false_1_node1=1,2 false_1_node2=3,4");
     params = stonith_key_value_add(params, "mode", "fail");
 
     single_test(st->
@@ -200,7 +219,8 @@ run_fence_failure_rollover_test(void)
                 "Register device1 for rollover test", 1, 0);
     stonith_key_value_freeall(params, 1, 1);
     params = NULL;
-    params = stonith_key_value_add(params, "pcmk_host_map", "false_1_node1=1,2 false_1_node2=3,4");
+    params = stonith_key_value_add(params, PCMK_STONITH_HOST_MAP,
+                                   "false_1_node1=1,2 false_1_node2=3,4");
     params = stonith_key_value_add(params, "mode", "pass");
 
     single_test(st->
@@ -228,7 +248,8 @@ run_standard_test(void)
 {
     stonith_key_value_t *params = NULL;
 
-    params = stonith_key_value_add(params, "pcmk_host_map", "false_1_node1=1,2 false_1_node2=3,4");
+    params = stonith_key_value_add(params, PCMK_STONITH_HOST_MAP,
+                                   "false_1_node1=1,2 false_1_node2=3,4");
     params = stonith_key_value_add(params, "mode", "pass");
     params = stonith_key_value_add(params, "mock_dynamic_hosts", "false_1_node1 false_1_node2");
 
@@ -304,7 +325,8 @@ standard_dev_test(void)
         crm_exit(CRM_EX_DISCONNECT);
     }
 
-    params = stonith_key_value_add(params, "pcmk_host_map", "some-host=pcmk-7 true_1_node1=3,4");
+    params = stonith_key_value_add(params, PCMK_STONITH_HOST_MAP,
+                                   "some-host=pcmk-7 true_1_node1=3,4");
 
     rc = st->cmds->register_device(st, st_opts, "test-id", "stonith-ng", "fence_xvm", params);
     crm_debug("Register: %d", rc);
@@ -482,19 +504,22 @@ test_register_async_devices(int check_event)
     char buf[16] = { 0, };
     stonith_key_value_t *params = NULL;
 
-    params = stonith_key_value_add(params, "pcmk_host_map", "false_1_node1=1,2");
+    params = stonith_key_value_add(params, PCMK_STONITH_HOST_MAP,
+                                   "false_1_node1=1,2");
     params = stonith_key_value_add(params, "mode", "fail");
     st->cmds->register_device(st, st_opts, "false_1", "stonith-ng", "fence_dummy", params);
     stonith_key_value_freeall(params, 1, 1);
 
     params = NULL;
-    params = stonith_key_value_add(params, "pcmk_host_map", "true_1_node1=1,2");
+    params = stonith_key_value_add(params, PCMK_STONITH_HOST_MAP,
+                                   "true_1_node1=1,2");
     params = stonith_key_value_add(params, "mode", "pass");
     st->cmds->register_device(st, st_opts, "true_1", "stonith-ng", "fence_dummy", params);
     stonith_key_value_freeall(params, 1, 1);
 
     params = NULL;
-    params = stonith_key_value_add(params, "pcmk_host_map", "custom_timeout_node1=1,2");
+    params = stonith_key_value_add(params, PCMK_STONITH_HOST_MAP,
+                                   "custom_timeout_node1=1,2");
     params = stonith_key_value_add(params, "mode", "fail");
     params = stonith_key_value_add(params, "delay", "1000");
     snprintf(buf, sizeof(buf) - 1, "%d", MAINLOOP_DEFAULT_TIMEOUT + CUSTOM_TIMEOUT_ADDITION);
@@ -586,13 +611,13 @@ main(int argc, char **argv)
 
     enum test_modes mode = test_standard;
 
-    crm_log_cli_init("cts-fence-helper");
-    crm_set_options(NULL, "mode [options]", long_options,
-                    "Provides a summary of cluster's current state."
-                    "\n\nOutputs varying levels of detail in a number of different formats.\n");
+    pcmk__cli_init_logging("cts-fence-helper", 0);
+    pcmk__set_cli_options(NULL, "<mode> [options]", long_options,
+                          "inject commands into the Pacemaker fencer, "
+                          "and watch for events");
 
     while (1) {
-        flag = crm_get_option(argc, argv, &option_index);
+        flag = pcmk__next_cli_option(argc, argv, &option_index, NULL);
         if (flag == -1) {
             break;
         }
@@ -603,7 +628,7 @@ main(int argc, char **argv)
                 break;
             case '$':
             case '?':
-                crm_help(flag, CRM_EX_OK);
+                pcmk__cli_help(flag, CRM_EX_OK);
                 break;
             case 'p':
                 mode = test_passive;
@@ -628,7 +653,7 @@ main(int argc, char **argv)
     }
 
     if (argerr) {
-        crm_help('?', CRM_EX_USAGE);
+        pcmk__cli_help('?', CRM_EX_USAGE);
     }
 
     st = stonith_api_new();
