@@ -1,5 +1,5 @@
 /*
- * Copyright 2009-2020 the Pacemaker project contributors
+ * Copyright 2009-2021 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -149,7 +149,11 @@ op_time_sort(const void *a_voidp, const void *b_voidp)
     } else if (b_pending) {
         return 1;
     } else if ((*b)->completed == (*a)->completed) {
-        return 0;
+        if ((*b)->completed_nsec > (*a)->completed_nsec) {
+            return 1;
+        } else if ((*b)->completed_nsec == (*a)->completed_nsec) {
+            return 0;
+        }
     } else if ((*b)->completed > (*a)->completed) {
         return 1;
     }
@@ -230,9 +234,10 @@ stonith_xml_history_to_list(xmlNode *history)
         char *id = crm_element_value_copy(xml_op, F_STONITH_REMOTE_OP_ID);
         int state;
         long long completed;
+        long long completed_nsec = 0L;
 
         if (!id) {
-            crm_warn("History to convert to hashtable has no id in entry");
+            crm_warn("Malformed fencing history received from peer");
             continue;
         }
 
@@ -248,6 +253,8 @@ stonith_xml_history_to_list(xmlNode *history)
         op->client_name = crm_element_value_copy(xml_op, F_STONITH_CLIENTNAME);
         crm_element_value_ll(xml_op, F_STONITH_DATE, &completed);
         op->completed = (time_t) completed;
+        crm_element_value_ll(xml_op, F_STONITH_DATE_NSEC, &completed_nsec);
+        op->completed_nsec = completed_nsec;
         crm_element_value_int(xml_op, F_STONITH_STATE, &state);
         op->state = (enum op_state) state;
 
@@ -261,7 +268,7 @@ stonith_xml_history_to_list(xmlNode *history)
 /*!
  * \internal
  * \brief Craft xml difference between local fence-history and a history
- *        coming from remote
+ *        coming from remote, and merge the remote history into the local
  *
  * \param[in] remote_history    Fence-history as hash-table (may be NULL)
  * \param[in] add_id            If crafting the answer for an API
@@ -271,26 +278,63 @@ stonith_xml_history_to_list(xmlNode *history)
  * \return The fence-history as xml
  */
 static xmlNode *
-stonith_local_history_diff(GHashTable *remote_history,
+stonith_local_history_diff_and_merge(GHashTable *remote_history,
                            gboolean add_id,
                            const char *target)
 {
     xmlNode *history = NULL;
+    GHashTableIter iter;
+    remote_fencing_op_t *op = NULL;
+    gboolean updated = FALSE;
     int cnt = 0;
 
     if (stonith_remote_op_list) {
-            GHashTableIter iter;
-            remote_fencing_op_t *op = NULL;
+            char *id = NULL;
 
             history = create_xml_node(NULL, F_STONITH_HISTORY_LIST);
 
             g_hash_table_iter_init(&iter, stonith_remote_op_list);
-            while (g_hash_table_iter_next(&iter, NULL, (void **)&op)) {
+            while (g_hash_table_iter_next(&iter, (void **)&id, (void **)&op)) {
                 xmlNode *entry = NULL;
 
-                if (remote_history &&
-                    g_hash_table_lookup(remote_history, op->id)) {
-                    continue; /* skip entries broadcasted already */
+                if (remote_history) {
+                    remote_fencing_op_t *remote_op =
+                        g_hash_table_lookup(remote_history, op->id);
+
+                    if (remote_op) {
+                        if (stonith__op_state_pending(op->state)
+                            && !stonith__op_state_pending(remote_op->state)) {
+
+                            crm_debug("Updating outdated pending operation %.8s "
+                                      "(state=%s) according to the one (state=%s) from "
+                                      "remote peer history",
+                                      op->id, stonith_op_state_str(op->state),
+                                      stonith_op_state_str(remote_op->state));
+
+                            g_hash_table_steal(remote_history, op->id);
+                            op->id = remote_op->id;
+                            remote_op->id = id;
+                            g_hash_table_iter_replace(&iter, remote_op);
+
+                            updated = TRUE;
+                            continue; /* skip outdated entries */
+
+                        } else if (!stonith__op_state_pending(op->state)
+                                   && stonith__op_state_pending(remote_op->state)) {
+
+                            crm_debug("Broadcasting operation %.8s (state=%s) to "
+                                      "update the outdated pending one "
+                                      "(state=%s) in remote peer history",
+                                      op->id, stonith_op_state_str(op->state),
+                                      stonith_op_state_str(remote_op->state));
+
+                            g_hash_table_remove(remote_history, op->id);
+
+                        } else {
+                            g_hash_table_remove(remote_history, op->id);
+                            continue; /* skip entries broadcasted already */
+                        }
+                    }
                 }
 
                 if (!pcmk__str_eq(target, op->target, pcmk__str_null_matches)) {
@@ -309,10 +353,52 @@ stonith_local_history_diff(GHashTable *remote_history,
                 crm_xml_add(entry, F_STONITH_DELEGATE, op->delegate);
                 crm_xml_add(entry, F_STONITH_CLIENTNAME, op->client_name);
                 crm_xml_add_ll(entry, F_STONITH_DATE, op->completed);
+                crm_xml_add_ll(entry, F_STONITH_DATE_NSEC, op->completed_nsec);
                 crm_xml_add_int(entry, F_STONITH_STATE, op->state);
             }
     }
 
+    if (remote_history) {
+        init_stonith_remote_op_hash_table(&stonith_remote_op_list);
+
+        updated |= g_hash_table_size(remote_history);
+
+        g_hash_table_iter_init(&iter, remote_history);
+        while (g_hash_table_iter_next(&iter, NULL, (void **)&op)) {
+
+            if (stonith__op_state_pending(op->state) &&
+                pcmk__str_eq(op->originator, stonith_our_uname, pcmk__str_casei)) {
+                crm_warn("Failing pending operation %.8s originated by us but "
+                         "known only from peer history", op->id);
+                op->state = st_failed;
+                set_fencing_completed(op);
+                /* use -EHOSTUNREACH to not introduce a new return-code that might
+                   trigger unexpected results at other places and to prevent
+                   remote_op_done from setting the delegate if not present
+                */
+                stonith_bcast_result_to_peers(op, -EHOSTUNREACH, FALSE);
+            }
+
+            g_hash_table_iter_steal(&iter);
+            g_hash_table_replace(stonith_remote_op_list, op->id, op);
+            /* we could trim the history here but if we bail
+             * out after trim we might miss more recent entries
+             * of those that might still be in the list
+             * if we don't bail out trimming once is more
+             * efficient and memory overhead is minimal as
+             * we are just moving pointers from one hash to
+             * another
+             */
+        }
+
+        g_hash_table_destroy(remote_history); /* remove what is left */
+    }
+
+    if (updated) {
+        stonith_fence_history_trim();
+        do_stonith_notify(0, T_STONITH_NOTIFY_HISTORY, 0, NULL);
+    }
+
     if (cnt == 0) {
         free_xml(history);
         return NULL;
@@ -323,64 +409,18 @@ stonith_local_history_diff(GHashTable *remote_history,
 
 /*!
  * \internal
- * \brief Merge fence-history coming from remote into local history
+ * \brief Craft xml from the local fence-history
  *
- * \param[in] history   Hash-table holding remote history to be merged in
+ * \param[in] add_id            If crafting the answer for an API
+ *                              history-request there is no need for the id
+ * \param[in] target            Optionally limit to certain fence-target
+ *
+ * \return The fence-history as xml
  */
-static void
-stonith_merge_in_history_list(GHashTable *history)
+static xmlNode *
+stonith_local_history(gboolean add_id, const char *target)
 {
-    GHashTableIter iter;
-    remote_fencing_op_t *op = NULL;
-    gboolean updated = FALSE;
-
-    if (!history) {
-        return;
-    }
-
-    init_stonith_remote_op_hash_table(&stonith_remote_op_list);
-
-    g_hash_table_iter_init(&iter, history);
-    while (g_hash_table_iter_next(&iter, NULL, (void **)&op)) {
-        remote_fencing_op_t *stored_op =
-            g_hash_table_lookup(stonith_remote_op_list, op->id);
-
-        if (stored_op) {
-            continue; // Skip existent (@TODO state-merging might be desirable)
-        }
-
-        updated = TRUE;
-        g_hash_table_iter_steal(&iter);
-
-        if ((op->state != st_failed) &&
-            (op->state != st_done) &&
-            pcmk__str_eq(op->originator, stonith_our_uname, pcmk__str_casei)) {
-            crm_warn("received pending action we are supposed to be the "
-                     "owner but it's not in our records -> fail it");
-            op->state = st_failed;
-            op->completed = time(NULL);
-            /* use -EHOSTUNREACH to not introduce a new return-code that might
-               trigger unexpected results at other places and to prevent
-               remote_op_done from setting the delegate if not present
-             */
-            stonith_bcast_result_to_peers(op, -EHOSTUNREACH, FALSE);
-        }
-
-        g_hash_table_insert(stonith_remote_op_list, op->id, op);
-        /* we could trim the history here but if we bail
-         * out after trim we might miss more recent entries
-         * of those that might still be in the list
-         * if we don't bail out trimming once is more
-         * efficient and memory overhead is minimal as
-         * we are just moving pointers from one hash to
-         * another
-         */
-    }
-    stonith_fence_history_trim();
-    if (updated) {
-        do_stonith_notify(0, T_STONITH_NOTIFY_HISTORY, 0, NULL);
-    }
-    g_hash_table_destroy(history); /* remove what is left */
+    return stonith_local_history_diff_and_merge(NULL, add_id, target);
 }
 
 /*!
@@ -408,9 +448,11 @@ stonith_fence_history(xmlNode *msg, xmlNode **output,
     if (dev) {
         target = crm_element_value(dev, F_STONITH_TARGET);
         if (target && (options & st_opt_cs_nodeid)) {
-            int nodeid = crm_atoi(target, NULL);
-            crm_node_t *node = crm_find_known_peer_full(nodeid, NULL, CRM_GET_PEER_ANY);
+            int nodeid;
+            crm_node_t *node;
 
+            pcmk__scan_min_int(target, &nodeid, 0);
+            node = pcmk__search_known_node_cache(nodeid, NULL, CRM_GET_PEER_ANY);
             if (node) {
                 target = node->uname;
             }
@@ -436,7 +478,7 @@ stonith_fence_history(xmlNode *msg, xmlNode **output,
             * so that every node can merge and broadcast
             * what it has on top
             */
-            out_history = stonith_local_history_diff(NULL, TRUE, NULL);
+            out_history = stonith_local_history(TRUE, NULL);
             crm_trace("Broadcasting history to peers");
             stonith_send_broadcast_history(out_history,
                                         st_opt_broadcast | st_opt_discard_reply,
@@ -461,7 +503,7 @@ stonith_fence_history(xmlNode *msg, xmlNode **output,
                 !crm_is_true(crm_element_value(history,
                                                F_STONITH_DIFFERENTIAL))) {
                 out_history =
-                    stonith_local_history_diff(received_history, TRUE, NULL);
+                    stonith_local_history_diff_and_merge(received_history, TRUE, NULL);
                 if (out_history) {
                     crm_trace("Broadcasting history-diff to peers");
                     crm_xml_add(out_history, F_STONITH_DIFFERENTIAL,
@@ -473,7 +515,6 @@ stonith_fence_history(xmlNode *msg, xmlNode **output,
                     crm_trace("History-diff is empty - skip broadcast");
                 }
             }
-            stonith_merge_in_history_list(received_history);
         } else {
             crm_trace("Skipping history-query-broadcast (%s%s)"
                       " we sent ourselves",
@@ -484,7 +525,7 @@ stonith_fence_history(xmlNode *msg, xmlNode **output,
         /* plain history request */
         crm_trace("Looking for operations on %s in %p", target,
                   stonith_remote_op_list);
-        *output = stonith_local_history_diff(NULL, FALSE, target);
+        *output = stonith_local_history(FALSE, target);
     }
     free_xml(out_history);
     return rc;
