@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2020 the Pacemaker project contributors
+ * Copyright 2010-2021 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -29,6 +29,7 @@
 #include <crm/common/ipc_internal.h>  /* PCMK__SPECIAL_PID* */
 
 static corosync_cfg_handle_t cfg_handle = 0;
+static mainloop_timer_t *reconnect_timer = NULL;
 
 /* =::=::=::= CFG - Shutdown stuff =::=::=::= */
 
@@ -60,24 +61,64 @@ pcmk_cfg_dispatch(gpointer user_data)
 }
 
 static void
-cfg_connection_destroy(gpointer user_data)
+close_cfg(void)
 {
-    crm_err("Lost connection to cluster layer");
-    corosync_cfg_finalize(cfg_handle);
-    cfg_handle = 0;
-    pcmk_shutdown(SIGTERM);
-}
-
-gboolean
-cluster_disconnect_cfg(void)
-{
-    if (cfg_handle) {
+    if (cfg_handle != 0) {
+#ifdef HAVE_COROSYNC_CFG_TRACKSTART
+        /* Ideally, we would call corosync_cfg_trackstop(cfg_handle) here, but a
+         * bug in corosync 3.1.1 and 3.1.2 makes it hang forever. Thankfully,
+         * it's not necessary since we exit immediately after this.
+         */
+#endif
         corosync_cfg_finalize(cfg_handle);
         cfg_handle = 0;
     }
+}
 
-    pcmk_shutdown(SIGTERM);
-    return TRUE;
+static gboolean
+cluster_reconnect_cb(gpointer data)
+{
+    if (cluster_connect_cfg()) {
+        mainloop_timer_del(reconnect_timer);
+        reconnect_timer = NULL;
+        crm_notice("Cluster reconnect succeeded");
+        mcp_read_config();
+        restart_cluster_subdaemons();
+        return G_SOURCE_REMOVE;
+    } else {
+        crm_info("Cluster reconnect failed"
+                 "(connection will be reattempted once per second)");
+    }
+    /*
+     * In theory this will continue forever. In practice the CIB connection from
+     * attrd will timeout and shut down Pacemaker when it gets bored.
+     */
+    return G_SOURCE_CONTINUE;
+}
+
+
+static void
+cfg_connection_destroy(gpointer user_data)
+{
+    crm_warn("Lost connection to cluster layer "
+             "(connection will be reattempted once per second)");
+    corosync_cfg_finalize(cfg_handle);
+    cfg_handle = 0;
+    reconnect_timer = mainloop_timer_add("corosync reconnect", 1000, TRUE, cluster_reconnect_cb, NULL);
+    mainloop_timer_start(reconnect_timer);
+}
+
+void
+cluster_disconnect_cfg(void)
+{
+    close_cfg();
+    if (reconnect_timer != NULL) {
+        /* The mainloop should be gone by this point, so this isn't necessary,
+         * but cleaning up memory should make valgrind happier.
+         */
+        mainloop_timer_del(reconnect_timer);
+        reconnect_timer = NULL;
+    }
 }
 
 #define cs_repeat(counter, max, code) do {		\
@@ -144,6 +185,15 @@ cluster_connect_cfg(void)
     }
     crm_debug("Corosync reports local node ID is %lu", (unsigned long) nodeid);
 
+#ifdef HAVE_COROSYNC_CFG_TRACKSTART
+    rc = corosync_cfg_trackstart(cfg_handle, 0);
+    if (rc != CS_OK) {
+        crm_crit("Could not enable Corosync CFG shutdown tracker: %s " CRM_XS " rc=%d",
+                 cs_strerror(rc), rc);
+        goto bail;
+    }
+#endif
+
     mainloop_add_fd("corosync-cfg", G_PRIORITY_DEFAULT, fd, &cfg_handle, &cfg_fd_callbacks);
     return TRUE;
 
@@ -165,14 +215,32 @@ pcmkd_shutdown_corosync(void)
     rc = corosync_cfg_try_shutdown(cfg_handle,
                                     COROSYNC_CFG_SHUTDOWN_FLAG_IMMEDIATE);
     if (rc == CS_OK) {
-        corosync_cfg_finalize(cfg_handle);
-        cfg_handle = 0;
+        close_cfg();
     } else {
         crm_warn("Corosync shutdown failed: %s " CRM_XS " rc=%d",
                  cs_strerror(rc), rc);
     }
 }
 
+bool
+pcmkd_corosync_connected(void)
+{
+    cpg_handle_t local_handle = 0;
+    cpg_model_v1_data_t cpg_model_info = {CPG_MODEL_V1, NULL, NULL, NULL, 0};
+    int fd = -1;
+
+    if (cpg_model_initialize(&local_handle, CPG_MODEL_V1, (cpg_model_data_t *) &cpg_model_info, NULL) != CS_OK) {
+        return false;
+    }
+
+    if (cpg_fd_get(local_handle, &fd) != CS_OK) {
+        return false;
+    }
+
+    cpg_finalize(local_handle);
+
+    return true;
+}
 
 /* =::=::=::= Configuration =::=::=::= */
 static int
@@ -210,7 +278,7 @@ mcp_read_config(void)
 
     // There can be only one possibility
     do {
-        rc = cmap_initialize(&local_handle);
+        rc = pcmk__init_cmap(&local_handle);
         if (rc != CS_OK) {
             retries++;
             crm_info("Could not connect to Corosync CMAP: %s (retrying in %ds) "
@@ -255,30 +323,30 @@ mcp_read_config(void)
 
     stack = get_cluster_type();
     if (stack != pcmk_cluster_corosync) {
-        crm_crit("Expected corosync stack but detected %s " CRM_XS " stack=%d",
-                 name_for_cluster_type(stack), stack);
+        crm_crit("Expected Corosync cluster layer but detected %s "
+                 CRM_XS " stack=%d", name_for_cluster_type(stack), stack);
         return FALSE;
     }
 
     crm_info("Reading configuration for %s stack",
              name_for_cluster_type(stack));
-    pcmk__set_env_option("cluster_type", "corosync");
-    pcmk__set_env_option("quorum_type", "corosync");
+    pcmk__set_env_option(PCMK__ENV_CLUSTER_TYPE, "corosync");
+    pcmk__set_env_option(PCMK__ENV_QUORUM_TYPE, "corosync");
 
     // If debug logging is not configured, check whether corosync has it
-    if (pcmk__env_option("debug") == NULL) {
+    if (pcmk__env_option(PCMK__ENV_DEBUG) == NULL) {
         char *debug_enabled = NULL;
 
         get_config_opt(config, local_handle, "logging.debug", &debug_enabled, "off");
 
         if (crm_is_true(debug_enabled)) {
-            pcmk__set_env_option("debug", "1");
+            pcmk__set_env_option(PCMK__ENV_DEBUG, "1");
             if (get_crm_log_level() < LOG_DEBUG) {
                 set_crm_log_level(LOG_DEBUG);
             }
 
         } else {
-            pcmk__set_env_option("debug", "0");
+            pcmk__set_env_option(PCMK__ENV_DEBUG, "0");
         }
 
         free(debug_enabled);
@@ -296,7 +364,7 @@ mcp_read_config(void)
             rc = cmap_set_uint8(local_handle, key, 1);
             if (rc != CS_OK) {
                 crm_warn("Could not authorize group with Corosync: %s " CRM_XS
-                         " group=%u rc=%d", ais_error2text(rc), gid, rc);
+                         " group=%u rc=%d", pcmk__cs_err_str(rc), gid, rc);
             }
         }
     }
