@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2020 the Pacemaker project contributors
+ * Copyright 2004-2021 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -67,12 +67,19 @@ crm_extended_logging(int t, int e)
 extern unsigned int crm_log_level;
 extern unsigned int crm_trace_nonlog;
 
-/* These are set when a configuration issue is found, and turn on extra messages
- * at the end of processing. They are set via wrapper functions and do not need
- * to be set directly.
+/*! \deprecated Pacemaker library functions set this when a configuration
+ *              error is found, which turns on extra messages at the end of
+ *              processing. It should not be used directly and will be removed
+ *              from the public C API in a future release.
  */
-extern bool pcmk__config_error;
-extern bool pcmk__config_warning;
+extern gboolean crm_config_error;
+
+/*! \deprecated Pacemaker library functions set this when a configuration
+ *              warning is found, which turns on extra messages at the end of
+ *              processing. It should not be used directly and will be removed
+ *              from the public C API in a future release.
+ */
+extern gboolean crm_config_warning;
 
 enum xml_log_options
 {
@@ -98,8 +105,13 @@ void crm_update_callsites(void);
 
 void crm_log_deinit(void);
 
-gboolean crm_log_cli_init(const char *entity);
-
+/*!
+ * \brief Initializes the logging system and defaults to the least verbose output level
+ *
+ * \param[in] entity  If not NULL, will be used as the identity for logging purposes
+ * \param[in] argc    The number of command line parameters
+ * \param[in] argv    The command line parameter values
+ */
 void crm_log_preinit(const char *entity, int argc, char **argv);
 gboolean crm_log_init(const char *entity, uint8_t level, gboolean daemon,
                       gboolean to_stderr, int argc, char **argv, gboolean quiet);
@@ -112,8 +124,6 @@ void crm_log_output_fn(const char *file, const char *function, int line, int lev
 #define crm_log_output(level, prefix, output)   \
     crm_log_output_fn(__FILE__, __func__, __LINE__, level, prefix, output)
 
-gboolean crm_add_logfile(const char *filename);
-
 void crm_bump_log_level(int argc, char **argv);
 
 void crm_enable_stderr(int enable);
@@ -197,7 +207,7 @@ unsigned int get_crm_log_level(void);
     } while (0)
 
 #  define CRM_LOG_ASSERT(expr) do {					\
-        if(__unlikely((expr) == FALSE)) {				\
+        if (!(expr)) {                                                  \
             static struct qb_log_callsite *core_cs = NULL;              \
             if(core_cs == NULL) {                                       \
                 core_cs = qb_log_callsite_get(__func__, __FILE__,       \
@@ -213,7 +223,7 @@ unsigned int get_crm_log_level(void);
  * macro's do-while loop
  */
 #  define CRM_CHECK(expr, failure_action) do {				            \
-	    if (__unlikely((expr) == FALSE)) {				                \
+        if (!(expr)) {                                                  \
             static struct qb_log_callsite *core_cs = NULL;              \
             if (core_cs == NULL) {                                      \
                 core_cs = qb_log_callsite_get(__func__, __FILE__,       \
@@ -372,34 +382,9 @@ unsigned int get_crm_log_level(void);
 
 #  define crm_str(x)    (const char*)(x?x:"<null>")
 
-#ifndef PCMK__NO_COMPAT
-
-/* Everything here is deprecated and kept only for public API backward
- * compatibility. It will be moved to compatibility.h in a future release.
- */
-
-/*!
- * \brief Log a message using constant priority
- *
- * \param[in] level     Priority at which to log the message
- * \param[in] fmt       printf-style format string literal for message
- * \param[in] args      Any arguments needed by format string
- *
- * \deprecated Use one of the other logging functions instead
- * \note This is a macro, and \p level may be evaluated more than once.
- *       This does nothing when level is LOG_STDOUT.
- */
-#  define do_crm_log_always(level, fmt, args...) do {                       \
-        switch (level) {                                                    \
-            case LOG_STDOUT: case LOG_NEVER:                                \
-                break;                                                      \
-            default:                                                        \
-                qb_log((level), fmt , ##args);                              \
-                break;                                                      \
-        }                                                                   \
-    } while (0)
-
-#endif // PCMK__NO_COMPAT
+#if !defined(PCMK_ALLOW_DEPRECATED) || (PCMK_ALLOW_DEPRECATED == 1)
+#include <crm/common/logging_compat.h>
+#endif
 
 #ifdef __cplusplus
 }
