@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2019 the Pacemaker project contributors
+ * Copyright 2004-2021 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -208,7 +208,7 @@ get_resource_type(const char *name)
         return pe_group;
 
     } else if (safe_str_eq(name, XML_CIB_TAG_INCARNATION)
-               || safe_str_eq(name, XML_CIB_TAG_MASTER)) {
+                || safe_str_eq(name, PCMK_XE_PROMOTABLE_LEGACY)) {
         return pe_clone;
 
     } else if (safe_str_eq(name, XML_CIB_TAG_CONTAINER)) {
