@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2020 the Pacemaker project contributors
+ * Copyright 2004-2021 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -88,30 +88,42 @@ enum rsc_start_requirement {
     rsc_req_stonith             /* Enforced by native_start_constraints() */
 };
 
+//! Possible roles that a resource can be in
 enum rsc_role_e {
-    RSC_ROLE_UNKNOWN,
-    RSC_ROLE_STOPPED,
-    RSC_ROLE_STARTED,
-    RSC_ROLE_SLAVE,
-    RSC_ROLE_MASTER,
+    RSC_ROLE_UNKNOWN    = 0,
+    RSC_ROLE_STOPPED    = 1,
+    RSC_ROLE_STARTED    = 2,
+    RSC_ROLE_UNPROMOTED = 3,
+    RSC_ROLE_PROMOTED   = 4,
+
+#if !defined(PCMK_ALLOW_DEPRECATED) || (PCMK_ALLOW_DEPRECATED == 1)
+    //! \deprecated Use RSC_ROLE_UNPROMOTED instead
+    RSC_ROLE_SLAVE      = RSC_ROLE_UNPROMOTED,
+
+    //! \deprecated Use RSC_ROLE_PROMOTED instead
+    RSC_ROLE_MASTER     = RSC_ROLE_PROMOTED,
+#endif
 };
 
-#  define RSC_ROLE_MAX  RSC_ROLE_MASTER+1
+#  define RSC_ROLE_MAX  (RSC_ROLE_PROMOTED + 1)
 
 #  define RSC_ROLE_UNKNOWN_S "Unknown"
 #  define RSC_ROLE_STOPPED_S "Stopped"
 #  define RSC_ROLE_STARTED_S "Started"
-#  define RSC_ROLE_SLAVE_S   "Slave"
-#  define RSC_ROLE_MASTER_S  "Master"
+#  define RSC_ROLE_UNPROMOTED_S         "Unpromoted"
+#  define RSC_ROLE_PROMOTED_S           "Promoted"
+#  define RSC_ROLE_UNPROMOTED_LEGACY_S  "Slave"
+#  define RSC_ROLE_PROMOTED_LEGACY_S    "Master"
 
+//! Deprecated
 enum pe_print_options {
     pe_print_log            = (1 << 0),
     pe_print_html           = (1 << 1),
     pe_print_ncurses        = (1 << 2),
     pe_print_printf         = (1 << 3),
-    pe_print_dev            = (1 << 4),  // Debugging (@COMPAT probably not useful)
-    pe_print_details        = (1 << 5),
-    pe_print_max_details    = (1 << 6),
+    pe_print_dev            = (1 << 4),  //! Ignored
+    pe_print_details        = (1 << 5),  //! Ignored
+    pe_print_max_details    = (1 << 6),  //! Ignored
     pe_print_rsconly        = (1 << 7),
     pe_print_ops            = (1 << 8),
     pe_print_suppres_nl     = (1 << 9),
@@ -178,6 +190,10 @@ typedef struct pe_rule_eval_data {
     pe_op_eval_data_t *op_data;
 } pe_rule_eval_data_t;
 
+#if !defined(PCMK_ALLOW_DEPRECATED) || (PCMK_ALLOW_DEPRECATED == 1)
+#include <crm/pengine/common_compat.h>
+#endif
+
 #ifdef __cplusplus
 }
 #endif
