@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2020 the Pacemaker project contributors
+ * Copyright 2004-2021 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -20,6 +20,12 @@ extern "C" {
  * \ingroup fencing
  */
 
+/* IMPORTANT: DLM source code includes this file directly, without having access
+ * to other Pacemaker headers on its include path, so this file should *not*
+ * include any other Pacemaker headers. (DLM might be updated to avoid the
+ * issue, but we should still follow this guideline for a long time after.)
+ */
+
 #  include <dlfcn.h>
 #  include <errno.h>
 #  include <stdbool.h>  // bool
@@ -104,6 +110,7 @@ typedef struct stonith_history_s {
     int state;
     time_t completed;
     struct stonith_history_s *next;
+    long completed_nsec;
 } stonith_history_t;
 
 typedef struct stonith_s stonith_t;
@@ -126,13 +133,17 @@ typedef struct stonith_event_s
     /*! The name of the client that initiated the action. */
     char *client_origin;
 
+    //! \internal This field should be treated as internal to Pacemaker
+    void *opaque;
 } stonith_event_t;
 
-typedef struct stonith_callback_data_s
-{
+typedef struct stonith_callback_data_s {
     int rc;
     int call_id;
     void *userdata;
+
+    //! \internal This field should be treated as internal to Pacemaker
+    void *opaque;
 } stonith_callback_data_t;
 
 typedef struct stonith_api_operations_s
@@ -145,16 +156,14 @@ typedef struct stonith_api_operations_s
     /*!
      * \brief Connect to the local stonith daemon.
      *
-     * \retval 0, success
-     * \retval negative error code on failure
+     * \return Legacy Pacemaker return code
      */
     int (*connect) (stonith_t *st, const char *name, int *stonith_fd);
 
     /*!
      * \brief Disconnect from the local stonith daemon.
      *
-     * \retval 0, success
-     * \retval negative error code on failure
+     * \return Legacy Pacemaker return code
      */
     int (*disconnect)(stonith_t *st);
 
@@ -163,8 +172,7 @@ typedef struct stonith_api_operations_s
      *
      * \note Synchronous, guaranteed to occur in daemon before function returns.
      *
-     * \retval 0, success
-     * \retval negative error code on failure
+     * \return Legacy Pacemaker return code
      */
     int (*remove_device)(
         stonith_t *st, int options, const char *name);
@@ -174,8 +182,7 @@ typedef struct stonith_api_operations_s
      *
      * \note Synchronous, guaranteed to occur in daemon before function returns.
      *
-     * \retval 0, success
-     * \retval negative error code on failure
+     * \return Legacy Pacemaker return code
      */
     int (*register_device)(
         stonith_t *st, int options, const char *id,
@@ -184,8 +191,7 @@ typedef struct stonith_api_operations_s
     /*!
      * \brief Remove a fencing level for a specific node.
      *
-     * \retval 0, success
-     * \retval negative error code on failure
+     * \return Legacy Pacemaker return code
      */
     int (*remove_level)(
         stonith_t *st, int options, const char *node, int level);
@@ -194,8 +200,7 @@ typedef struct stonith_api_operations_s
      * \brief Register a fencing level containing the fencing devices to be used
      *        at that level for a specific node.
      *
-     * \retval 0, success
-     * \retval negative error code on failure
+     * \return Legacy Pacemaker return code
      */
     int (*register_level)(
         stonith_t *st, int options, const char *node, int level, stonith_key_value_t *device_list);
@@ -205,8 +210,7 @@ typedef struct stonith_api_operations_s
      *
      * \note Value is returned in output.  Output must be freed when set.
      *
-     * \retval 0 success
-     * \retval negative error code on failure
+     * \return Legacy Pacemaker return code
      */
     int (*metadata)(stonith_t *st, int options,
             const char *device, const char *provider, char **output, int timeout);
@@ -218,8 +222,7 @@ typedef struct stonith_api_operations_s
      * \note list must be freed using stonith_key_value_freeall()
      * \note call_options parameter is not used, it is reserved for future use.
      *
-     * \retval num items in list on success
-     * \retval negative error code on failure
+     * \return Number of items in list on success, or negative errno otherwise
      */
     int (*list_agents)(stonith_t *stonith, int call_options, const char *provider,
             stonith_key_value_t **devices, int timeout);
@@ -227,24 +230,21 @@ typedef struct stonith_api_operations_s
     /*!
      * \brief Retrieve string listing hosts and port assignments from a local stonith device.
      *
-     * \retval 0 on success
-     * \retval negative error code on failure
+     * \return Legacy Pacemaker return code
      */
     int (*list)(stonith_t *st, int options, const char *id, char **list_output, int timeout);
 
     /*!
      * \brief Check to see if a local stonith device is reachable
      *
-     * \retval 0 on success
-     * \retval negative error code on failure
+     * \return Legacy Pacemaker return code
      */
     int (*monitor)(stonith_t *st, int options, const char *id, int timeout);
 
     /*!
      * \brief Check to see if a local stonith device's port is reachable
      *
-     * \retval 0 on success
-     * \retval negative error code on failure
+     * \return Legacy Pacemaker return code
      */
     int (*status)(stonith_t *st, int options, const char *id, const char *port, int timeout);
 
@@ -254,8 +254,7 @@ typedef struct stonith_api_operations_s
      * \note If node is provided, only devices that can fence the node id
      *       will be returned.
      *
-     * \retval num items in list on success
-     * \retval negative error code on failure
+     * \return Number of items in list on success, or negative errno otherwise
      */
     int (*query)(stonith_t *st, int options, const char *node,
             stonith_key_value_t **devices, int timeout);
@@ -272,8 +271,7 @@ typedef struct stonith_api_operations_s
      * \param timeout, The default per device timeout to use with each device
      *                 capable of fencing the target.
      *
-     * \retval 0 success
-     * \retval negative error code on failure.
+     * \return Legacy Pacemaker return code
      */
     int (*fence)(stonith_t *st, int options, const char *node, const char *action,
                  int timeout, int tolerance);
@@ -281,16 +279,14 @@ typedef struct stonith_api_operations_s
     /*!
      * \brief Manually confirm that a node is down.
      *
-     * \retval 0 success
-     * \retval negative error code on failure.
+     * \return Legacy Pacemaker return code
      */
     int (*confirm)(stonith_t *st, int options, const char *node);
 
     /*!
      * \brief Retrieve a list of fencing operations that have occurred for a specific node.
      *
-     * \retval 0 success
-     * \retval negative error code on failure.
+     * \return Legacy Pacemaker return code
      */
     int (*history)(stonith_t *st, int options, const char *node, stonith_history_t **output, int timeout);
 
@@ -406,8 +402,7 @@ typedef struct stonith_api_operations_s
      * \param delay, Apply a fencing delay. Value -1 means disable also any
      *               static/random fencing delays from pcmk_delay_base/max
      *
-     * \retval 0 success
-     * \retval negative error code on failure.
+     * \return Legacy Pacemaker return code
      */
     int (*fence_with_delay)(stonith_t *st, int options, const char *node, const char *action,
                             int timeout, int tolerance, int delay);
@@ -443,6 +438,7 @@ void stonith_history_free(stonith_history_t *history);
 // Convenience functions
 int stonith_api_connect_retry(stonith_t *st, const char *name,
                               int max_attempts);
+const char *stonith_op_state_str(enum op_state state);
 
 /* Basic helpers that allows nodes to be fenced and the history to be
  * queried without mainloop or the caller understanding the full API
@@ -557,9 +553,10 @@ bool stonith_agent_exists(const char *agent, int timeout);
  */
 const char *stonith_action_str(const char *action);
 
-#ifndef PCMK__NO_COMPAT
-/* Everything here is deprecated and kept only for public API backward
- * compatibility. It will be moved to compatibility.h in a future release.
+#if !defined(PCMK_ALLOW_DEPRECATED) || (PCMK_ALLOW_DEPRECATED == 1)
+/* Normally we'd put this section in a separate file (crm/fencing/compat.h), but
+ * we can't do that for the reason noted at the top of this file. That does mean
+ * we have to duplicate these declarations where they're implemented.
  */
 
 //! \deprecated Use stonith_get_namespace() instead
