@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2019 the Pacemaker project contributors
+ * Copyright 2004-2021 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -26,9 +26,29 @@ struct resource_alloc_functions_s {
     void (*internal_constraints) (pe_resource_t *, pe_working_set_t *);
 
     void (*rsc_colocation_lh) (pe_resource_t *, pe_resource_t *,
-                               rsc_colocation_t *, pe_working_set_t *);
+                               pcmk__colocation_t *, pe_working_set_t *);
     void (*rsc_colocation_rh) (pe_resource_t *, pe_resource_t *,
-                               rsc_colocation_t *, pe_working_set_t *);
+                               pcmk__colocation_t *, pe_working_set_t *);
+
+    /*!
+     * \internal
+     * \brief Create list of all resources in colocations with a given resource
+     *
+     * Given a resource, create a list of all resources involved in mandatory
+     * colocations with it, whether directly or indirectly via chained colocations.
+     *
+     * \param[in] rsc             Resource to add to colocated list
+     * \param[in] orig_rsc        Resource originally requested
+     * \param[in] colocated_rscs  Existing list
+     *
+     * \return List of given resource and all resources involved in colocations
+     *
+     * \note This function is recursive; top-level callers should pass NULL as
+     *       \p colocated_rscs and \p orig_rsc, and the desired resource as
+     *       \p rsc. The recursive calls will use other values.
+     */
+    GList *(*colocated_resources)(pe_resource_t *rsc, pe_resource_t *orig_rsc,
+                                  GList *colocated_rscs);
 
     void (*rsc_location) (pe_resource_t *, pe__location_t *);
 
@@ -55,14 +75,12 @@ pe_node_t *pcmk__native_allocate(pe_resource_t *rsc, pe_node_t *preferred,
                                  pe_working_set_t *data_set);
 extern void native_create_actions(pe_resource_t * rsc, pe_working_set_t * data_set);
 extern void native_internal_constraints(pe_resource_t * rsc, pe_working_set_t * data_set);
-void native_rsc_colocation_lh(pe_resource_t *lh_rsc, pe_resource_t *rh_rsc,
-                              rsc_colocation_t *constraint,
+void native_rsc_colocation_lh(pe_resource_t *dependent, pe_resource_t *primary,
+                              pcmk__colocation_t *constraint,
                               pe_working_set_t *data_set);
-void native_rsc_colocation_rh(pe_resource_t *lh_rsc, pe_resource_t *rh_rsc,
-                              rsc_colocation_t *constraint,
+void native_rsc_colocation_rh(pe_resource_t *dependent, pe_resource_t *primary,
+                              pcmk__colocation_t *constraint,
                               pe_working_set_t *data_set);
-extern void rsc_ticket_constraint(pe_resource_t * lh_rsc, rsc_ticket_t * rsc_ticket,
-                                  pe_working_set_t * data_set);
 extern enum pe_action_flags native_action_flags(pe_action_t * action, pe_node_t * node);
 
 void native_rsc_location(pe_resource_t *rsc, pe__location_t *constraint);
@@ -75,11 +93,11 @@ pe_node_t *pcmk__group_allocate(pe_resource_t *rsc, pe_node_t *preferred,
                                 pe_working_set_t *data_set);
 extern void group_create_actions(pe_resource_t * rsc, pe_working_set_t * data_set);
 extern void group_internal_constraints(pe_resource_t * rsc, pe_working_set_t * data_set);
-void group_rsc_colocation_lh(pe_resource_t *lh_rsc, pe_resource_t *rh_rsc,
-                             rsc_colocation_t *constraint,
+void group_rsc_colocation_lh(pe_resource_t *dependent, pe_resource_t *primary,
+                             pcmk__colocation_t *constraint,
                              pe_working_set_t *data_set);
-void group_rsc_colocation_rh(pe_resource_t *lh_rsc, pe_resource_t *rh_rsc,
-                             rsc_colocation_t *constraint,
+void group_rsc_colocation_rh(pe_resource_t *dependent, pe_resource_t *primary,
+                             pcmk__colocation_t *constraint,
                              pe_working_set_t *data_set);
 extern enum pe_action_flags group_action_flags(pe_action_t * action, pe_node_t * node);
 void group_rsc_location(pe_resource_t *rsc, pe__location_t *constraint);
@@ -95,13 +113,13 @@ gboolean pcmk__bundle_create_probe(pe_resource_t *rsc, pe_node_t *node,
                                    pe_working_set_t *data_set);
 void pcmk__bundle_internal_constraints(pe_resource_t *rsc,
                                        pe_working_set_t *data_set);
-void pcmk__bundle_rsc_colocation_lh(pe_resource_t *lh_rsc,
-                                    pe_resource_t *rh_rsc,
-                                    rsc_colocation_t *constraint,
+void pcmk__bundle_rsc_colocation_lh(pe_resource_t *dependent,
+                                    pe_resource_t *primary,
+                                    pcmk__colocation_t *constraint,
                                     pe_working_set_t *data_set);
-void pcmk__bundle_rsc_colocation_rh(pe_resource_t *lh_rsc,
-                                    pe_resource_t *rh_rsc,
-                                    rsc_colocation_t *constraint,
+void pcmk__bundle_rsc_colocation_rh(pe_resource_t *dependent,
+                                    pe_resource_t *primary,
+                                    pcmk__colocation_t *constraint,
                                     pe_working_set_t *data_set);
 void pcmk__bundle_rsc_location(pe_resource_t *rsc, pe__location_t *constraint);
 enum pe_action_flags pcmk__bundle_action_flags(pe_action_t *action,
@@ -113,11 +131,11 @@ pe_node_t *pcmk__clone_allocate(pe_resource_t *rsc, pe_node_t *preferred,
                                 pe_working_set_t *data_set);
 extern void clone_create_actions(pe_resource_t * rsc, pe_working_set_t * data_set);
 extern void clone_internal_constraints(pe_resource_t * rsc, pe_working_set_t * data_set);
-void clone_rsc_colocation_lh(pe_resource_t *lh_rsc, pe_resource_t *rh_rsc,
-                             rsc_colocation_t *constraint,
+void clone_rsc_colocation_lh(pe_resource_t *dependent, pe_resource_t *primary,
+                             pcmk__colocation_t *constraint,
                              pe_working_set_t *data_set);
-void clone_rsc_colocation_rh(pe_resource_t *lh_rsc, pe_resource_t *rh_rsc,
-                             rsc_colocation_t *constraint,
+void clone_rsc_colocation_rh(pe_resource_t *dependent, pe_resource_t *primary,
+                             pcmk__colocation_t *constraint,
                              pe_working_set_t *data_set);
 void clone_rsc_location(pe_resource_t *rsc, pe__location_t *constraint);
 extern enum pe_action_flags clone_action_flags(pe_action_t * action, pe_node_t * node);
@@ -126,34 +144,22 @@ extern gboolean clone_create_probe(pe_resource_t * rsc, pe_node_t * node, pe_act
                                    gboolean force, pe_working_set_t * data_set);
 extern void clone_append_meta(pe_resource_t * rsc, xmlNode * xml);
 
-void apply_master_prefs(pe_resource_t *rsc);
+void pcmk__add_promotion_scores(pe_resource_t *rsc);
 pe_node_t *pcmk__set_instance_roles(pe_resource_t *rsc,
                                     pe_working_set_t *data_set);
 void create_promotable_actions(pe_resource_t *rsc, pe_working_set_t *data_set);
 void promote_demote_constraints(pe_resource_t *rsc, pe_working_set_t *data_set);
 void promotable_constraints(pe_resource_t *rsc, pe_working_set_t *data_set);
-void promotable_colocation_rh(pe_resource_t *lh_rsc, pe_resource_t *rh_rsc,
-                              rsc_colocation_t *constraint,
+void promotable_colocation_rh(pe_resource_t *dependent, pe_resource_t *primary,
+                              pcmk__colocation_t *constraint,
                               pe_working_set_t *data_set);
 
 /* extern resource_object_functions_t resource_variants[]; */
 extern resource_alloc_functions_t resource_class_alloc_functions[];
 
-extern gboolean unpack_rsc_order(xmlNode * xml_obj, pe_working_set_t * data_set);
-
-extern gboolean unpack_rsc_colocation(xmlNode * xml_obj, pe_working_set_t * data_set);
-
-extern gboolean unpack_location(xmlNode * xml_obj, pe_working_set_t * data_set);
-
-extern gboolean unpack_rsc_ticket(xmlNode * xml_obj, pe_working_set_t * data_set);
-
-void LogNodeActions(pe_working_set_t * data_set, gboolean terminal);
-void LogActions(pe_resource_t * rsc, pe_working_set_t * data_set, gboolean terminal);
-void pcmk__bundle_log_actions(pe_resource_t *rsc, pe_working_set_t *data_set,
-                              gboolean terminal);
-
-extern void rsc_stonith_ordering(pe_resource_t * rsc, pe_action_t * stonith_op,
-                                 pe_working_set_t * data_set);
+void LogNodeActions(pe_working_set_t * data_set);
+void LogActions(pe_resource_t * rsc, pe_working_set_t * data_set);
+void pcmk__bundle_log_actions(pe_resource_t *rsc, pe_working_set_t *data_set);
 
 enum pe_graph_flags native_update_actions(pe_action_t *first, pe_action_t *then,
                                           pe_node_t *node,
@@ -175,10 +181,9 @@ enum pe_graph_flags pcmk__multi_update_actions(pe_action_t *first,
                                                enum pe_ordering type,
                                                pe_working_set_t *data_set);
 
-gboolean update_action_flags(pe_action_t * action, enum pe_action_flags flags, const char *source, int line);
 gboolean update_action(pe_action_t *action, pe_working_set_t *data_set);
 void complex_set_cmds(pe_resource_t * rsc);
 void pcmk__log_transition_summary(const char *filename);
 void clone_create_pseudo_actions(
-    pe_resource_t * rsc, GListPtr children, notify_data_t **start_notify, notify_data_t **stop_notify,  pe_working_set_t * data_set);
+    pe_resource_t * rsc, GList *children, notify_data_t **start_notify, notify_data_t **stop_notify,  pe_working_set_t * data_set);
 #endif
