@@ -1,19 +1,21 @@
 /*
- * Copyright 2001-2020 the Pacemaker project contributors
+ * Copyright 2001-2021 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
  * This source code is licensed under the GNU Lesser General Public License
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
-#ifndef PORTABILITY_H
-#  define PORTABILITY_H
+#ifndef PCMK__PORTABILITY__H
+#  define PCMK__PORTABILITY__H
 
-#  define	EOS			'\0'
-#  define	DIMOF(a)		((int) (sizeof(a)/sizeof(a[0])) )
+/* This header provides replacements for required definitions and declarations
+ * that certain supported build environments don't provide
+ */
 
-/* Needs to be defined before any other includes, otherwise some system
- * headers do not behave as expected! Major black magic... */
+/* This header *MUST* be included before any system headers, because the
+ * following definition can change the behavior of some of them.
+ */
 #  undef _GNU_SOURCE            /* in case it was defined on the command line */
 #  define _GNU_SOURCE
 
@@ -30,59 +32,38 @@
 #  ifndef HAVE_DAEMON
   /* We supply a replacement function, but need a prototype */
 int daemon(int nochdir, int noclose);
-#  endif                        /* HAVE_DAEMON */
+#  endif
 
 #  ifndef HAVE_SETENV
   /* We supply a replacement function, but need a prototype */
 int setenv(const char *name, const char *value, int why);
-#  endif                        /* HAVE_SETENV */
+#  endif
 
 #  ifndef HAVE_STRERROR
   /* We supply a replacement function, but need a prototype */
 char *strerror(int errnum);
-#  endif                        /* HAVE_STRERROR */
+#  endif
 
 #  ifndef HAVE_STRCHRNUL
   /* We supply a replacement function, but need a prototype */
 char *strchrnul(const char *s, int c_in);
-#  endif                        /* HAVE_STRCHRNUL */
+#  endif
 
 #  ifndef HAVE_ALPHASORT
 #    include <dirent.h>
-int
- alphasort(const void *dirent1, const void *dirent2);
-#  endif                        /* HAVE_ALPHASORT */
+int alphasort(const void *dirent1, const void *dirent2);
+#  endif
 
 #  ifndef HAVE_STRNLEN
 size_t strnlen(const char *s, size_t maxlen);
 #  else
-#    	define USE_GNU
+#    define USE_GNU
 #  endif
 
 #  ifndef HAVE_STRNDUP
 char *strndup(const char *str, size_t len);
 #  else
-#    	define USE_GNU
-#  endif
-
-// This test could be better, but it covers platforms of interest
-#  if defined(ON_BSD) || defined(ON_SOLARIS)
-#    define SUPPORT_PROCFS 0
-#  else
-#    define SUPPORT_PROCFS 1
-#  endif
-
-#  include <glib.h>
-
-#  if !GLIB_CHECK_VERSION(2,28,0)
-#    include <string.h>
-/* Since: 2.28 */
-static inline void
-g_list_free_full(GList * list, GDestroyNotify free_func)
-{
-    g_list_foreach(list, (GFunc) free_func, NULL);
-    g_list_free(list);
-}
+#    define USE_GNU
 #  endif
 
 #  if SUPPORT_DBUS
@@ -127,10 +108,11 @@ typedef union
   char *str;           /**< as char* (string, object path or signature) */
   int fd;              /**< as Unix file descriptor */
 } DBusBasicValue;
-#    endif
-#  endif
+#    endif  // !defined(HAVE_DBUSBASICVALUE)
+#  endif    // !defined(SUPPORT_DBUS)
+
+// Replacement constants for Linux-specific errno values
 
-/* Replacement error codes for non-linux */
 #  include <errno.h>
 
 #  ifndef ENOTUNIQ
@@ -177,4 +159,4 @@ typedef union
 #    define EKEYREJECTED 200
 #  endif
 
-#endif                          /* PORTABILITY_H */
+#endif // PCMK__PORTABILITY__H
