@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2020 the Pacemaker project contributors
+ * Copyright 2004-2021 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -25,7 +25,7 @@
 #include <crm/msg_xml.h>
 #include <crm/common/xml.h>
 
-GHashTable *cib_op_callback_table = NULL;
+static GHashTable *cib_op_callback_table = NULL;
 
 int cib_client_set_op_callback(cib_t * cib, void (*callback) (const xmlNode * msg, int call_id,
                                                               int rc, xmlNode * output));
@@ -292,6 +292,7 @@ cib_t *
 cib_new(void)
 {
     const char *value = getenv("CIB_shadow");
+    int port;
 
     if (value && value[0] != 0) {
         return cib_shadow_new(value);
@@ -305,11 +306,17 @@ cib_new(void)
     value = getenv("CIB_port");
     if (value) {
         gboolean encrypted = TRUE;
-        int port = crm_parse_int(value, NULL);
         const char *server = getenv("CIB_server");
         const char *user = getenv("CIB_user");
         const char *pass = getenv("CIB_passwd");
 
+        /* We don't ensure port is valid (>= 0) because cib_new() currently
+         * can't return NULL in practice, and introducing a NULL return here
+         * could cause core dumps that would previously just cause signon()
+         * failures.
+         */
+        pcmk__scan_port(value, &port);
+
         value = getenv("CIB_encrypted");
         if (value && crm_is_true(value) == FALSE) {
             crm_info("Disabling TLS");
@@ -348,6 +355,10 @@ cib_new_variant(void)
 
     new_cib = calloc(1, sizeof(cib_t));
 
+    if (new_cib == NULL) {
+        return NULL;
+    }
+
     remove_cib_op_callback(0, TRUE); /* remove all */
 
     new_cib->call_id = 1;
@@ -363,6 +374,11 @@ cib_new_variant(void)
     /* the rest will get filled in by the variant constructor */
     new_cib->cmds = calloc(1, sizeof(cib_api_operations_t));
 
+    if (new_cib->cmds == NULL) {
+        free(new_cib);
+        return NULL;
+    }
+
     new_cib->cmds->set_op_callback = cib_client_set_op_callback;
     new_cib->cmds->add_notify_callback = cib_client_add_notify_callback;
     new_cib->cmds->del_notify_callback = cib_client_del_notify_callback;
@@ -397,14 +413,10 @@ cib_new_variant(void)
     return new_cib;
 }
 
-/*!
- * \brief Free all callbacks for a CIB connection
- *
- * \param[in] cib  CIB connection to clean up
- */
-void
-cib_free_callbacks(cib_t *cib)
+void 
+cib_free_notify(cib_t *cib)
 {
+
     if (cib) {
         GList *list = cib->notify_list;
 
@@ -416,6 +428,17 @@ cib_free_callbacks(cib_t *cib)
         }
         cib->notify_list = NULL;
     }
+}
+/*!
+ * \brief Free all callbacks for a CIB connection
+ *
+ * \param[in] cib  CIB connection to clean up
+ */
+void
+cib_free_callbacks(cib_t *cib)
+{
+    cib_free_notify(cib);
+
     destroy_op_callback_table();
 }
 
@@ -634,7 +657,7 @@ cib_client_register_callback_full(cib_t *cib, int call_id, int timeout,
     }
 
     crm_trace("Adding callback %s for call %d", callback_name, call_id);
-    g_hash_table_insert(cib_op_callback_table, GINT_TO_POINTER(call_id), blob);
+    pcmk__intkey_table_insert(cib_op_callback_table, call_id, blob);
 
     return TRUE;
 }
@@ -644,10 +667,9 @@ remove_cib_op_callback(int call_id, gboolean all_callbacks)
 {
     if (all_callbacks) {
         destroy_op_callback_table();
-        cib_op_callback_table = g_hash_table_new_full(g_direct_hash, g_direct_equal,
-                                                      NULL, cib_destroy_op_callback);
+        cib_op_callback_table = pcmk__intkey_table(cib_destroy_op_callback);
     } else {
-        g_hash_table_remove(cib_op_callback_table, GINT_TO_POINTER(call_id));
+        pcmk__intkey_table_remove(cib_op_callback_table, call_id);
     }
 }
 
@@ -677,3 +699,9 @@ cib_dump_pending_callbacks(void)
     }
     return g_hash_table_foreach(cib_op_callback_table, cib_dump_pending_op, NULL);
 }
+
+cib_callback_client_t*
+cib__lookup_id (int call_id)
+{
+    return pcmk__intkey_table_lookup(cib_op_callback_table, call_id);
+}
\ No newline at end of file
