@@ -1,6 +1,6 @@
 /*
  * Copyright 2004 International Business Machines
- * Later changes copyright 2004-2020 the Pacemaker project contributors
+ * Later changes copyright 2004-2021 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -58,8 +58,17 @@ cib_native_new(void)
     cib_native_opaque_t *native = NULL;
     cib_t *cib = cib_new_variant();
 
+    if (cib == NULL) {
+        return NULL;
+    }
+
     native = calloc(1, sizeof(cib_native_opaque_t));
 
+    if (native == NULL) {
+        free(cib);
+        return NULL;
+    }
+
     cib->variant = cib_native;
     cib->variant_opaque = native;
 
@@ -155,7 +164,7 @@ cib_native_signon_raw(cib_t * cib, const char *name, enum cib_conn_type type, in
         .destroy = cib_native_destroy
     };
 
-    cib->call_timeout = MAX_IPC_DELAY;
+    cib->call_timeout = PCMK__IPC_TIMEOUT;
 
     if (type == cib_command) {
         cib->state = cib_connected_command;
@@ -250,6 +259,9 @@ cib_native_signoff(cib_t * cib)
 
     crm_debug("Disconnecting from the CIB manager");
 
+    cib_free_notify(cib);
+    remove_cib_op_callback(0, TRUE);
+
     if (native->source != NULL) {
         /* Attached to mainloop */
         mainloop_del_ipc_client(native->source);
