@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2020 the Pacemaker project contributors
+ * Copyright 2004-2021 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -69,25 +69,6 @@ done:
     return rc;
 }
 
-/*!
- * \brief Create a directory, including any parent directories needed
- *
- * \param[in] path_c Pathname of the directory to create
- * \param[in] mode Permissions to be used (with current umask) when creating
- *
- * \note This logs errors but does not return them to the caller.
- */
-void
-crm_build_path(const char *path_c, mode_t mode)
-{
-    int rc = pcmk__build_path(path_c, mode);
-
-    if (rc != pcmk_rc_ok) {
-        crm_err("Could not create directory '%s': %s",
-                path_c, pcmk_rc_str(rc));
-    }
-}
-
 /*!
  * \internal
  * \brief Return canonicalized form of a path name
@@ -633,3 +614,49 @@ pcmk__close_fds_in_child(bool all)
         close(lpc);
     }
 }
+
+/*!
+ * \brief Duplicate a file path, inserting a prefix if not absolute
+ *
+ * \param[in] filename  File path to duplicate
+ * \param[in] dirname   If filename is not absolute, prefix to add
+ *
+ * \return Newly allocated memory with full path (guaranteed non-NULL)
+ */
+char *
+pcmk__full_path(const char *filename, const char *dirname)
+{
+    char *path = NULL;
+
+    CRM_ASSERT(filename != NULL);
+
+    if (filename[0] == '/') {
+        path = strdup(filename);
+        CRM_ASSERT(path != NULL);
+
+    } else {
+        CRM_ASSERT(dirname != NULL);
+        path = crm_strdup_printf("%s/%s", dirname, filename);
+    }
+
+    return path;
+}
+
+// Deprecated functions kept only for backward API compatibility
+// LCOV_EXCL_START
+
+#include <crm/common/util_compat.h>
+
+void
+crm_build_path(const char *path_c, mode_t mode)
+{
+    int rc = pcmk__build_path(path_c, mode);
+
+    if (rc != pcmk_rc_ok) {
+        crm_err("Could not create directory '%s': %s",
+                path_c, pcmk_rc_str(rc));
+    }
+}
+
+// LCOV_EXCL_STOP
+// End deprecated API
