@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2020 the Pacemaker project contributors
+ * Copyright 2004-2021 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -24,6 +24,7 @@
 #include <crm/lrmd.h>
 #include <crm/msg_xml.h>
 #include <crm/common/xml.h>
+#include <crm/common/xml_internal.h>
 #include <crm/common/util.h>
 
 static regex_t *notify_migrate_re = NULL;
@@ -361,6 +362,24 @@ decode_transition_key(const char *key, char **uuid, int *transition_id, int *act
     return TRUE;
 }
 
+// Return true if a is an attribute that should be filtered
+static bool
+should_filter_for_digest(xmlAttrPtr a, void *user_data)
+{
+    if (strncmp((const char *) a->name, CRM_META "_",
+                sizeof(CRM_META " ") - 1) == 0) {
+        return true;
+    }
+    return pcmk__str_any_of((const char *) a->name,
+                            XML_ATTR_ID,
+                            XML_ATTR_CRM_VERSION,
+                            XML_LRM_ATTR_OP_DIGEST,
+                            XML_LRM_ATTR_TARGET,
+                            XML_LRM_ATTR_TARGET_UUID,
+                            "pcmk_external_ip",
+                            NULL);
+}
+
 /*!
  * \internal
  * \brief Remove XML attributes not needed for operation digest
@@ -374,52 +393,31 @@ pcmk__filter_op_for_digest(xmlNode *param_set)
     char *timeout = NULL;
     guint interval_ms = 0;
 
-    const char *attr_filter[] = {
-        XML_ATTR_ID,
-        XML_ATTR_CRM_VERSION,
-        XML_LRM_ATTR_OP_DIGEST,
-        XML_LRM_ATTR_TARGET,
-        XML_LRM_ATTR_TARGET_UUID,
-        "pcmk_external_ip"
-    };
-
-    const int meta_len = strlen(CRM_META);
-
     if (param_set == NULL) {
         return;
     }
 
-    // Remove the specific attributes listed in attr_filter
-    for (int lpc = 0; lpc < DIMOF(attr_filter); lpc++) {
-        xml_remove_prop(param_set, attr_filter[lpc]);
-    }
-
+    /* Timeout is useful for recurring operation digests, so grab it before
+     * removing meta-attributes
+     */
     key = crm_meta_name(XML_LRM_ATTR_INTERVAL_MS);
     if (crm_element_value_ms(param_set, key, &interval_ms) != pcmk_ok) {
         interval_ms = 0;
     }
     free(key);
-
-    key = crm_meta_name(XML_ATTR_TIMEOUT);
-    timeout = crm_element_value_copy(param_set, key);
-
-    // Remove all CRM_meta_* attributes
-    for (xmlAttrPtr xIter = param_set->properties; xIter != NULL; ) {
-        const char *prop_name = (const char *) (xIter->name);
-
-        xIter = xIter->next;
-
-        // @TODO Why is this case-insensitive?
-        if (strncasecmp(prop_name, CRM_META, meta_len) == 0) {
-            xml_remove_prop(param_set, prop_name);
-        }
+    key = NULL;
+    if (interval_ms != 0) {
+        key = crm_meta_name(XML_ATTR_TIMEOUT);
+        timeout = crm_element_value_copy(param_set, key);
     }
 
-    if ((interval_ms != 0) && (timeout != NULL)) {
-        // Add the timeout back, it's useful for recurring operation digests
+    // Remove all CRM_meta_* attributes and certain other attributes
+    pcmk__xe_remove_matching_attrs(param_set, should_filter_for_digest, NULL);
+
+    // Add timeout back for recurring operation digests
+    if (timeout != NULL) {
         crm_xml_add(param_set, key, timeout);
     }
-
     free(timeout);
     free(key);
 }
@@ -439,15 +437,17 @@ gboolean
 did_rsc_op_fail(lrmd_event_data_t * op, int target_rc)
 {
     switch (op->op_status) {
-        case PCMK_LRM_OP_CANCELLED:
-        case PCMK_LRM_OP_PENDING:
+        case PCMK_EXEC_CANCELLED:
+        case PCMK_EXEC_PENDING:
             return FALSE;
 
-        case PCMK_LRM_OP_NOTSUPPORTED:
-        case PCMK_LRM_OP_TIMEOUT:
-        case PCMK_LRM_OP_ERROR:
-        case PCMK_LRM_OP_NOT_CONNECTED:
-        case PCMK_LRM_OP_INVALID:
+        case PCMK_EXEC_NOT_SUPPORTED:
+        case PCMK_EXEC_TIMEOUT:
+        case PCMK_EXEC_ERROR:
+        case PCMK_EXEC_NOT_CONNECTED:
+        case PCMK_EXEC_NO_FENCE_DEVICE:
+        case PCMK_EXEC_NO_SECRETS:
+        case PCMK_EXEC_INVALID:
             return TRUE;
 
         default:
@@ -500,9 +500,9 @@ crm_create_op_xml(xmlNode *parent, const char *prefix, const char *task,
 bool
 crm_op_needs_metadata(const char *rsc_class, const char *op)
 {
-    /* Agent meta-data is used to determine whether a reload is possible, and to
-     * evaluate versioned parameters -- so if this op is not relevant to those
-     * features, we don't need the meta-data.
+    /* Agent meta-data is used to determine whether an agent reload is possible,
+     * and to evaluate versioned parameters -- so if this op is not relevant to
+     * those features, we don't need the meta-data.
      */
 
     CRM_CHECK((rsc_class != NULL) || (op != NULL), return false);
@@ -519,6 +519,7 @@ crm_op_needs_metadata(const char *rsc_class, const char *op)
     /* Meta-data is only needed for these actions */
     return pcmk__str_any_of(op, CRMD_ACTION_START, CRMD_ACTION_STATUS,
                             CRMD_ACTION_PROMOTE, CRMD_ACTION_DEMOTE,
-                            CRMD_ACTION_RELOAD, CRMD_ACTION_MIGRATE,
-                            CRMD_ACTION_MIGRATED, CRMD_ACTION_NOTIFY, NULL);
+                            CRMD_ACTION_RELOAD, CRMD_ACTION_RELOAD_AGENT,
+                            CRMD_ACTION_MIGRATE, CRMD_ACTION_MIGRATED,
+                            CRMD_ACTION_NOTIFY, NULL);
 }
