@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2020 the Pacemaker project contributors
+ * Copyright 2004-2021 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -24,7 +24,6 @@
 
 #include <crm/crm.h>
 #include <crm/msg_xml.h>
-#include <crm/common/iso8601_internal.h>
 #include <crm/common/xml.h>
 #include <crm/common/xml_internal.h>  // CRM_XML_LOG_BASE, etc.
 #include "crmcommon_private.h"
@@ -92,13 +91,13 @@ add_xml_changes_to_patchset(xmlNode *xml, xmlNode *patchset)
     const char *value = NULL;
 
     // If this XML node is new, just report that
-    if (patchset && pcmk_is_set(p->flags, xpf_created)) {
+    if (patchset && pcmk_is_set(p->flags, pcmk__xf_created)) {
         int offset = 0;
         char buffer[PCMK__BUFFER_SIZE];
 
         if (pcmk__element_xpath(NULL, xml->parent, buffer, offset,
                                 sizeof(buffer)) > 0) {
-            int position = pcmk__xml_position(xml, xpf_deleted);
+            int position = pcmk__xml_position(xml, pcmk__xf_deleted);
 
             change = create_xml_node(patchset, XML_DIFF_CHANGE);
 
@@ -112,12 +111,12 @@ add_xml_changes_to_patchset(xmlNode *xml, xmlNode *patchset)
     }
 
     // Check each of the XML node's attributes for changes
-    for (pIter = pcmk__first_xml_attr(xml); pIter != NULL;
+    for (pIter = pcmk__xe_first_attr(xml); pIter != NULL;
          pIter = pIter->next) {
         xmlNode *attr = NULL;
 
         p = pIter->_private;
-        if (!pcmk_any_flags_set(p->flags, xpf_deleted|xpf_dirty)) {
+        if (!pcmk_any_flags_set(p->flags, pcmk__xf_deleted|pcmk__xf_dirty)) {
             continue;
         }
 
@@ -139,7 +138,7 @@ add_xml_changes_to_patchset(xmlNode *xml, xmlNode *patchset)
         attr = create_xml_node(change, XML_DIFF_ATTR);
 
         crm_xml_add(attr, XML_NVPAIR_ATTR_NAME, (const char *)pIter->name);
-        if (p->flags & xpf_deleted) {
+        if (p->flags & pcmk__xf_deleted) {
             crm_xml_add(attr, XML_DIFF_OP, "unset");
 
         } else {
@@ -156,10 +155,10 @@ add_xml_changes_to_patchset(xmlNode *xml, xmlNode *patchset)
         change = create_xml_node(change->parent, XML_DIFF_RESULT);
         result = create_xml_node(change, (const char *)xml->name);
 
-        for (pIter = pcmk__first_xml_attr(xml); pIter != NULL;
+        for (pIter = pcmk__xe_first_attr(xml); pIter != NULL;
              pIter = pIter->next) {
             p = pIter->_private;
-            if (!pcmk_is_set(p->flags, xpf_deleted)) {
+            if (!pcmk_is_set(p->flags, pcmk__xf_deleted)) {
                 value = crm_element_value(xml, (const char *) pIter->name);
                 crm_xml_add(result, (const char *)pIter->name, value);
             }
@@ -173,12 +172,12 @@ add_xml_changes_to_patchset(xmlNode *xml, xmlNode *patchset)
     }
 
     p = xml->_private;
-    if (patchset && pcmk_is_set(p->flags, xpf_moved)) {
+    if (patchset && pcmk_is_set(p->flags, pcmk__xf_moved)) {
         int offset = 0;
         char buffer[PCMK__BUFFER_SIZE];
 
         crm_trace("%s.%s moved to position %d",
-                  xml->name, ID(xml), pcmk__xml_position(xml, xpf_skip));
+                  xml->name, ID(xml), pcmk__xml_position(xml, pcmk__xf_skip));
         if (pcmk__element_xpath(NULL, xml, buffer, offset,
                                 sizeof(buffer)) > 0) {
             change = create_xml_node(patchset, XML_DIFF_CHANGE);
@@ -186,7 +185,7 @@ add_xml_changes_to_patchset(xmlNode *xml, xmlNode *patchset)
             crm_xml_add(change, XML_DIFF_OP, "move");
             crm_xml_add(change, XML_DIFF_PATH, buffer);
             crm_xml_add_int(change, XML_DIFF_POSITION,
-                            pcmk__xml_position(xml, xpf_deleted));
+                            pcmk__xml_position(xml, pcmk__xf_deleted));
         }
     }
 }
@@ -194,14 +193,14 @@ add_xml_changes_to_patchset(xmlNode *xml, xmlNode *patchset)
 static bool
 is_config_change(xmlNode *xml)
 {
-    GListPtr gIter = NULL;
+    GList *gIter = NULL;
     xml_private_t *p = NULL;
     xmlNode *config = first_named_child(xml, XML_CIB_TAG_CONFIGURATION);
 
     if (config) {
         p = config->_private;
     }
-    if ((p != NULL) && pcmk_is_set(p->flags, xpf_dirty)) {
+    if ((p != NULL) && pcmk_is_set(p->flags, pcmk__xf_dirty)) {
         return TRUE;
     }
 
@@ -252,7 +251,7 @@ xml_repair_v1_diff(xmlNode *last, xmlNode *next, xmlNode *local_diff,
         cib = create_xml_node(diff_child, tag);
     }
 
-    for (lpc = 0; (last != NULL) && (lpc < DIMOF(vfields)); lpc++) {
+    for (lpc = 0; (last != NULL) && (lpc < PCMK__NELEM(vfields)); lpc++) {
         const char *value = crm_element_value(last, vfields[lpc]);
 
         crm_xml_add(diff_child, vfields[lpc], value);
@@ -273,21 +272,16 @@ xml_repair_v1_diff(xmlNode *last, xmlNode *next, xmlNode *local_diff,
         cib = create_xml_node(diff_child, tag);
     }
 
-    for (lpc = 0; next && lpc < DIMOF(vfields); lpc++) {
+    for (lpc = 0; next && lpc < PCMK__NELEM(vfields); lpc++) {
         const char *value = crm_element_value(next, vfields[lpc]);
 
         crm_xml_add(diff_child, vfields[lpc], value);
     }
 
-    if (next) {
-        xmlAttrPtr xIter = NULL;
+    for (xmlAttrPtr a = pcmk__xe_first_attr(next); a != NULL; a = a->next) {
+        const char *p_value = crm_element_value(next, (const char *) a->name);
 
-        for (xIter = next->properties; xIter; xIter = xIter->next) {
-            const char *p_name = (const char *) xIter->name;
-            const char *p_value = crm_element_value(next, p_name);
-
-            xmlSetProp(cib, (pcmkXmlStr) p_name, (pcmkXmlStr) p_value);
-        }
+        xmlSetProp(cib, a->name, (pcmkXmlStr) p_value);
     }
 
     crm_log_xml_explicit(local_diff, "Repaired-diff");
@@ -311,7 +305,7 @@ static xmlNode *
 xml_create_patchset_v2(xmlNode *source, xmlNode *target)
 {
     int lpc = 0;
-    GListPtr gIter = NULL;
+    GList *gIter = NULL;
     xml_private_t *doc = NULL;
 
     xmlNode *v = NULL;
@@ -337,7 +331,7 @@ xml_create_patchset_v2(xmlNode *source, xmlNode *target)
     version = create_xml_node(patchset, XML_DIFF_VERSION);
 
     v = create_xml_node(version, XML_DIFF_VSOURCE);
-    for (lpc = 0; lpc < DIMOF(vfields); lpc++) {
+    for (lpc = 0; lpc < PCMK__NELEM(vfields); lpc++) {
         const char *value = crm_element_value(source, vfields[lpc]);
 
         if (value == NULL) {
@@ -347,7 +341,7 @@ xml_create_patchset_v2(xmlNode *source, xmlNode *target)
     }
 
     v = create_xml_node(version, XML_DIFF_VTARGET);
-    for (lpc = 0; lpc < DIMOF(vfields); lpc++) {
+    for (lpc = 0; lpc < PCMK__NELEM(vfields); lpc++) {
         const char *value = crm_element_value(target, vfields[lpc]);
 
         if (value == NULL) {
@@ -638,13 +632,19 @@ xml_log_patchset(uint8_t log_level, const char *function, xmlNode *patchset)
     }
 }
 
+// Return true if attribute name is not "id"
+static bool
+not_id(xmlAttrPtr attr, void *user_data)
+{
+    return strcmp((const char *) attr->name, XML_ATTR_ID) != 0;
+}
+
 // Apply the removals section of an v1 patchset to an XML node
 static void
 process_v1_removals(xmlNode *target, xmlNode *patch)
 {
     xmlNode *patch_child = NULL;
     xmlNode *cIter = NULL;
-    xmlAttrPtr xIter = NULL;
 
     char *id = NULL;
     const char *name = NULL;
@@ -677,15 +677,8 @@ process_v1_removals(xmlNode *target, xmlNode *patch)
         return;
     }
 
-    for (xIter = pcmk__first_xml_attr(patch); xIter != NULL;
-         xIter = xIter->next) {
-        const char *p_name = (const char *)xIter->name;
-
-        // Removing then restoring id would change ordering of properties
-        if (!pcmk__str_eq(p_name, XML_ATTR_ID, pcmk__str_casei)) {
-            xml_remove_prop(target, p_name);
-        }
-    }
+    // Removing then restoring id would change ordering of properties
+    pcmk__xe_remove_matching_attrs(patch, not_id, NULL);
 
     // Changes to child objects
     cIter = pcmk__xml_first_child(target);
@@ -745,7 +738,7 @@ process_v1_additions(xmlNode *parent, xmlNode *target, xmlNode *patch)
               return);
     CRM_CHECK(pcmk__str_eq(ID(target), ID(patch), pcmk__str_casei), return);
 
-    for (xIter = pcmk__first_xml_attr(patch); xIter != NULL;
+    for (xIter = pcmk__xe_first_attr(patch); xIter != NULL;
          xIter = xIter->next) {
         const char *p_name = (const char *) xIter->name;
         const char *p_value = crm_element_value(patch, p_name);
@@ -825,7 +818,7 @@ xml_patch_versions(xmlNode *patchset, int add[3], int del[3])
         return -EINVAL;
     }
     if (tmp != NULL) {
-        for (lpc = 0; lpc < DIMOF(vfields); lpc++) {
+        for (lpc = 0; lpc < PCMK__NELEM(vfields); lpc++) {
             crm_element_value_int(tmp, vfields[lpc], &(del[lpc]));
             crm_trace("Got %d for del[%s]", del[lpc], vfields[lpc]);
         }
@@ -836,7 +829,7 @@ xml_patch_versions(xmlNode *patchset, int add[3], int del[3])
         return -EINVAL;
     }
     if (tmp != NULL) {
-        for (lpc = 0; lpc < DIMOF(vfields); lpc++) {
+        for (lpc = 0; lpc < PCMK__NELEM(vfields); lpc++) {
             crm_element_value_int(tmp, vfields[lpc], &(add[lpc]));
             crm_trace("Got %d for add[%s]", add[lpc], vfields[lpc]);
         }
@@ -870,7 +863,7 @@ xml_patch_version_check(xmlNode *xml, xmlNode *patchset, int format)
         XML_ATTR_NUMUPDATES,
     };
 
-    for (lpc = 0; lpc < DIMOF(vfields); lpc++) {
+    for (lpc = 0; lpc < PCMK__NELEM(vfields); lpc++) {
         crm_element_value_int(xml, vfields[lpc], &(this[lpc]));
         crm_trace("Got %d for this[%s]", this[lpc], vfields[lpc]);
         if (this[lpc] < 0) {
@@ -882,13 +875,13 @@ xml_patch_version_check(xmlNode *xml, xmlNode *patchset, int format)
     add[0] = this[0];
     add[1] = this[1];
     add[2] = this[2] + 1;
-    for (lpc = 0; lpc < DIMOF(vfields); lpc++) {
+    for (lpc = 0; lpc < PCMK__NELEM(vfields); lpc++) {
         del[lpc] = this[lpc];
     }
 
     xml_patch_versions(patchset, add, del);
 
-    for (lpc = 0; lpc < DIMOF(vfields); lpc++) {
+    for (lpc = 0; lpc < PCMK__NELEM(vfields); lpc++) {
         if (this[lpc] < del[lpc]) {
             crm_debug("Current %s is too low (%d.%d.%d < %d.%d.%d --> %d.%d.%d)",
                       vfields[lpc], this[0], this[1], this[2],
@@ -904,7 +897,7 @@ xml_patch_version_check(xmlNode *xml, xmlNode *patchset, int format)
         }
     }
 
-    for (lpc = 0; lpc < DIMOF(vfields); lpc++) {
+    for (lpc = 0; lpc < PCMK__NELEM(vfields); lpc++) {
         if (add[lpc] > del[lpc]) {
             changed = TRUE;
         }
@@ -1006,7 +999,7 @@ first_matching_xml_child(xmlNode *parent, const char *name, const char *id,
         // "position" makes sense only for XML comments for now
         if ((cIter->type == XML_COMMENT_NODE)
             && (position >= 0)
-            && (pcmk__xml_position(cIter, xpf_skip) != position)) {
+            && (pcmk__xml_position(cIter, pcmk__xf_skip) != position)) {
             continue;
         }
 
@@ -1150,8 +1143,8 @@ apply_v2_patchset(xmlNode *xml, xmlNode *patchset)
 {
     int rc = pcmk_rc_ok;
     xmlNode *change = NULL;
-    GListPtr change_objs = NULL;
-    GListPtr gIter = NULL;
+    GList *change_objs = NULL;
+    GList *gIter = NULL;
 
     for (change = pcmk__xml_first_child(patchset); change != NULL;
          change = pcmk__xml_next(change)) {
@@ -1204,7 +1197,6 @@ apply_v2_patchset(xmlNode *xml, xmlNode *patchset)
             free_xml(match);
 
         } else if (strcmp(op, "modify") == 0) {
-            xmlAttr *pIter = pcmk__first_xml_attr(match);
             xmlNode *attrs = NULL;
 
             attrs = pcmk__xml_first_child(first_named_child(change,
@@ -1213,14 +1205,9 @@ apply_v2_patchset(xmlNode *xml, xmlNode *patchset)
                 rc = ENOMSG;
                 continue;
             }
-            while (pIter != NULL) {
-                const char *name = (const char *)pIter->name;
+            pcmk__xe_remove_matching_attrs(match, NULL, NULL); // Remove all
 
-                pIter = pIter->next;
-                xml_remove_prop(match, name);
-            }
-
-            for (pIter = pcmk__first_xml_attr(attrs); pIter != NULL;
+            for (xmlAttrPtr pIter = pcmk__xe_first_attr(attrs); pIter != NULL;
                  pIter = pIter->next) {
                 const char *name = (const char *) pIter->name;
                 const char *value = crm_element_value(attrs, name);
@@ -1259,7 +1246,7 @@ apply_v2_patchset(xmlNode *xml, xmlNode *patchset)
             crm_element_value_int(change, XML_DIFF_POSITION, &position);
 
             while ((match_child != NULL)
-                   && (position != pcmk__xml_position(match_child, xpf_skip))) {
+                   && (position != pcmk__xml_position(match_child, pcmk__xf_skip))) {
                 match_child = match_child->next;
             }
 
@@ -1285,11 +1272,11 @@ apply_v2_patchset(xmlNode *xml, xmlNode *patchset)
             int position = 0;
 
             crm_element_value_int(change, XML_DIFF_POSITION, &position);
-            if (position != pcmk__xml_position(match, xpf_skip)) {
+            if (position != pcmk__xml_position(match, pcmk__xf_skip)) {
                 xmlNode *match_child = NULL;
                 int p = position;
 
-                if (p > pcmk__xml_position(match, xpf_skip)) {
+                if (p > pcmk__xml_position(match, pcmk__xf_skip)) {
                     p++; // Skip ourselves
                 }
 
@@ -1297,13 +1284,13 @@ apply_v2_patchset(xmlNode *xml, xmlNode *patchset)
                 match_child = match->parent->children;
 
                 while ((match_child != NULL)
-                       && (p != pcmk__xml_position(match_child, xpf_skip))) {
+                       && (p != pcmk__xml_position(match_child, pcmk__xf_skip))) {
                     match_child = match_child->next;
                 }
 
                 crm_trace("Moving %s to position %d (was %d, prev %p, %s %p)",
                           match->name, position,
-                          pcmk__xml_position(match, xpf_skip),
+                          pcmk__xml_position(match, pcmk__xf_skip),
                           match->prev, (match_child? "next":"last"),
                           (match_child? match_child : match->parent->last));
 
@@ -1320,10 +1307,10 @@ apply_v2_patchset(xmlNode *xml, xmlNode *patchset)
                           match->name, position);
             }
 
-            if (position != pcmk__xml_position(match, xpf_skip)) {
+            if (position != pcmk__xml_position(match, pcmk__xf_skip)) {
                 crm_err("Moved %s.%s to position %d instead of %d (%p)",
                         match->name, ID(match),
-                        pcmk__xml_position(match, xpf_skip),
+                        pcmk__xml_position(match, pcmk__xf_skip),
                         position, match->prev);
                 rc = pcmk_rc_diff_failed;
             }
@@ -1553,7 +1540,7 @@ subtract_xml_object(xmlNode *parent, xmlNode *left, xmlNode *right,
     } else if (full) {
         xmlAttrPtr pIter = NULL;
 
-        for (pIter = pcmk__first_xml_attr(left); pIter != NULL;
+        for (pIter = pcmk__xe_first_attr(left); pIter != NULL;
              pIter = pIter->next) {
             const char *p_name = (const char *)pIter->name;
             const char *p_value = pcmk__xml_attr_value(pIter);
@@ -1566,7 +1553,7 @@ subtract_xml_object(xmlNode *parent, xmlNode *left, xmlNode *right,
     }
 
     // Changes to name/value pairs
-    for (xIter = pcmk__first_xml_attr(left); xIter != NULL;
+    for (xIter = pcmk__xe_first_attr(left); xIter != NULL;
          xIter = xIter->next) {
         const char *prop_name = (const char *) xIter->name;
         xmlAttrPtr right_attr = NULL;
@@ -1588,13 +1575,13 @@ subtract_xml_object(xmlNode *parent, xmlNode *left, xmlNode *right,
         }
 
         right_val = crm_element_value(right, prop_name);
-        if ((right_val == NULL) || (p && pcmk_is_set(p->flags, xpf_deleted))) {
+        if ((right_val == NULL) || (p && pcmk_is_set(p->flags, pcmk__xf_deleted))) {
             /* new */
             *changed = TRUE;
             if (full) {
                 xmlAttrPtr pIter = NULL;
 
-                for (pIter = pcmk__first_xml_attr(left); pIter != NULL;
+                for (pIter = pcmk__xe_first_attr(left); pIter != NULL;
                      pIter = pIter->next) {
                     const char *p_name = (const char *) pIter->name;
                     const char *p_value = pcmk__xml_attr_value(pIter);
@@ -1624,7 +1611,7 @@ subtract_xml_object(xmlNode *parent, xmlNode *left, xmlNode *right,
 
                     crm_trace("Changes detected to %s in <%s id=%s>", prop_name,
                               crm_element_name(left), id);
-                    for (pIter = pcmk__first_xml_attr(left); pIter != NULL;
+                    for (pIter = pcmk__xe_first_attr(left); pIter != NULL;
                          pIter = pIter->next) {
                         const char *p_name = (const char *) pIter->name;
                         const char *p_value = pcmk__xml_attr_value(pIter);
@@ -1656,13 +1643,10 @@ subtract_xml_object(xmlNode *parent, xmlNode *left, xmlNode *right,
 }
 
 // Deprecated functions kept only for backward API compatibility
+// LCOV_EXCL_START
 
-gboolean apply_xml_diff(xmlNode *old_xml, xmlNode *diff, xmlNode **new_xml);
+#include <crm/common/xml_compat.h>
 
-/*!
- * \deprecated Use xml_apply_patchset() instead
- * \note This function will be removed in a future version of Pacemaker.
- */
 gboolean
 apply_xml_diff(xmlNode *old_xml, xmlNode *diff, xmlNode **new_xml)
 {
@@ -1752,3 +1736,6 @@ apply_xml_diff(xmlNode *old_xml, xmlNode *diff, xmlNode **new_xml)
 
     return result;
 }
+
+// LCOV_EXCL_STOP
+// End deprecated API
