@@ -1,5 +1,5 @@
 /*
- * Copyright 2013-2019 the Pacemaker project contributors
+ * Copyright 2013-2020 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -24,15 +24,7 @@
 #  include <sys/mman.h>
 #endif
 
-static int sbd_pid = 0;
-
-enum pcmk_panic_flags
-{
-    pcmk_panic_none     = 0x00,
-    pcmk_panic_delay    = 0x01,
-    pcmk_panic_kdump    = 0x02,
-    pcmk_panic_shutdown = 0x04,
-};
+static pid_t sbd_pid = 0;
 
 static void
 sysrq_trigger(char t)
@@ -54,8 +46,12 @@ sysrq_trigger(char t)
 }
 
 
+/*!
+ * \internal
+ * \brief Panic the local host (if root) or tell pacemakerd to do so
+ */
 static void
-pcmk_panic_local(void) 
+panic_local(void)
 {
     int rc = pcmk_ok;
     uid_t uid = geteuid();
@@ -68,7 +64,7 @@ pcmk_panic_local(void)
          *
          * Of these, only the controller is likely to be initiating resets.
          */
-        do_crm_log_always(LOG_EMERG, "Signaling parent %d to panic", ppid);
+        crm_emerg("Signaling parent %lld to panic", (long long) ppid);
         crm_exit(CRM_EX_PANIC);
         return;
 
@@ -81,11 +77,12 @@ pcmk_panic_local(void)
         union sigval signal_value;
 
         memset(&signal_value, 0, sizeof(signal_value));
-        ppid = crm_procfs_pid_of("pacemakerd");
-        do_crm_log_always(LOG_EMERG, "Signaling pacemakerd(%d) to panic", ppid);
+        ppid = pcmk__procfs_pid_of("pacemakerd");
+        crm_emerg("Signaling pacemakerd[%lld] to panic", (long long) ppid);
 
         if(ppid > 1 && sigqueue(ppid, SIGQUIT, signal_value) < 0) {
-            crm_perror(LOG_EMERG, "Cannot signal pacemakerd(%d) to panic", ppid);
+            crm_perror(LOG_EMERG, "Cannot signal pacemakerd[%lld] to panic",
+                       (long long) ppid);
         }
 #endif // SUPPORT_PROCFS
 
@@ -96,16 +93,23 @@ pcmk_panic_local(void)
 
     /* We're either pacemakerd, or a pacemaker daemon running as root */
 
-    if (safe_str_eq("crash", getenv("PCMK_panic_action"))) {
+    if (pcmk__str_eq("crash", getenv("PCMK_panic_action"), pcmk__str_casei)) {
+        sysrq_trigger('c');
+    } else if (pcmk__str_eq("sync-crash", getenv("PCMK_panic_action"), pcmk__str_casei)) {
+        sync();
         sysrq_trigger('c');
     } else {
+        if (pcmk__str_eq("sync-reboot", getenv("PCMK_panic_action"), pcmk__str_casei)) {
+            sync();
+        }
         sysrq_trigger('b');
     }
     /* reboot(RB_HALT_SYSTEM); rc = errno; */
     reboot(RB_AUTOBOOT);
     rc = errno;
 
-    do_crm_log_always(LOG_EMERG, "Reboot failed, escalating to %d: %s (%d)", ppid, pcmk_strerror(rc), rc);
+    crm_emerg("Reboot failed, escalating to parent %lld: %s " CRM_XS " rc=%d",
+              (long long) ppid, pcmk_rc_str(rc), rc);
 
     if(ppid > 1) {
         /* child daemon */
@@ -116,19 +120,24 @@ pcmk_panic_local(void)
     }
 }
 
+/*!
+ * \internal
+ * \brief Tell sbd to kill the local host, then exit
+ */
 static void
-pcmk_panic_sbd(void) 
+panic_sbd(void)
 {
     union sigval signal_value;
     pid_t ppid = getppid();
 
-    do_crm_log_always(LOG_EMERG, "Signaling sbd(%d) to panic", sbd_pid);
+    crm_emerg("Signaling sbd[%lld] to panic", (long long) sbd_pid);
 
     memset(&signal_value, 0, sizeof(signal_value));
     /* TODO: Arrange for a slightly less brutal option? */
     if(sigqueue(sbd_pid, SIGKILL, signal_value) < 0) {
-        crm_perror(LOG_EMERG, "Cannot signal SBD(%d) to terminate", sbd_pid);
-        pcmk_panic_local();
+        crm_perror(LOG_EMERG, "Cannot signal sbd[%lld] to terminate",
+                   (long long) sbd_pid);
+        panic_local();
     }
 
     if(ppid > 1) {
@@ -140,42 +149,58 @@ pcmk_panic_sbd(void)
     }
 }
 
+/*!
+ * \internal
+ * \brief Panic the local host
+ *
+ * Panic the local host either by sbd (if running), directly, or by asking
+ * pacemakerd. If trace logging this function, exit instead.
+ *
+ * \param[in] origin   Function caller (for logging only)
+ */
 void
-pcmk_panic(const char *origin) 
+pcmk__panic(const char *origin)
 {
     static struct qb_log_callsite *panic_cs = NULL;
 
     if (panic_cs == NULL) {
-        panic_cs = qb_log_callsite_get(__func__, __FILE__, "panic-delay", LOG_TRACE, __LINE__, crm_trace_nonlog);
+        panic_cs = qb_log_callsite_get(__func__, __FILE__, "panic-delay",
+                                       LOG_TRACE, __LINE__, crm_trace_nonlog);
     }
 
     /* Ensure sbd_pid is set */
-    (void)pcmk_locate_sbd();
+    (void) pcmk__locate_sbd();
 
     if (panic_cs && panic_cs->targets) {
         /* getppid() == 1 means our original parent no longer exists */
-        do_crm_log_always(LOG_EMERG,
-                          "Shutting down instead of panicking the node: origin=%s, sbd=%d, parent=%d",
-                          origin, sbd_pid, getppid());
+        crm_emerg("Shutting down instead of panicking the node "
+                  CRM_XS " origin=%s sbd=%lld parent=%d",
+                  origin, (long long) sbd_pid, getppid());
         crm_exit(CRM_EX_FATAL);
         return;
     }
 
     if(sbd_pid > 1) {
-        do_crm_log_always(LOG_EMERG, "Signaling sbd(%d) to panic the system: %s", sbd_pid, origin);
-        pcmk_panic_sbd();
+        crm_emerg("Signaling sbd[%lld] to panic the system: %s",
+                  (long long) sbd_pid, origin);
+        panic_sbd();
 
     } else {
-        do_crm_log_always(LOG_EMERG, "Panicking the system directly: %s", origin);
-        pcmk_panic_local();
+        crm_emerg("Panicking the system directly: %s", origin);
+        panic_local();
     }
 }
 
+/*!
+ * \internal
+ * \brief Return the process ID of sbd (or 0 if it is not running)
+ */
 pid_t
-pcmk_locate_sbd(void)
+pcmk__locate_sbd(void)
 {
     char *pidfile = NULL;
     char *sbd_path = NULL;
+    int rc;
 
     if(sbd_pid > 1) {
         return sbd_pid;
@@ -186,17 +211,17 @@ pcmk_locate_sbd(void)
     sbd_path = crm_strdup_printf("%s/sbd", SBIN_DIR);
 
     /* Read the pid file */
-    CRM_ASSERT(pidfile);
-
-    sbd_pid = crm_pidfile_inuse(pidfile, 0, sbd_path);
-    if(sbd_pid > 0) {
-        crm_trace("SBD detected at pid=%d (file)", sbd_pid);
+    rc = pcmk__pidfile_matches(pidfile, 0, sbd_path, &sbd_pid);
+    if (rc == pcmk_rc_ok) {
+        crm_trace("SBD detected at pid %lld (via PID file %s)",
+                  (long long) sbd_pid, pidfile);
 
 #if SUPPORT_PROCFS
     } else {
         /* Fall back to /proc for systems that support it */
-        sbd_pid = crm_procfs_pid_of("sbd");
-        crm_trace("SBD detected at pid=%d (proc)", sbd_pid);
+        sbd_pid = pcmk__procfs_pid_of("sbd");
+        crm_trace("SBD detected at pid %lld (via procfs)",
+                  (long long) sbd_pid);
 #endif // SUPPORT_PROCFS
     }
 
@@ -212,7 +237,7 @@ pcmk_locate_sbd(void)
 }
 
 long
-crm_get_sbd_timeout(void)
+pcmk__get_sbd_timeout(void)
 {
     static long sbd_timeout = -2;
 
@@ -222,21 +247,44 @@ crm_get_sbd_timeout(void)
     return sbd_timeout;
 }
 
-long
-crm_auto_watchdog_timeout()
+bool
+pcmk__get_sbd_sync_resource_startup(void)
 {
-    long sbd_timeout = crm_get_sbd_timeout();
+    static int sync_resource_startup = PCMK__SBD_SYNC_DEFAULT;
+    static bool checked_sync_resource_startup = false;
+
+    if (!checked_sync_resource_startup) {
+        const char *sync_env = getenv("SBD_SYNC_RESOURCE_STARTUP");
+
+        if (sync_env == NULL) {
+            crm_trace("Defaulting to %sstart-up synchronization with sbd",
+                      (PCMK__SBD_SYNC_DEFAULT? "" : "no "));
+
+        } else if (crm_str_to_boolean(sync_env, &sync_resource_startup) < 0) {
+            crm_warn("Defaulting to %sstart-up synchronization with sbd "
+                     "because environment value '%s' is invalid",
+                     (PCMK__SBD_SYNC_DEFAULT? "" : "no "), sync_env);
+        }
+        checked_sync_resource_startup = true;
+    }
+    return sync_resource_startup != 0;
+}
+
+long
+pcmk__auto_watchdog_timeout()
+{
+    long sbd_timeout = pcmk__get_sbd_timeout();
 
     return (sbd_timeout <= 0)? 0 : (2 * sbd_timeout);
 }
 
-gboolean
-check_sbd_timeout(const char *value)
+bool
+pcmk__valid_sbd_timeout(const char *value)
 {
     long st_timeout = value? crm_get_msec(value) : 0;
 
     if (st_timeout < 0) {
-        st_timeout = crm_auto_watchdog_timeout();
+        st_timeout = pcmk__auto_watchdog_timeout();
         crm_debug("Using calculated value %ld for stonith-watchdog-timeout (%s)",
                   st_timeout, value);
     }
@@ -245,25 +293,23 @@ check_sbd_timeout(const char *value)
         crm_debug("Watchdog may be enabled but stonith-watchdog-timeout is disabled (%s)",
                   value? value : "default");
 
-    } else if (pcmk_locate_sbd() == 0) {
-        do_crm_log_always(LOG_EMERG,
-                          "Shutting down: stonith-watchdog-timeout configured (%s) but SBD not active",
-                          (value? value : "auto"));
+    } else if (pcmk__locate_sbd() == 0) {
+        crm_emerg("Shutting down: stonith-watchdog-timeout configured (%s) "
+                  "but SBD not active", (value? value : "auto"));
         crm_exit(CRM_EX_FATAL);
-        return FALSE;
+        return false;
 
     } else {
-        long sbd_timeout = crm_get_sbd_timeout();
+        long sbd_timeout = pcmk__get_sbd_timeout();
 
         if (st_timeout < sbd_timeout) {
-            do_crm_log_always(LOG_EMERG,
-                              "Shutting down: stonith-watchdog-timeout (%s) too short (must be >%ldms)",
-                              value, sbd_timeout);
+            crm_emerg("Shutting down: stonith-watchdog-timeout (%s) too short "
+                      "(must be >%ldms)", value, sbd_timeout);
             crm_exit(CRM_EX_FATAL);
-            return FALSE;
+            return false;
         }
         crm_info("Watchdog configured with stonith-watchdog-timeout %s and SBD timeout %ldms",
                  value, sbd_timeout);
     }
-    return TRUE;
+    return true;
 }
