@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2020 the Pacemaker project contributors
+ * Copyright 2004-2021 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -15,6 +15,7 @@
 #include <string.h>
 #include <ctype.h>
 #include <libgen.h>
+#include <inttypes.h>
 
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -31,6 +32,8 @@
 
 #include <crm/common/mainloop.h>
 
+#include "fencing_private.h"
+
 CRM_TRACE_INIT_DATA(stonith);
 
 struct stonith_action_s {
@@ -42,8 +45,8 @@ struct stonith_action_s {
     int timeout;
     int async;
     void *userdata;
-    void (*done_cb) (GPid pid, gint status, const char *output, gpointer user_data);
-    void (*fork_cb) (GPid pid, gpointer user_data);
+    void (*done_cb) (int pid, int status, const char *output, void *user_data);
+    void (*fork_cb) (int pid, void *user_data);
 
     svc_action_t *svc_action;
 
@@ -53,8 +56,7 @@ struct stonith_action_s {
     int remaining_timeout;
     int max_retries;
 
-    /* device output data */
-    GPid pid;
+    int pid;
     int rc;
     char *output;
     char *error;
@@ -194,6 +196,66 @@ stonith_get_namespace(const char *agent, const char *namespace_s)
     return st_namespace_invalid;
 }
 
+gboolean
+stonith__watchdog_fencing_enabled_for_node_api(stonith_t *st, const char *node)
+{
+    gboolean rv = FALSE;
+    stonith_t *stonith_api = st?st:stonith_api_new();
+    char *list = NULL;
+
+    if(stonith_api) {
+        if (stonith_api->state == stonith_disconnected) {
+            int rc = stonith_api->cmds->connect(stonith_api, "stonith-api", NULL);
+
+            if (rc != pcmk_ok) {
+                crm_err("Failed connecting to Stonith-API for watchdog-fencing-query.");
+            }
+        }
+
+        if (stonith_api->state != stonith_disconnected) {
+            /* caveat!!!
+             * this might fail when when stonithd is just updating the device-list
+             * probably something we should fix as well for other api-calls */
+            int rc = stonith_api->cmds->list(stonith_api, st_opt_sync_call, STONITH_WATCHDOG_ID, &list, 0);
+            if ((rc != pcmk_ok) || (list == NULL)) {
+                /* due to the race described above it can happen that
+                 * we drop in here - so as not to make remote nodes
+                 * panic on that answer
+                 */
+                crm_warn("watchdog-fencing-query failed");
+            } else if (list[0] == '\0') {
+                rv = TRUE;
+            } else {
+                GList *targets = stonith__parse_targets(list);
+                rv = pcmk__str_in_list(node, targets, pcmk__str_casei);
+                g_list_free_full(targets, free);
+            }
+            free(list);
+            if (!st) {
+                /* if we're provided the api we still might have done the
+                 * connection - but let's assume the caller won't bother
+                 */
+                stonith_api->cmds->disconnect(stonith_api);
+            }
+        }
+
+        if (!st) {
+            stonith_api_delete(stonith_api);
+        }
+    } else {
+        crm_err("Stonith-API for watchdog-fencing-query couldn't be created.");
+    }
+    crm_trace("Pacemaker assumes node %s %sto do watchdog-fencing.",
+              node, rv?"":"not ");
+    return rv;
+}
+
+gboolean
+stonith__watchdog_fencing_enabled_for_node(const char *node)
+{
+    return stonith__watchdog_fencing_enabled_for_node_api(NULL, node);
+}
+
 static void
 log_action(stonith_action_t *action, pid_t pid)
 {
@@ -456,40 +518,23 @@ stonith_api_register_level(stonith_t * st, int options, const char *node, int le
                                            level, device_list);
 }
 
-static void
-append_arg(const char *key, const char *value, GHashTable **args)
-{
-    CRM_CHECK(key != NULL, return);
-    CRM_CHECK(value != NULL, return);
-    CRM_CHECK(args != NULL, return);
-
-    if (strstr(key, "pcmk_")) {
-        return;
-    } else if (strstr(key, CRM_META)) {
-        return;
-    } else if (pcmk__str_eq(key, "crm_feature_set", pcmk__str_casei)) {
-        return;
-    }
-
-    if (!*args) {
-        *args = crm_str_table_new();
-    }
-
-    CRM_CHECK(*args != NULL, return);
-    crm_trace("Appending: %s=%s", key, value);
-    g_hash_table_replace(*args, strdup(key), strdup(value));
-}
-
 static void
 append_config_arg(gpointer key, gpointer value, gpointer user_data)
 {
-    /* The fencer will filter action out when it registers the device,
-     * but ignore it here just in case any other library callers
-     * fail to do so.
+    /* The fencer will filter "action" out when it registers the device,
+     * but ignore it here in case any external API users don't.
+     *
+     * Also filter out parameters handled directly by Pacemaker.
      */
-    if (!pcmk__str_eq(key, STONITH_ATTR_ACTION_OP, pcmk__str_casei)) {
-        append_arg(key, value, user_data);
-        return;
+    if (!pcmk__str_eq(key, STONITH_ATTR_ACTION_OP, pcmk__str_casei)
+        && !pcmk_stonith_param(key)
+        && (strstr(key, CRM_META) == NULL)
+        && !pcmk__str_eq(key, "crm_feature_set", pcmk__str_casei)) {
+
+        crm_trace("Passing %s=%s with fence action",
+                  (const char *) key, (const char *) (value? value : ""));
+        g_hash_table_insert((GHashTable *) user_data,
+                            strdup(key), strdup(value? value : ""));
     }
 }
 
@@ -498,75 +543,84 @@ make_args(const char *agent, const char *action, const char *victim,
           uint32_t victim_nodeid, GHashTable * device_args,
           GHashTable * port_map, const char *host_arg)
 {
-    char buffer[512];
     GHashTable *arg_list = NULL;
     const char *value = NULL;
 
     CRM_CHECK(action != NULL, return NULL);
 
-    snprintf(buffer, sizeof(buffer), "pcmk_%s_action", action);
+    arg_list = pcmk__strkey_table(free, free);
+
+    // Add action to arguments (using an alias if requested)
     if (device_args) {
+        char buffer[512];
+
+        snprintf(buffer, sizeof(buffer), "pcmk_%s_action", action);
         value = g_hash_table_lookup(device_args, buffer);
-    }
-    if (value) {
-        crm_debug("Substituting action '%s' for requested operation '%s'", value, action);
-        action = value;
-    }
-
-    append_arg(STONITH_ATTR_ACTION_OP, action, &arg_list);
-    if (victim && device_args) {
-        const char *alias = victim;
-        const char *param = g_hash_table_lookup(device_args, STONITH_ATTR_HOSTARG);
-
-        if (port_map && g_hash_table_lookup(port_map, victim)) {
-            alias = g_hash_table_lookup(port_map, victim);
+        if (value) {
+            crm_debug("Substituting '%s' for fence action %s targeting %s",
+                      value, action, victim);
+            action = value;
         }
+    }
+    g_hash_table_insert(arg_list, strdup(STONITH_ATTR_ACTION_OP),
+                        strdup(action));
 
-        /* Always supply the node's name, too:
+    /* If this is a fencing operation against another node, add more standard
+     * arguments.
+     */
+    if (victim && device_args) {
+        const char *param = NULL;
+
+        /* Always pass the target's name, per
          * https://github.com/ClusterLabs/fence-agents/blob/master/doc/FenceAgentAPI.md
          */
-        append_arg("nodename", victim, &arg_list);
+        g_hash_table_insert(arg_list, strdup("nodename"), strdup(victim));
+
+        // If the target's node ID was specified, pass it, too
         if (victim_nodeid) {
-            char nodeid_str[33] = { 0, };
-            if (snprintf(nodeid_str, 33, "%u", (unsigned int)victim_nodeid)) {
-                crm_info("For stonith action (%s) for victim %s, adding nodeid (%s) to parameters",
-                         action, victim, nodeid_str);
-                append_arg("nodeid", nodeid_str, &arg_list);
-            }
+            char *nodeid = crm_strdup_printf("%" PRIu32, victim_nodeid);
+
+            // cts-fencing looks for this log message
+            crm_info("Passing '%s' as nodeid with fence action '%s' targeting %s",
+                     nodeid, action, victim);
+            g_hash_table_insert(arg_list, strdup("nodeid"), nodeid);
         }
 
-        /* Check if we need to supply the victim in any other form */
-        if(pcmk__str_eq(agent, "fence_legacy", pcmk__str_casei)) {
-            value = agent;
+        // Check whether target must be specified in some other way
+        param = g_hash_table_lookup(device_args, PCMK_STONITH_HOST_ARGUMENT);
+        if (!pcmk__str_eq(agent, "fence_legacy", pcmk__str_none)
+            && !pcmk__str_eq(param, "none", pcmk__str_casei)) {
 
-        } else if (param == NULL) {
-            // By default, `port` is added
-            if (host_arg == NULL) {
-                param = "port";
-
-            } else {
-                param = host_arg;
+            if (param == NULL) {
+                /* Use the caller's default for pcmk_host_argument, or "port" if
+                 * none was given
+                 */
+                param = (host_arg == NULL)? "port" : host_arg;
             }
-
             value = g_hash_table_lookup(device_args, param);
 
-        } else if (pcmk__str_eq(param, "none", pcmk__str_casei)) {
-            value = param;      /* Nothing more to do */
+            if (pcmk__str_eq(value, "dynamic",
+                             pcmk__str_casei|pcmk__str_null_matches)) {
+                /* If the host argument was "dynamic" or not explicitly specified,
+                 * add it with the target
+                 */
+                const char *alias = NULL;
 
-        } else {
-            value = g_hash_table_lookup(device_args, param);
-        }
-
-        /* Don't overwrite explictly set values for $param */
-        if (pcmk__str_eq(value, "dynamic", pcmk__str_null_matches | pcmk__str_casei)) {
-            crm_debug("Performing '%s' action targeting '%s' as '%s=%s'", action, victim, param,
-                      alias);
-            append_arg(param, alias, &arg_list);
+                if (port_map) {
+                    alias = g_hash_table_lookup(port_map, victim);
+                }
+                if (alias == NULL) {
+                    alias = victim;
+                }
+                crm_debug("Passing %s='%s' with fence action %s targeting %s",
+                          param, alias, action, victim);
+                g_hash_table_insert(arg_list, strdup(param), strdup(alias));
+            }
         }
     }
 
     if (device_args) {
-        g_hash_table_foreach(device_args, append_config_arg, &arg_list);
+        g_hash_table_foreach(device_args, append_config_arg, arg_list);
     }
 
     return arg_list;
@@ -699,14 +753,14 @@ static int
 svc_action_to_errno(svc_action_t *svc_action) {
     int rv = pcmk_ok;
 
-    if (svc_action->rc > 0) {
-        /* Try to provide a useful error code based on the fence agent's
-            * error output.
-            */
-        if (svc_action->rc == PCMK_OCF_TIMEOUT) {
+    if (svc_action->status == PCMK_EXEC_TIMEOUT) {
             rv = -ETIME;
 
-        } else if (svc_action->stderr_data == NULL) {
+    } else if (svc_action->rc != PCMK_OCF_OK) {
+        /* Try to provide a useful error code based on the fence agent's
+         * error output.
+         */
+        if (svc_action->stderr_data == NULL) {
             rv = -ENODATA;
 
         } else if (strstr(svc_action->stderr_data, "imed out")) {
@@ -781,6 +835,11 @@ internal_stonith_action_execute(stonith_action_t * action)
     static int stonith_sequence = 0;
     char *buffer = NULL;
 
+    if ((action == NULL) || (action->action == NULL) || (action->args == NULL)
+        || (action->agent == NULL)) {
+        return -EPROTO;
+    }
+
     if (!action->tries) {
         action->initial_start_time = time(NULL);
     }
@@ -792,13 +851,16 @@ internal_stonith_action_execute(stonith_action_t * action)
         is_retry = 1;
     }
 
-    if (action->args == NULL || action->agent == NULL)
-        goto fail;
-
     buffer = crm_strdup_printf(PCMK__FENCE_BINDIR "/%s",
                                basename(action->agent));
     svc_action = services_action_create_generic(buffer, NULL);
     free(buffer);
+
+    if (svc_action->rc != PCMK_OCF_UNKNOWN) {
+        services_action_free(svc_action);
+        return -E2BIG;
+    }
+
     svc_action->timeout = 1000 * action->remaining_timeout;
     svc_action->standard = strdup(PCMK_RESOURCE_CLASS_STONITH);
     svc_action->id = crm_strdup_printf("%s_%s_%d", basename(action->agent),
@@ -824,34 +886,27 @@ internal_stonith_action_execute(stonith_action_t * action)
 
     if (action->async) {
         /* async */
-        if(services_action_async_fork_notify(svc_action,
-            &stonith_action_async_done,
-            &stonith_action_async_forked) == FALSE) {
-            services_action_free(svc_action);
-            svc_action = NULL;
-        } else {
-            rc = 0;
+        if (services_action_async_fork_notify(svc_action,
+                                              &stonith_action_async_done,
+                                              &stonith_action_async_forked)) {
+            return pcmk_ok;
         }
 
-    } else {
-        /* sync */
-        if (services_action_sync(svc_action)) {
-            rc = 0;
-            action->rc = svc_action_to_errno(svc_action);
-            action->output = svc_action->stdout_data;
-            svc_action->stdout_data = NULL;
-            action->error = svc_action->stderr_data;
-            svc_action->stderr_data = NULL;
-        } else {
-            action->rc = -ECONNABORTED;
-            rc = action->rc;
-        }
+    } else if (services_action_sync(svc_action)) { // sync success
+        rc = pcmk_ok;
+        action->rc = svc_action_to_errno(svc_action);
+        action->output = svc_action->stdout_data;
+        svc_action->stdout_data = NULL;
+        action->error = svc_action->stderr_data;
+        svc_action->stderr_data = NULL;
 
-        svc_action->params = NULL;
-        services_action_free(svc_action);
+    } else { // sync failure
+        action->rc = -ECONNABORTED;
+        rc = action->rc;
     }
 
-  fail:
+    svc_action->params = NULL;
+    services_action_free(svc_action);
     return rc;
 }
 
@@ -869,9 +924,9 @@ internal_stonith_action_execute(stonith_action_t * action)
 int
 stonith_action_execute_async(stonith_action_t * action,
                              void *userdata,
-                             void (*done) (GPid pid, int rc, const char *output,
-                                           gpointer user_data),
-                             void (*fork_cb) (GPid pid, gpointer user_data))
+                             void (*done) (int pid, int rc, const char *output,
+                                           void *user_data),
+                             void (*fork_cb) (int pid, void *user_data))
 {
     if (!action) {
         return -EINVAL;
@@ -1137,6 +1192,7 @@ stonith_api_history(stonith_t * stonith, int call_options, const char *node,
              op = pcmk__xml_next(op)) {
             stonith_history_t *kvp;
             long long completed;
+            long long completed_nsec = 0L;
 
             kvp = calloc(1, sizeof(stonith_history_t));
             kvp->target = crm_element_value_copy(op, F_STONITH_TARGET);
@@ -1146,6 +1202,8 @@ stonith_api_history(stonith_t * stonith, int call_options, const char *node,
             kvp->client = crm_element_value_copy(op, F_STONITH_CLIENTNAME);
             crm_element_value_ll(op, F_STONITH_DATE, &completed);
             kvp->completed = (time_t) completed;
+            crm_element_value_ll(op, F_STONITH_DATE_NSEC, &completed_nsec);
+            kvp->completed_nsec = completed_nsec;
             crm_element_value_int(op, F_STONITH_STATE, &kvp->state);
 
             if (last) {
@@ -1278,15 +1336,13 @@ stonith_api_del_callback(stonith_t * stonith, int call_id, bool all_callbacks)
     if (all_callbacks) {
         private->op_callback = NULL;
         g_hash_table_destroy(private->stonith_op_callback_table);
-        private->stonith_op_callback_table = g_hash_table_new_full(g_direct_hash, g_direct_equal,
-                                                                   NULL,
-                                                                   stonith_destroy_op_callback);
+        private->stonith_op_callback_table = pcmk__intkey_table(stonith_destroy_op_callback);
 
     } else if (call_id == 0) {
         private->op_callback = NULL;
 
     } else {
-        g_hash_table_remove(private->stonith_op_callback_table, GINT_TO_POINTER(call_id));
+        pcmk__intkey_table_remove(private->stonith_op_callback_table, call_id);
     }
     return pcmk_ok;
 }
@@ -1328,8 +1384,8 @@ stonith_perform_callback(stonith_t * stonith, xmlNode * msg, int call_id, int rc
 
     CRM_CHECK(call_id > 0, crm_log_xml_err(msg, "Bad result"));
 
-    blob = g_hash_table_lookup(private->stonith_op_callback_table, GINT_TO_POINTER(call_id));
-
+    blob = pcmk__intkey_table_lookup(private->stonith_op_callback_table,
+                                     call_id);
     if (blob != NULL) {
         local_blob = *blob;
         blob = NULL;
@@ -1405,7 +1461,8 @@ update_callback_timeout(int call_id, int timeout, stonith_t * st)
     stonith_callback_client_t *callback = NULL;
     stonith_private_t *private = st->st_private;
 
-    callback = g_hash_table_lookup(private->stonith_op_callback_table, GINT_TO_POINTER(call_id));
+    callback = pcmk__intkey_table_lookup(private->stonith_op_callback_table,
+                                         call_id);
     if (!callback || !callback->allow_timeout_updates) {
         return;
     }
@@ -1535,7 +1592,7 @@ stonith_api_signon(stonith_t * stonith, const char *name, int *stonith_fd)
 
             } else {
 #if HAVE_MSGFROMIPC_TIMEOUT
-                stonith->call_timeout = MAX_IPC_DELAY;
+                stonith->call_timeout = PCMK__IPC_TIMEOUT;
 #endif
                 crm_debug("Connection to fencer by %s succeeded (registration token: %s)",
                           display_name, native->token);
@@ -1690,7 +1747,8 @@ stonith_api_add_callback(stonith_t * stonith, int call_id, int timeout, int opti
         set_callback_timeout(blob, stonith, call_id, timeout);
     }
 
-    g_hash_table_insert(private->stonith_op_callback_table, GINT_TO_POINTER(call_id), blob);
+    pcmk__intkey_table_insert(private->stonith_op_callback_table, call_id,
+                              blob);
     crm_trace("Added callback to %s for call %d", callback_name, call_id);
 
     return TRUE;
@@ -2043,18 +2101,15 @@ stonith_api_validate(stonith_t *st, int call_options, const char *rsc_id,
     const char *target = "node1";
     const char *host_arg = NULL;
 
-    GHashTable *params_table = crm_str_table_new();
+    GHashTable *params_table = pcmk__strkey_table(free, free);
 
     // Convert parameter list to a hash table
     for (; params; params = params->next) {
-        if (pcmk__str_eq(params->key, STONITH_ATTR_HOSTARG, pcmk__str_casei)) {
+        if (pcmk__str_eq(params->key, PCMK_STONITH_HOST_ARGUMENT,
+                         pcmk__str_casei)) {
             host_arg = params->value;
         }
-
-        // Strip out Pacemaker-implemented parameters
-        if (!pcmk__starts_with(params->key, "pcmk_")
-                && strcmp(params->key, "provides")
-                && strcmp(params->key, "stonith-timeout")) {
+        if (!pcmk_stonith_param(params->key)) {
             g_hash_table_insert(params_table, strdup(params->key),
                                 strdup(params->value));
         }
@@ -2121,8 +2176,7 @@ stonith_api_new(void)
     }
     new_stonith->st_private = private;
 
-    private->stonith_op_callback_table = g_hash_table_new_full(g_direct_hash, g_direct_equal,
-                                                               NULL, stonith_destroy_op_callback);
+    private->stonith_op_callback_table = pcmk__intkey_table(stonith_destroy_op_callback);
     private->notify_list = NULL;
     private->notify_refcnt = 0;
     private->notify_deletes = FALSE;
@@ -2270,7 +2324,7 @@ stonith_api_kick(uint32_t nodeid, const char *uname, int timeout, bool off)
         api_log(LOG_ERR, "Connection failed, could not kick (%s) node %u/%s : %s (%d)",
                 action, nodeid, uname, pcmk_strerror(rc), rc);
     } else {
-        char *name = (uname == NULL)? crm_itoa(nodeid) : strdup(uname);
+        char *name = (uname == NULL)? pcmk__itoa(nodeid) : strdup(uname);
         int opts = 0;
 
         stonith__set_call_options(opts, name,
@@ -2315,7 +2369,7 @@ stonith_api_time(uint32_t nodeid, const char *uname, bool in_progress)
         int progress = 0;
         int completed = 0;
         int opts = 0;
-        char *name = (uname == NULL)? crm_itoa(nodeid) : strdup(uname);
+        char *name = (uname == NULL)? pcmk__itoa(nodeid) : strdup(uname);
 
         stonith__set_call_options(opts, name, st_opt_sync_call);
         if ((uname == NULL) && (nodeid > 0)) {
@@ -2407,7 +2461,7 @@ stonith_action_str(const char *action)
  * \brief Parse a target name from one line of a target list string
  *
  * \param[in]     line    One line of a target list string
- * \parma[in]     len     String length of line
+ * \param[in]     len     String length of line
  * \param[in,out] output  List to add newly allocated target name to
  */
 static void
@@ -2550,7 +2604,8 @@ stonith__later_succeeded(stonith_history_t *event, stonith_history_t *top_histor
             pcmk__str_eq(event->target, prev_hp->target, pcmk__str_casei) &&
             pcmk__str_eq(event->action, prev_hp->action, pcmk__str_casei) &&
             pcmk__str_eq(event->delegate, prev_hp->delegate, pcmk__str_casei) &&
-            (event->completed < prev_hp->completed)) {
+            ((event->completed < prev_hp->completed) || 
+             ((event->completed == prev_hp->completed) && (event->completed_nsec < prev_hp->completed_nsec)))) {
             ret = TRUE;
             break;
         }
@@ -2576,13 +2631,15 @@ stonith__sort_history(stonith_history_t *history)
         tmp = hp->next;
         if ((hp->state == st_done) || (hp->state == st_failed)) {
             /* sort into new */
-            if ((!new) || (hp->completed > new->completed)) {
+            if ((!new) || (hp->completed > new->completed) || 
+                ((hp->completed == new->completed) && (hp->completed_nsec > new->completed_nsec))) {
                 hp->next = new;
                 new = hp;
             } else {
                 np = new;
                 do {
-                    if ((!np->next) || (hp->completed > np->next->completed)) {
+                    if ((!np->next) || (hp->completed > np->next->completed) ||
+                        ((hp->completed == np->next->completed) && (hp->completed_nsec > np->next->completed_nsec))) {
                         hp->next = np->next;
                         np->next = hp;
                         break;
@@ -2612,6 +2669,26 @@ stonith__sort_history(stonith_history_t *history)
     return new;
 }
 
+/*!
+ * \brief Return string equivalent of an operation state value
+ *
+ * \param[in] state  Fencing operation state value
+ *
+ * \return Human-friendly string equivalent of state
+ */
+const char *
+stonith_op_state_str(enum op_state state)
+{
+    switch (state) {
+        case st_query:      return "querying";
+        case st_exec:       return "executing";
+        case st_done:       return "completed";
+        case st_duplicate:  return "duplicate";
+        case st_failed:     return "failed";
+    }
+    return "unknown";
+}
+
 stonith_history_t *
 stonith__first_matching_event(stonith_history_t *history,
                               bool (*matching_fn)(stonith_history_t *, void *),
@@ -2644,18 +2721,6 @@ stonith__event_state_neq(stonith_history_t *history, void *user_data)
     return history->state != GPOINTER_TO_INT(user_data);
 }
 
-// Deprecated functions kept only for backward API compatibility
-const char *get_stonith_provider(const char *agent, const char *provider);
-
-/*!
- * \brief Deprecated (use stonith_get_namespace() instead)
- */
-const char *
-get_stonith_provider(const char *agent, const char *provider)
-{
-    return stonith_namespace2text(stonith_get_namespace(agent, provider));
-}
-
 void
 stonith__device_parameter_flags(uint32_t *device_flags, const char *device_name,
                                 xmlNode *metadata)
@@ -2697,3 +2762,17 @@ stonith__device_parameter_flags(uint32_t *device_flags, const char *device_name,
 
     freeXpathObject(xpath);
 }
+
+// Deprecated functions kept only for backward API compatibility
+// LCOV_EXCL_START
+
+const char *get_stonith_provider(const char *agent, const char *provider);
+
+const char *
+get_stonith_provider(const char *agent, const char *provider)
+{
+    return stonith_namespace2text(stonith_get_namespace(agent, provider));
+}
+
+// LCOV_EXCL_STOP
+// End deprecated API
