@@ -1,5 +1,5 @@
 /*
- * Copyright 2015-2020 the Pacemaker project contributors
+ * Copyright 2015-2021 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -17,7 +17,6 @@
 #include <crm/services.h>
 #include <crm/common/mainloop.h>
 #include <crm/common/alerts_internal.h>
-#include <crm/common/iso8601_internal.h>
 #include <crm/lrmd_internal.h>
 
 #include <crm/pengine/status.h>
@@ -44,7 +43,7 @@ static lrmd_key_value_t *
 alert_key2param_int(lrmd_key_value_t *head, enum pcmk__alert_keys_e name,
                     int value)
 {
-    char *value_s = crm_itoa(value);
+    char *value_s = pcmk__itoa(value);
 
     head = alert_key2param(head, name, value_s);
     free(value_s);
@@ -136,7 +135,8 @@ exec_alert_list(lrmd_t *lrmd, GList *alert_list, enum pcmk__alert_flags kind,
     char timestamp_usec[7];
 
     params = alert_key2param(params, PCMK__alert_key_kind, kind_s);
-    params = alert_key2param(params, PCMK__alert_key_version, VERSION);
+    params = alert_key2param(params, PCMK__alert_key_version,
+                             PACEMAKER_VERSION);
 
     for (GList *iter = g_list_first(alert_list); iter; iter = g_list_next(iter)) {
         pcmk__alert_t *entry = (pcmk__alert_t *)(iter->data);
@@ -377,7 +377,7 @@ lrmd_send_resource_alert(lrmd_t *lrmd, GList *alert_list,
     /* Reoccurring operations do not set exec_time, so on timeout, set it
      * to the operation timeout since that's closer to the actual value.
      */
-    if (op->op_status == PCMK_LRM_OP_TIMEOUT && op->exec_time == 0) {
+    if ((op->op_status == PCMK_EXEC_TIMEOUT) && (op->exec_time == 0)) {
         params = alert_key2param_int(params, PCMK__alert_key_exec_time,
                                      op->timeout);
     } else {
@@ -385,12 +385,12 @@ lrmd_send_resource_alert(lrmd_t *lrmd, GList *alert_list,
                                      op->exec_time);
     }
 
-    if (op->op_status == PCMK_LRM_OP_DONE) {
+    if (op->op_status == PCMK_EXEC_DONE) {
         params = alert_key2param(params, PCMK__alert_key_desc,
                                  services_ocf_exitcode_str(op->rc));
     } else {
         params = alert_key2param(params, PCMK__alert_key_desc,
-                                 services_lrm_status_str(op->op_status));
+                                 pcmk_exec_status_str(op->op_status));
     }
 
     rc = exec_alert_list(lrmd, alert_list, pcmk__alert_resource, NULL, params);
