@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2020 the Pacemaker project contributors
+ * Copyright 2004-2021 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -9,20 +9,85 @@
 
 #include <crm_internal.h>
 
+#include <stdbool.h>
+
 #include <crm/msg_xml.h>
 
 #include <pacemaker-internal.h>
 
+#include "libpacemaker_private.h"
+
 #define VARIANT_GROUP 1
 #include <lib/pengine/variant.h>
 
+/*!
+ * \internal
+ * \brief Expand a group's colocations to its members
+ *
+ * \param[in,out] rsc  Group resource
+ */
+static void
+expand_group_colocations(pe_resource_t *rsc)
+{
+    group_variant_data_t *group_data = NULL;
+    pe_resource_t *member = NULL;
+    bool any_unmanaged = false;
+
+    get_group_variant_data(group_data, rsc);
+
+    // Treat "group with R" colocations as "first member with R"
+    member = group_data->first_child;
+    member->rsc_cons = g_list_concat(member->rsc_cons, rsc->rsc_cons);
+
+
+    /* The above works for the whole group because each group member is
+     * colocated with the previous one.
+     *
+     * However, there is a special case when a group has a mandatory colocation
+     * with a resource that can't start. In that case,
+     * pcmk__block_colocated_starts() will ensure that dependent resources in
+     * mandatory colocations (i.e. the first member for groups) can't start
+     * either. But if any group member is unmanaged and already started, the
+     * internal group colocations are no longer sufficient to make that apply to
+     * later members.
+     *
+     * To handle that case, add mandatory colocations to each member after the
+     * first.
+     */
+    any_unmanaged = !pcmk_is_set(member->flags, pe_rsc_managed);
+    for (GList *item = rsc->children->next; item != NULL; item = item->next) {
+        member = item->data;
+        if (any_unmanaged) {
+            for (GList *cons_iter = rsc->rsc_cons; cons_iter != NULL;
+                 cons_iter = cons_iter->next) {
+
+                pcmk__colocation_t *constraint = (pcmk__colocation_t *) cons_iter->data;
+
+                if (constraint->score == INFINITY) {
+                    member->rsc_cons = g_list_prepend(member->rsc_cons, constraint);
+                }
+            }
+        } else if (!pcmk_is_set(member->flags, pe_rsc_managed)) {
+            any_unmanaged = true;
+        }
+    }
+
+    rsc->rsc_cons = NULL;
+
+    // Treat "R with group" colocations as "R with last member"
+    member = group_data->last_child;
+    member->rsc_cons_lhs = g_list_concat(member->rsc_cons_lhs,
+                                         rsc->rsc_cons_lhs);
+    rsc->rsc_cons_lhs = NULL;
+}
+
 pe_node_t *
 pcmk__group_allocate(pe_resource_t *rsc, pe_node_t *prefer,
                      pe_working_set_t *data_set)
 {
     pe_node_t *node = NULL;
     pe_node_t *group_node = NULL;
-    GListPtr gIter = NULL;
+    GList *gIter = NULL;
     group_variant_data_t *group_data = NULL;
 
     get_group_variant_data(group_data, rsc);
@@ -44,15 +109,10 @@ pcmk__group_allocate(pe_resource_t *rsc, pe_node_t *prefer,
     pe__set_resource_flags(rsc, pe_rsc_allocating);
     rsc->role = group_data->first_child->role;
 
-    group_data->first_child->rsc_cons =
-        g_list_concat(group_data->first_child->rsc_cons, rsc->rsc_cons);
-    rsc->rsc_cons = NULL;
+    expand_group_colocations(rsc);
 
-    group_data->last_child->rsc_cons_lhs =
-        g_list_concat(group_data->last_child->rsc_cons_lhs, rsc->rsc_cons_lhs);
-    rsc->rsc_cons_lhs = NULL;
-
-    pe__show_node_weights(!show_scores, rsc, __func__, rsc->allowed_nodes);
+    pe__show_node_weights(!pcmk_is_set(data_set->flags, pe_flag_show_scores),
+                          rsc, __func__, rsc->allowed_nodes, data_set);
 
     gIter = rsc->children;
     for (; gIter != NULL; gIter = gIter->next) {
@@ -66,7 +126,8 @@ pcmk__group_allocate(pe_resource_t *rsc, pe_node_t *prefer,
         }
     }
 
-    rsc->next_role = group_data->first_child->next_role;
+    pe__set_next_role(rsc, group_data->first_child->next_role,
+                      "first group member");
     pe__clear_resource_flags(rsc, pe_rsc_allocating|pe_rsc_provisional);
 
     if (group_data->colocated) {
@@ -82,7 +143,7 @@ group_create_actions(pe_resource_t * rsc, pe_working_set_t * data_set)
 {
     pe_action_t *op = NULL;
     const char *value = NULL;
-    GListPtr gIter = rsc->children;
+    GList *gIter = rsc->children;
 
     pe_rsc_trace(rsc, "Creating actions for %s", rsc->id);
 
@@ -126,7 +187,7 @@ group_create_actions(pe_resource_t * rsc, pe_working_set_t * data_set)
 void
 group_update_pseudo_status(pe_resource_t * parent, pe_resource_t * child)
 {
-    GListPtr gIter = child->actions;
+    GList *gIter = child->actions;
     group_variant_data_t *group_data = NULL;
 
     get_group_variant_data(group_data, parent);
@@ -163,7 +224,7 @@ group_update_pseudo_status(pe_resource_t * parent, pe_resource_t * child)
 void
 group_internal_constraints(pe_resource_t * rsc, pe_working_set_t * data_set)
 {
-    GListPtr gIter = rsc->children;
+    GList *gIter = rsc->children;
     pe_resource_t *last_rsc = NULL;
     pe_resource_t *last_active = NULL;
     pe_resource_t *top = uber_parent(rsc);
@@ -171,9 +232,12 @@ group_internal_constraints(pe_resource_t * rsc, pe_working_set_t * data_set)
 
     get_group_variant_data(group_data, rsc);
 
-    new_rsc_order(rsc, RSC_STOPPED, rsc, RSC_START, pe_order_optional, data_set);
-    new_rsc_order(rsc, RSC_START, rsc, RSC_STARTED, pe_order_runnable_left, data_set);
-    new_rsc_order(rsc, RSC_STOP, rsc, RSC_STOPPED, pe_order_runnable_left, data_set);
+    pcmk__order_resource_actions(rsc, RSC_STOPPED, rsc, RSC_START,
+                                 pe_order_optional, data_set);
+    pcmk__order_resource_actions(rsc, RSC_START, rsc, RSC_STARTED,
+                                 pe_order_runnable_left, data_set);
+    pcmk__order_resource_actions(rsc, RSC_STOP, rsc, RSC_STOPPED,
+                                 pe_order_runnable_left, data_set);
 
     for (; gIter != NULL; gIter = gIter->next) {
         pe_resource_t *child_rsc = (pe_resource_t *) gIter->data;
@@ -192,63 +256,70 @@ group_internal_constraints(pe_resource_t * rsc, pe_working_set_t * data_set)
             }
 
         } else if (group_data->colocated) {
-            rsc_colocation_new("group:internal_colocation", NULL, INFINITY,
-                               child_rsc, last_rsc, NULL, NULL, data_set);
+            pcmk__new_colocation("group:internal_colocation", NULL, INFINITY,
+                                 child_rsc, last_rsc, NULL, NULL,
+                                 pcmk_is_set(child_rsc->flags, pe_rsc_critical),
+                                 data_set);
         }
 
         if (pcmk_is_set(top->flags, pe_rsc_promotable)) {
-            new_rsc_order(rsc, RSC_DEMOTE, child_rsc, RSC_DEMOTE,
-                          stop | pe_order_implies_first_printed, data_set);
+            pcmk__order_resource_actions(rsc, RSC_DEMOTE, child_rsc, RSC_DEMOTE,
+                                         stop|pe_order_implies_first_printed,
+                                         data_set);
 
-            new_rsc_order(child_rsc, RSC_DEMOTE, rsc, RSC_DEMOTED, stopped, data_set);
+            pcmk__order_resource_actions(child_rsc, RSC_DEMOTE, rsc,
+                                         RSC_DEMOTED, stopped, data_set);
 
-            new_rsc_order(child_rsc, RSC_PROMOTE, rsc, RSC_PROMOTED, started, data_set);
+            pcmk__order_resource_actions(child_rsc, RSC_PROMOTE, rsc,
+                                         RSC_PROMOTED, started, data_set);
 
-            new_rsc_order(rsc, RSC_PROMOTE, child_rsc, RSC_PROMOTE,
-                          pe_order_implies_first_printed, data_set);
+            pcmk__order_resource_actions(rsc, RSC_PROMOTE, child_rsc,
+                                         RSC_PROMOTE,
+                                         pe_order_implies_first_printed,
+                                         data_set);
 
         }
 
-        order_start_start(rsc, child_rsc, pe_order_implies_first_printed);
-        order_stop_stop(rsc, child_rsc, stop | pe_order_implies_first_printed);
+        pcmk__order_starts(rsc, child_rsc, pe_order_implies_first_printed,
+                           data_set);
+        pcmk__order_stops(rsc, child_rsc,
+                          stop|pe_order_implies_first_printed, data_set);
 
-        new_rsc_order(child_rsc, RSC_STOP, rsc, RSC_STOPPED, stopped, data_set);
-
-        new_rsc_order(child_rsc, RSC_START, rsc, RSC_STARTED, started, data_set);
+        pcmk__order_resource_actions(child_rsc, RSC_STOP, rsc, RSC_STOPPED,
+                                     stopped, data_set);
+        pcmk__order_resource_actions(child_rsc, RSC_START, rsc, RSC_STARTED,
+                                     started, data_set);
 
         if (group_data->ordered == FALSE) {
-            order_start_start(rsc, child_rsc, start | pe_order_implies_first_printed);
+            pcmk__order_starts(rsc, child_rsc,
+                               start|pe_order_implies_first_printed, data_set);
             if (pcmk_is_set(top->flags, pe_rsc_promotable)) {
-                new_rsc_order(rsc, RSC_PROMOTE, child_rsc, RSC_PROMOTE,
-                              start | pe_order_implies_first_printed, data_set);
+                pcmk__order_resource_actions(rsc, RSC_PROMOTE, child_rsc,
+                                             RSC_PROMOTE,
+                                             start|pe_order_implies_first_printed,
+                                             data_set);
             }
 
         } else if (last_rsc != NULL) {
-            child_rsc->restart_type = pe_restart_restart;
-
-            order_start_start(last_rsc, child_rsc, start);
-            order_stop_stop(child_rsc, last_rsc, pe_order_optional | pe_order_restart);
+            pcmk__order_starts(last_rsc, child_rsc, start, data_set);
+            pcmk__order_stops(child_rsc, last_rsc,
+                              pe_order_optional|pe_order_restart, data_set);
 
             if (pcmk_is_set(top->flags, pe_rsc_promotable)) {
-                new_rsc_order(last_rsc, RSC_PROMOTE, child_rsc, RSC_PROMOTE, start, data_set);
-                new_rsc_order(child_rsc, RSC_DEMOTE, last_rsc, RSC_DEMOTE, pe_order_optional,
-                              data_set);
+                pcmk__order_resource_actions(last_rsc, RSC_PROMOTE, child_rsc,
+                                             RSC_PROMOTE, start, data_set);
+                pcmk__order_resource_actions(child_rsc, RSC_DEMOTE, last_rsc,
+                                             RSC_DEMOTE, pe_order_optional,
+                                             data_set);
             }
 
         } else {
-            /* If anyone in the group is starting, then
-             *  pe_order_implies_then will cause _everyone_ in the group
-             *  to be sent a start action
-             * But this is safe since starting something that is already
-             *  started is required to be "safe"
-             */
-            int flags = pe_order_none;
-
-            order_start_start(rsc, child_rsc, flags);
+            pcmk__order_starts(rsc, child_rsc, pe_order_none, data_set);
             if (pcmk_is_set(top->flags, pe_rsc_promotable)) {
-                new_rsc_order(rsc, RSC_PROMOTE, child_rsc, RSC_PROMOTE, flags, data_set);
+                pcmk__order_resource_actions(rsc, RSC_PROMOTE, child_rsc,
+                                             RSC_PROMOTE, pe_order_none,
+                                             data_set);
             }
-
         }
 
         /* Look for partially active groups
@@ -258,7 +329,8 @@ group_internal_constraints(pe_resource_t * rsc, pe_working_set_t * data_set)
             if (group_data->ordered
                 && last_rsc
                 && last_rsc->running_on == NULL && last_active && last_active->running_on) {
-                order_stop_stop(child_rsc, last_active, pe_order_optional);
+                pcmk__order_stops(child_rsc, last_active, pe_order_optional,
+                                  data_set);
             }
             last_active = child_rsc;
         }
@@ -270,92 +342,89 @@ group_internal_constraints(pe_resource_t * rsc, pe_working_set_t * data_set)
         int stop_stop_flags = pe_order_implies_then;
         int stop_stopped_flags = pe_order_optional;
 
-        order_stop_stop(rsc, last_rsc, stop_stop_flags);
-        new_rsc_order(last_rsc, RSC_STOP, rsc, RSC_STOPPED, stop_stopped_flags, data_set);
+        pcmk__order_stops(rsc, last_rsc, stop_stop_flags, data_set);
+        pcmk__order_resource_actions(last_rsc, RSC_STOP, rsc, RSC_STOPPED,
+                                     stop_stopped_flags, data_set);
 
         if (pcmk_is_set(top->flags, pe_rsc_promotable)) {
-            new_rsc_order(rsc, RSC_DEMOTE, last_rsc, RSC_DEMOTE, stop_stop_flags, data_set);
-            new_rsc_order(last_rsc, RSC_DEMOTE, rsc, RSC_DEMOTED, stop_stopped_flags, data_set);
+            pcmk__order_resource_actions(rsc, RSC_DEMOTE, last_rsc, RSC_DEMOTE,
+                                         stop_stop_flags, data_set);
+            pcmk__order_resource_actions(last_rsc, RSC_DEMOTE, rsc, RSC_DEMOTED,
+                                         stop_stopped_flags, data_set);
         }
     }
 }
 
 void
-group_rsc_colocation_lh(pe_resource_t *rsc_lh, pe_resource_t *rsc_rh,
-                        rsc_colocation_t *constraint,
+group_rsc_colocation_lh(pe_resource_t *dependent, pe_resource_t *primary,
+                        pcmk__colocation_t *constraint,
                         pe_working_set_t *data_set)
 {
-    GListPtr gIter = NULL;
+    GList *gIter = NULL;
     group_variant_data_t *group_data = NULL;
 
-    if (rsc_lh == NULL) {
-        pe_err("rsc_lh was NULL for %s", constraint->id);
+    if (dependent == NULL) {
+        pe_err("dependent was NULL for %s", constraint->id);
         return;
 
-    } else if (rsc_rh == NULL) {
-        pe_err("rsc_rh was NULL for %s", constraint->id);
-        return;
-    }
-    if (constraint->score == 0) {
+    } else if (primary == NULL) {
+        pe_err("primary was NULL for %s", constraint->id);
         return;
     }
 
-    gIter = rsc_lh->children;
-    pe_rsc_trace(rsc_lh, "Processing constraints from %s", rsc_lh->id);
+    gIter = dependent->children;
+    pe_rsc_trace(dependent, "Processing constraints from %s", dependent->id);
 
-    get_group_variant_data(group_data, rsc_lh);
+    get_group_variant_data(group_data, dependent);
 
     if (group_data->colocated) {
         group_data->first_child->cmds->rsc_colocation_lh(group_data->first_child,
-                                                         rsc_rh, constraint,
+                                                         primary, constraint,
                                                          data_set);
         return;
 
     } else if (constraint->score >= INFINITY) {
         pcmk__config_err("%s: Cannot perform mandatory colocation "
                          "between non-colocated group and %s",
-                         rsc_lh->id, rsc_rh->id);
+                         dependent->id, primary->id);
         return;
     }
 
     for (; gIter != NULL; gIter = gIter->next) {
         pe_resource_t *child_rsc = (pe_resource_t *) gIter->data;
 
-        child_rsc->cmds->rsc_colocation_lh(child_rsc, rsc_rh, constraint,
+        child_rsc->cmds->rsc_colocation_lh(child_rsc, primary, constraint,
                                            data_set);
     }
 }
 
 void
-group_rsc_colocation_rh(pe_resource_t *rsc_lh, pe_resource_t *rsc_rh,
-                        rsc_colocation_t *constraint,
+group_rsc_colocation_rh(pe_resource_t *dependent, pe_resource_t *primary,
+                        pcmk__colocation_t *constraint,
                         pe_working_set_t *data_set)
 {
-    GListPtr gIter = rsc_rh->children;
+    GList *gIter = primary->children;
     group_variant_data_t *group_data = NULL;
 
-    get_group_variant_data(group_data, rsc_rh);
-    CRM_CHECK(rsc_lh->variant == pe_native, return);
+    get_group_variant_data(group_data, primary);
+    CRM_CHECK(dependent->variant == pe_native, return);
 
-    if (constraint->score == 0) {
-        return;
-    }
-    pe_rsc_trace(rsc_rh, "Processing RH %s of constraint %s (LH is %s)",
-                 rsc_rh->id, constraint->id, rsc_lh->id);
+    pe_rsc_trace(primary, "Processing RH %s of constraint %s (LH is %s)",
+                 primary->id, constraint->id, dependent->id);
 
-    if (pcmk_is_set(rsc_rh->flags, pe_rsc_provisional)) {
+    if (pcmk_is_set(primary->flags, pe_rsc_provisional)) {
         return;
 
     } else if (group_data->colocated && group_data->first_child) {
         if (constraint->score >= INFINITY) {
             /* Ensure RHS is _fully_ up before can start LHS */
-            group_data->last_child->cmds->rsc_colocation_rh(rsc_lh,
+            group_data->last_child->cmds->rsc_colocation_rh(dependent,
                                                             group_data->last_child,
                                                             constraint,
                                                             data_set);
         } else {
             /* A partially active RHS is fine */
-            group_data->first_child->cmds->rsc_colocation_rh(rsc_lh,
+            group_data->first_child->cmds->rsc_colocation_rh(dependent,
                                                              group_data->first_child,
                                                              constraint,
                                                              data_set);
@@ -365,14 +434,14 @@ group_rsc_colocation_rh(pe_resource_t *rsc_lh, pe_resource_t *rsc_rh,
 
     } else if (constraint->score >= INFINITY) {
         pcmk__config_err("%s: Cannot perform mandatory colocation with"
-                         " non-colocated group %s", rsc_lh->id, rsc_rh->id);
+                         " non-colocated group %s", dependent->id, primary->id);
         return;
     }
 
     for (; gIter != NULL; gIter = gIter->next) {
         pe_resource_t *child_rsc = (pe_resource_t *) gIter->data;
 
-        child_rsc->cmds->rsc_colocation_rh(rsc_lh, child_rsc, constraint,
+        child_rsc->cmds->rsc_colocation_rh(dependent, child_rsc, constraint,
                                            data_set);
     }
 }
@@ -380,7 +449,7 @@ group_rsc_colocation_rh(pe_resource_t *rsc_lh, pe_resource_t *rsc_rh,
 enum pe_action_flags
 group_action_flags(pe_action_t * action, pe_node_t * node)
 {
-    GListPtr gIter = NULL;
+    GList *gIter = NULL;
     enum pe_action_flags flags = (pe_action_optional | pe_action_runnable | pe_action_pseudo);
 
     for (gIter = action->rsc->children; gIter != NULL; gIter = gIter->next) {
@@ -427,7 +496,7 @@ group_update_actions(pe_action_t *first, pe_action_t *then, pe_node_t *node,
                      enum pe_action_flags flags, enum pe_action_flags filter,
                      enum pe_ordering type, pe_working_set_t *data_set)
 {
-    GListPtr gIter = then->rsc->children;
+    GList *gIter = then->rsc->children;
     enum pe_graph_flags changed = pe_graph_none;
 
     CRM_ASSERT(then->rsc != NULL);
@@ -451,9 +520,9 @@ group_update_actions(pe_action_t *first, pe_action_t *then, pe_node_t *node,
 void
 group_rsc_location(pe_resource_t *rsc, pe__location_t *constraint)
 {
-    GListPtr gIter = rsc->children;
-    GListPtr saved = constraint->node_list_rh;
-    GListPtr zero = pcmk__copy_node_list(constraint->node_list_rh, true);
+    GList *gIter = rsc->children;
+    GList *saved = constraint->node_list_rh;
+    GList *zero = pcmk__copy_node_list(constraint->node_list_rh, true);
     gboolean reset_scores = TRUE;
     group_variant_data_t *group_data = NULL;
 
@@ -461,7 +530,7 @@ group_rsc_location(pe_resource_t *rsc, pe__location_t *constraint)
 
     pe_rsc_debug(rsc, "Processing rsc_location %s for %s", constraint->id, rsc->id);
 
-    native_rsc_location(rsc, constraint);
+    pcmk__apply_location(constraint, rsc);
 
     for (; gIter != NULL; gIter = gIter->next) {
         pe_resource_t *child_rsc = (pe_resource_t *) gIter->data;
@@ -485,7 +554,7 @@ group_expand(pe_resource_t * rsc, pe_working_set_t * data_set)
     pe_rsc_trace(rsc, "Processing actions from %s", rsc->id);
     native_expand(rsc, data_set);
 
-    for (GListPtr gIter = rsc->children; gIter != NULL; gIter = gIter->next) {
+    for (GList *gIter = rsc->children; gIter != NULL; gIter = gIter->next) {
         pe_resource_t *child_rsc = (pe_resource_t *) gIter->data;
 
         child_rsc->cmds->expand(child_rsc, data_set);
@@ -493,34 +562,32 @@ group_expand(pe_resource_t * rsc, pe_working_set_t * data_set)
 }
 
 GHashTable *
-pcmk__group_merge_weights(pe_resource_t *rsc, const char *rhs,
+pcmk__group_merge_weights(pe_resource_t *rsc, const char *primary_id,
                           GHashTable *nodes, const char *attr, float factor,
                           uint32_t flags)
 {
-    GListPtr gIter = rsc->rsc_cons_lhs;
+    GList *gIter = rsc->rsc_cons_lhs;
     group_variant_data_t *group_data = NULL;
 
     get_group_variant_data(group_data, rsc);
 
     if (pcmk_is_set(rsc->flags, pe_rsc_merging)) {
-        pe_rsc_info(rsc, "Breaking dependency loop with %s at %s", rsc->id, rhs);
+        pe_rsc_info(rsc, "Breaking dependency loop with %s at %s",
+                    rsc->id, primary_id);
         return nodes;
     }
 
     pe__set_resource_flags(rsc, pe_rsc_merging);
 
-    nodes =
-        group_data->first_child->cmds->merge_weights(group_data->first_child, rhs, nodes, attr,
-                                                     factor, flags);
+    nodes = group_data->first_child->cmds->merge_weights(group_data->first_child,
+                                                         primary_id, nodes,
+                                                         attr, factor, flags);
 
     for (; gIter != NULL; gIter = gIter->next) {
-        rsc_colocation_t *constraint = (rsc_colocation_t *) gIter->data;
+        pcmk__colocation_t *constraint = (pcmk__colocation_t *) gIter->data;
 
-        if (constraint->score == 0) {
-            continue;
-        }
-        nodes = pcmk__native_merge_weights(constraint->rsc_lh, rsc->id, nodes,
-                                           constraint->node_attribute,
+        nodes = pcmk__native_merge_weights(constraint->dependent, rsc->id,
+                                           nodes, constraint->node_attribute,
                                            constraint->score / (float) INFINITY,
                                            flags);
     }
@@ -533,3 +600,44 @@ void
 group_append_meta(pe_resource_t * rsc, xmlNode * xml)
 {
 }
+
+// Group implementation of resource_alloc_functions_t:colocated_resources()
+GList *
+pcmk__group_colocated_resources(pe_resource_t *rsc, pe_resource_t *orig_rsc,
+                                GList *colocated_rscs)
+{
+    pe_resource_t *child_rsc = NULL;
+    group_variant_data_t *group_data = NULL;
+
+    get_group_variant_data(group_data, rsc);
+
+    if (orig_rsc == NULL) {
+        orig_rsc = rsc;
+    }
+
+    if (group_data->colocated || pe_rsc_is_clone(rsc->parent)) {
+        /* This group has colocated members and/or is cloned -- either way,
+         * add every child's colocated resources to the list.
+         */
+        for (GList *gIter = rsc->children; gIter != NULL; gIter = gIter->next) {
+            child_rsc = (pe_resource_t *) gIter->data;
+            colocated_rscs = child_rsc->cmds->colocated_resources(child_rsc,
+                                                                  orig_rsc,
+                                                                  colocated_rscs);
+        }
+
+    } else if (group_data->first_child != NULL) {
+        /* This group's members are not colocated, and the group is not cloned,
+         * so just add the first child's colocations to the list.
+         */
+        child_rsc = group_data->first_child;
+        colocated_rscs = child_rsc->cmds->colocated_resources(child_rsc,
+                                                              orig_rsc,
+                                                              colocated_rscs);
+    }
+
+    // Now consider colocations where the group itself is specified
+    colocated_rscs = pcmk__colocated_resources(rsc, orig_rsc, colocated_rscs);
+
+    return colocated_rscs;
+}
