@@ -1,5 +1,5 @@
 /*
- * Copyright 2009-2019 the Pacemaker project contributors
+ * Copyright 2009-2021 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -23,25 +23,19 @@
 #include <crm/cib.h>
 #include <crm/common/util.h>
 #include <crm/common/iso8601.h>
+#include <crm/common/xml_internal.h>
+#include <crm/lrmd_internal.h>
 #include <crm/pengine/status.h>
 #include <pacemaker-internal.h>
 
-static bool fake_quiet = FALSE;
+static pcmk__output_t *out = NULL;
 static cib_t *fake_cib = NULL;
-static GListPtr fake_resource_list = NULL;
-static GListPtr fake_op_fail_list = NULL;
+static GList *fake_resource_list = NULL;
+static GList *fake_op_fail_list = NULL;
 gboolean bringing_nodes_online = FALSE;
 
 #define STATUS_PATH_MAX 512
 
-#define quiet_log(fmt, args...) do {              \
-              if(fake_quiet) {                         \
-                  crm_trace(fmt, ##args);         \
-              } else {                            \
-                  printf(fmt , ##args);           \
-              }                                   \
-    } while(0)
-
 #define NEW_NODE_TEMPLATE "//"XML_CIB_TAG_NODE"[@uname='%s']"
 #define NODE_TEMPLATE "//"XML_CIB_TAG_STATE"[@uname='%s']"
 #define RSC_TEMPLATE "//"XML_CIB_TAG_STATE"[@uname='%s']//"XML_LRM_TAG_RESOURCE"[@id='%s']"
@@ -52,13 +46,9 @@ inject_transient_attr(xmlNode * cib_node, const char *name, const char *value)
 {
     xmlNode *attrs = NULL;
     xmlNode *instance_attrs = NULL;
-    xmlChar *node_path;
     const char *node_uuid = ID(cib_node);
 
-    node_path = xmlGetNodePath(cib_node);
-    quiet_log(" + Injecting attribute %s=%s into %s '%s'\n",
-              name, value, node_path, ID(cib_node));
-    free(node_path);
+    out->message(out, "inject-attr", name, value, cib_node);
 
     attrs = first_named_child(cib_node, XML_TAG_TRANSIENT_NODEATTRS);
     if (attrs == NULL) {
@@ -87,13 +77,13 @@ update_failcounts(xmlNode * cib_node, const char *resource, const char *task,
 
     } else {
         char *name = NULL;
-        char *now = crm_ttoa(time(NULL));
+        char *now = pcmk__ttoa(time(NULL));
 
-        name = crm_failcount_name(resource, task, interval_ms);
+        name = pcmk__failcount_name(resource, task, interval_ms);
         inject_transient_attr(cib_node, name, "value++");
         free(name);
 
-        name = crm_lastfailure_name(resource, task, interval_ms);
+        name = pcmk__lastfailure_name(resource, task, interval_ms);
         inject_transient_attr(cib_node, name, now);
         free(name);
         free(now);
@@ -131,21 +121,15 @@ create_op(xmlNode *cib_resource, const char *task, guint interval_ms,
     lrmd_event_data_t *op = NULL;
     xmlNode *xop = NULL;
 
-    op = calloc(1, sizeof(lrmd_event_data_t));
-
-    op->rsc_id = strdup(ID(cib_resource));
-    op->interval_ms = interval_ms;
-    op->op_type = strdup(task);
-
-    op->rc = outcome;
-    op->op_status = 0;
+    op = lrmd_new_event(ID(cib_resource), task, interval_ms);
+    lrmd__set_result(op, outcome, PCMK_EXEC_DONE, "Simulated action result");
     op->params = NULL;          /* TODO: Fill me in */
     op->t_run = (unsigned int) time(NULL);
     op->t_rcchange = op->t_run;
 
     op->call_id = 0;
-    for (xop = __xml_first_child_element(cib_resource); xop != NULL;
-         xop = __xml_next_element(xop)) {
+    for (xop = pcmk__xe_first_child(cib_resource); xop != NULL;
+         xop = pcmk__xe_next(xop)) {
 
         int tmp = 0;
 
@@ -279,22 +263,19 @@ inject_resource(xmlNode * cib_node, const char *resource, const char *lrm_name,
     /* One day, add query for class, provider, type */
 
     if (rclass == NULL || rtype == NULL) {
-        fprintf(stderr, "Resource %s not found in the status section of %s."
-                "  Please supply the class and type to continue\n", resource, ID(cib_node));
+        out->err(out, "Resource %s not found in the status section of %s."
+                 "  Please supply the class and type to continue", resource, ID(cib_node));
         return NULL;
 
-    } else if (safe_str_neq(rclass, PCMK_RESOURCE_CLASS_OCF)
-               && safe_str_neq(rclass, PCMK_RESOURCE_CLASS_STONITH)
-               && safe_str_neq(rclass, PCMK_RESOURCE_CLASS_SERVICE)
-               && safe_str_neq(rclass, PCMK_RESOURCE_CLASS_UPSTART)
-               && safe_str_neq(rclass, PCMK_RESOURCE_CLASS_SYSTEMD)
-               && safe_str_neq(rclass, PCMK_RESOURCE_CLASS_LSB)) {
-        fprintf(stderr, "Invalid class for %s: %s\n", resource, rclass);
+    } else if (!pcmk__strcase_any_of(rclass, PCMK_RESOURCE_CLASS_OCF, PCMK_RESOURCE_CLASS_STONITH,
+                                     PCMK_RESOURCE_CLASS_SERVICE, PCMK_RESOURCE_CLASS_UPSTART,
+                                     PCMK_RESOURCE_CLASS_SYSTEMD, PCMK_RESOURCE_CLASS_LSB, NULL)) {
+        out->err(out, "Invalid class for %s: %s", resource, rclass);
         return NULL;
 
-    } else if (is_set(pcmk_get_ra_caps(rclass), pcmk_ra_cap_provider)
+    } else if (pcmk_is_set(pcmk_get_ra_caps(rclass), pcmk_ra_cap_provider)
                 && (rprovider == NULL)) {
-        fprintf(stderr, "Please specify the provider for resource %s\n", resource);
+        out->err(out, "Please specify the provider for resource %s", resource);
         return NULL;
     }
 
@@ -359,7 +340,7 @@ find_ticket_state(cib_t * the_cib, const char *ticket_id, xmlNode ** ticket_stat
     crm_log_xml_debug(xml_search, "Match");
     if (xml_has_children(xml_search)) {
         if (ticket_id) {
-            fprintf(stdout, "Multiple ticket_states match ticket_id=%s\n", ticket_id);
+            out->err(out, "Multiple ticket_states match ticket_id=%s", ticket_id);
         }
         *ticket_state_xml = xml_search;
     } else {
@@ -408,13 +389,10 @@ set_ticket_state_attr(const char *ticket_id, const char *attr_name,
 }
 
 void
-modify_configuration(pe_working_set_t * data_set, cib_t *cib,
-                     const char *quorum, const char *watchdog, GListPtr node_up, GListPtr node_down, GListPtr node_fail,
-                     GListPtr op_inject, GListPtr ticket_grant, GListPtr ticket_revoke,
-                     GListPtr ticket_standby, GListPtr ticket_activate)
+modify_configuration(pe_working_set_t * data_set, cib_t *cib, pcmk_injections_t *injections)
 {
     int rc = pcmk_ok;
-    GListPtr gIter = NULL;
+    GList *gIter = NULL;
 
     xmlNode *cib_op = NULL;
     xmlNode *cib_node = NULL;
@@ -422,31 +400,33 @@ modify_configuration(pe_working_set_t * data_set, cib_t *cib,
 
     lrmd_event_data_t *op = NULL;
 
-    if (quorum) {
+    out = data_set->priv;
+
+    out->message(out, "inject-modify-config", injections->quorum, injections->watchdog);
+
+    if (injections->quorum) {
         xmlNode *top = create_xml_node(NULL, XML_TAG_CIB);
 
-        quiet_log(" + Setting quorum: %s\n", quorum);
         /* crm_xml_add(top, XML_ATTR_DC_UUID, dc_uuid);      */
-        crm_xml_add(top, XML_ATTR_HAVE_QUORUM, quorum);
+        crm_xml_add(top, XML_ATTR_HAVE_QUORUM, injections->quorum);
 
         rc = cib->cmds->modify(cib, NULL, top, cib_sync_call | cib_scope_local);
         CRM_ASSERT(rc == pcmk_ok);
     }
 
-    if (watchdog) {
-        quiet_log(" + Setting watchdog: %s\n", watchdog);
-
+    if (injections->watchdog) {
         rc = update_attr_delegate(cib, cib_sync_call | cib_scope_local,
                              XML_CIB_TAG_CRMCONFIG, NULL, NULL, NULL, NULL,
-                             XML_ATTR_HAVE_WATCHDOG, watchdog, FALSE, NULL, NULL);
+                             XML_ATTR_HAVE_WATCHDOG, injections->watchdog, FALSE, NULL, NULL);
 
         CRM_ASSERT(rc == pcmk_ok);
     }
 
-    for (gIter = node_up; gIter != NULL; gIter = gIter->next) {
+    for (gIter = injections->node_up; gIter != NULL; gIter = gIter->next) {
         char *node = (char *)gIter->data;
 
-        quiet_log(" + Bringing node %s online\n", node);
+        out->message(out, "inject-modify-node", "Online", node);
+
         cib_node = modify_node(cib, node, TRUE);
         CRM_ASSERT(cib_node != NULL);
 
@@ -456,11 +436,12 @@ modify_configuration(pe_working_set_t * data_set, cib_t *cib,
         free_xml(cib_node);
     }
 
-    for (gIter = node_down; gIter != NULL; gIter = gIter->next) {
+    for (gIter = injections->node_down; gIter != NULL; gIter = gIter->next) {
         char xpath[STATUS_PATH_MAX];
         char *node = (char *)gIter->data;
 
-        quiet_log(" + Taking node %s offline\n", node);
+        out->message(out, "inject-modify-node", "Offline", node);
+
         cib_node = modify_node(cib, node, FALSE);
         CRM_ASSERT(cib_node != NULL);
 
@@ -480,10 +461,11 @@ modify_configuration(pe_working_set_t * data_set, cib_t *cib,
 
     }
 
-    for (gIter = node_fail; gIter != NULL; gIter = gIter->next) {
+    for (gIter = injections->node_fail; gIter != NULL; gIter = gIter->next) {
         char *node = (char *)gIter->data;
 
-        quiet_log(" + Failing node %s\n", node);
+        out->message(out, "inject-modify-node", "Failing", node);
+
         cib_node = modify_node(cib, node, TRUE);
         crm_xml_add(cib_node, XML_NODE_IN_CLUSTER, XML_BOOLEAN_NO);
         CRM_ASSERT(cib_node != NULL);
@@ -494,51 +476,55 @@ modify_configuration(pe_working_set_t * data_set, cib_t *cib,
         free_xml(cib_node);
     }
 
-    for (gIter = ticket_grant; gIter != NULL; gIter = gIter->next) {
+    for (gIter = injections->ticket_grant; gIter != NULL; gIter = gIter->next) {
         char *ticket_id = (char *)gIter->data;
 
-        quiet_log(" + Granting ticket %s\n", ticket_id);
+        out->message(out, "inject-modify-ticket", "Granting", ticket_id);
+
         rc = set_ticket_state_attr(ticket_id, "granted", "true",
                                    cib, cib_sync_call | cib_scope_local);
 
         CRM_ASSERT(rc == pcmk_ok);
     }
 
-    for (gIter = ticket_revoke; gIter != NULL; gIter = gIter->next) {
+    for (gIter = injections->ticket_revoke; gIter != NULL; gIter = gIter->next) {
         char *ticket_id = (char *)gIter->data;
 
-        quiet_log(" + Revoking ticket %s\n", ticket_id);
+        out->message(out, "inject-modify-ticket", "Revoking", ticket_id);
+
         rc = set_ticket_state_attr(ticket_id, "granted", "false",
                                    cib, cib_sync_call | cib_scope_local);
 
         CRM_ASSERT(rc == pcmk_ok);
     }
 
-    for (gIter = ticket_standby; gIter != NULL; gIter = gIter->next) {
+    for (gIter = injections->ticket_standby; gIter != NULL; gIter = gIter->next) {
         char *ticket_id = (char *)gIter->data;
 
-        quiet_log(" + Making ticket %s standby\n", ticket_id);
+        out->message(out, "inject-modify-ticket", "Standby", ticket_id);
+
         rc = set_ticket_state_attr(ticket_id, "standby", "true",
                                    cib, cib_sync_call | cib_scope_local);
 
         CRM_ASSERT(rc == pcmk_ok);
     }
 
-    for (gIter = ticket_activate; gIter != NULL; gIter = gIter->next) {
+    for (gIter = injections->ticket_activate; gIter != NULL; gIter = gIter->next) {
         char *ticket_id = (char *)gIter->data;
 
-        quiet_log(" + Activating ticket %s\n", ticket_id);
+        out->message(out, "inject-modify-ticket", "Activating", ticket_id);
+
         rc = set_ticket_state_attr(ticket_id, "standby", "false",
                                    cib, cib_sync_call | cib_scope_local);
 
         CRM_ASSERT(rc == pcmk_ok);
     }
 
-    for (gIter = op_inject; gIter != NULL; gIter = gIter->next) {
+    for (gIter = injections->op_inject; gIter != NULL; gIter = gIter->next) {
         char *spec = (char *)gIter->data;
 
         int rc = 0;
-        int outcome = 0;
+        int outcome = PCMK_OCF_OK;
         guint interval_ms = 0;
 
         char *key = NULL;
@@ -550,15 +536,15 @@ modify_configuration(pe_working_set_t * data_set, cib_t *cib,
         const char *rclass = NULL;
         const char *rprovider = NULL;
 
-        resource_t *rsc = NULL;
+        pe_resource_t *rsc = NULL;
 
-        quiet_log(" + Injecting %s into the configuration\n", spec);
+        out->message(out, "inject-spec", spec);
 
         key = calloc(1, strlen(spec) + 1);
         node = calloc(1, strlen(spec) + 1);
         rc = sscanf(spec, "%[^@]@%[^=]=%d", key, node, &outcome);
         if (rc != 3) {
-            fprintf(stderr, "Invalid operation spec: %s.  Only found %d fields\n", spec, rc);
+            out->err(out, "Invalid operation spec: %s.  Only found %d fields", spec, rc);
             free(key);
             free(node);
             continue;
@@ -568,7 +554,7 @@ modify_configuration(pe_working_set_t * data_set, cib_t *cib,
 
         rsc = pe_find_resource(data_set->resources, resource);
         if (rsc == NULL) {
-            fprintf(stderr, " - Invalid resource name: %s\n", resource);
+            out->err(out, "Invalid resource name: %s", resource);
         } else {
             rclass = crm_element_value(rsc->xml, XML_AGENT_ATTR_CLASS);
             rtype = crm_element_value(rsc->xml, XML_ATTR_TYPE);
@@ -598,6 +584,10 @@ modify_configuration(pe_working_set_t * data_set, cib_t *cib,
         free(node);
         free(key);
     }
+
+    if (!out->is_quiet(out)) {
+        out->end_list(out);
+    }
 }
 
 static gboolean
@@ -606,10 +596,10 @@ exec_pseudo_action(crm_graph_t * graph, crm_action_t * action)
     const char *node = crm_element_value(action->xml, XML_LRM_ATTR_TARGET);
     const char *task = crm_element_value(action->xml, XML_LRM_ATTR_TASK_KEY);
 
-    action->confirmed = TRUE;
+    crm__set_graph_action_flags(action, pcmk__graph_action_confirmed);
+    out->message(out, "inject-pseudo-action", node, task);
 
-    quiet_log(" * Pseudo action:   %s%s%s\n", task, node ? " on " : "", node ? node : "");
-    update_graph(graph, action);
+    pcmk__update_graph(graph, action);
     return TRUE;
 }
 
@@ -617,9 +607,9 @@ static gboolean
 exec_rsc_action(crm_graph_t * graph, crm_action_t * action)
 {
     int rc = 0;
-    GListPtr gIter = NULL;
+    GList *gIter = NULL;
     lrmd_event_data_t *op = NULL;
-    int target_outcome = 0;
+    int target_outcome = PCMK_OCF_OK;
 
     const char *rtype = NULL;
     const char *rclass = NULL;
@@ -637,8 +627,7 @@ exec_rsc_action(crm_graph_t * graph, crm_action_t * action)
     char *uuid = crm_element_value_copy(action->xml, XML_LRM_ATTR_TARGET_UUID);
     const char *router_node = crm_element_value(action->xml, XML_LRM_ATTR_ROUTER_NODE);
 
-    if (safe_str_eq(operation, CRM_OP_PROBED)
-        || safe_str_eq(operation, CRM_OP_REPROBE)) {
+    if (pcmk__strcase_any_of(operation, CRM_OP_PROBED, CRM_OP_REPROBE, NULL)) {
         crm_info("Skipping %s op for %s", operation, node);
         goto done;
     }
@@ -664,8 +653,8 @@ exec_rsc_action(crm_graph_t * graph, crm_action_t * action)
         }
     }
 
-    if (safe_str_eq(operation, "delete") || safe_str_eq(operation, RSC_METADATA)) {
-        quiet_log(" * Resource action: %-15s %s on %s\n", resource, operation, node);
+    if (pcmk__strcase_any_of(operation, "delete", RSC_METADATA, NULL)) {
+        out->message(out, "inject-rsc-action", resource, operation, node, (guint) 0);
         goto done;
     }
 
@@ -673,9 +662,7 @@ exec_rsc_action(crm_graph_t * graph, crm_action_t * action)
     rtype = crm_element_value(action_rsc, XML_ATTR_TYPE);
     rprovider = crm_element_value(action_rsc, XML_AGENT_ATTR_PROVIDER);
 
-    if (target_rc_s != NULL) {
-        target_outcome = crm_parse_int(target_rc_s, "0");
-    }
+    pcmk__scan_min_int(target_rc_s, &target_outcome, 0);
 
     CRM_ASSERT(fake_cib->cmds->query(fake_cib, NULL, NULL, cib_sync_call | cib_scope_local) ==
                pcmk_ok);
@@ -692,13 +679,10 @@ exec_rsc_action(crm_graph_t * graph, crm_action_t * action)
         return FALSE;
     }
 
-    op = convert_graph_action(cib_resource, action, 0, target_outcome);
-    if (op->interval_ms) {
-        quiet_log(" * Resource action: %-15s %s=%u on %s\n",
-                  resource, op->op_type, op->interval_ms, node);
-    } else {
-        quiet_log(" * Resource action: %-15s %s on %s\n", resource, op->op_type, node);
-    }
+    op = pcmk__event_from_graph_action(cib_resource, action, PCMK_EXEC_DONE,
+                                       target_outcome, "User-injected result");
+
+    out->message(out, "inject-rsc-action", resource, op->op_type, node, op->interval_ms);
 
     for (gIter = fake_op_fail_list; gIter != NULL; gIter = gIter->next) {
         char *spec = (char *)gIter->data;
@@ -706,7 +690,7 @@ exec_rsc_action(crm_graph_t * graph, crm_action_t * action)
         const char *match_name = NULL;
 
         // Allow user to specify anonymous clone with or without instance number
-        key = crm_strdup_printf(CRM_OP_FMT "@%s=", resource, op->op_type,
+        key = crm_strdup_printf(PCMK__OP_FMT "@%s=", resource, op->op_type,
                                 op->interval_ms, node);
         if (strncasecmp(key, spec, strlen(key)) == 0) {
             match_name = resource;
@@ -714,7 +698,7 @@ exec_rsc_action(crm_graph_t * graph, crm_action_t * action)
         free(key);
 
         if ((match_name == NULL) && strcmp(resource, lrm_name)) {
-            key = crm_strdup_printf(CRM_OP_FMT "@%s=", lrm_name, op->op_type,
+            key = crm_strdup_printf(PCMK__OP_FMT "@%s=", lrm_name, op->op_type,
                                     op->interval_ms, node);
             if (strncasecmp(key, spec, strlen(key)) == 0) {
                 match_name = lrm_name;
@@ -728,14 +712,14 @@ exec_rsc_action(crm_graph_t * graph, crm_action_t * action)
             // ${match_name}_${task}_${interval_in_ms}@${node}=${rc}
 
             if (rc != 1) {
-                fprintf(stderr,
-                        "Invalid failed operation spec: %s. Result code must be integer\n",
-                        spec);
+                out->err(out,
+                         "Invalid failed operation spec: %s. Result code must be integer",
+                         spec);
                 continue;
             }
-            action->failed = TRUE;
+            crm__set_graph_action_flags(action, pcmk__graph_action_failed);
             graph->abort_priority = INFINITY;
-            printf("\tPretending action %d failed with rc=%d\n", action->id, op->rc);
+            out->info(out, "Pretending action %d failed with rc=%d", action->id, op->rc);
             update_failcounts(cib_node, match_name, op->op_type,
                               op->interval_ms, op->rc);
             break;
@@ -752,8 +736,8 @@ exec_rsc_action(crm_graph_t * graph, crm_action_t * action)
   done:
     free(node); free(uuid);
     free_xml(cib_node);
-    action->confirmed = TRUE;
-    update_graph(graph, action);
+    crm__set_graph_action_flags(action, pcmk__graph_action_confirmed);
+    pcmk__update_graph(graph, action);
     return TRUE;
 }
 
@@ -764,14 +748,9 @@ exec_crmd_action(crm_graph_t * graph, crm_action_t * action)
     const char *task = crm_element_value(action->xml, XML_LRM_ATTR_TASK);
     xmlNode *rsc = first_named_child(action->xml, XML_CIB_TAG_RESOURCE);
 
-    action->confirmed = TRUE;
-
-    if(rsc) {
-        quiet_log(" * Cluster action:  %s for %s on %s\n", task, ID(rsc), node);
-    } else {
-        quiet_log(" * Cluster action:  %s on %s\n", task, node);
-    }
-    update_graph(graph, action);
+    crm__set_graph_action_flags(action, pcmk__graph_action_confirmed);
+    out->message(out, "inject-cluster-action", node, task, rsc);
+    pcmk__update_graph(graph, action);
     return TRUE;
 }
 
@@ -781,13 +760,14 @@ exec_stonith_action(crm_graph_t * graph, crm_action_t * action)
     const char *op = crm_meta_value(action->params, "stonith_action");
     char *target = crm_element_value_copy(action->xml, XML_LRM_ATTR_TARGET);
 
-    quiet_log(" * Fencing %s (%s)\n", target, op);
-    if(safe_str_neq(op, "on")) {
+    out->message(out, "inject-fencing-action", target, op);
+
+    if(!pcmk__str_eq(op, "on", pcmk__str_casei)) {
         int rc = 0;
         char xpath[STATUS_PATH_MAX];
         xmlNode *cib_node = modify_node(fake_cib, target, FALSE);
 
-        crm_xml_add(cib_node, XML_ATTR_ORIGIN, __FUNCTION__);
+        crm_xml_add(cib_node, XML_ATTR_ORIGIN, __func__);
         CRM_ASSERT(cib_node != NULL);
 
         rc = fake_cib->cmds->replace(fake_cib, XML_CIB_TAG_STATUS, cib_node,
@@ -806,17 +786,17 @@ exec_stonith_action(crm_graph_t * graph, crm_action_t * action)
         free_xml(cib_node);
     }
 
-    action->confirmed = TRUE;
-    update_graph(graph, action);
+    crm__set_graph_action_flags(action, pcmk__graph_action_confirmed);
+    pcmk__update_graph(graph, action);
     free(target);
     return TRUE;
 }
 
-int
-run_simulation(pe_working_set_t * data_set, cib_t *cib, GListPtr op_fail_list, bool quiet)
+enum transition_status
+run_simulation(pe_working_set_t * data_set, cib_t *cib, GList *op_fail_list)
 {
     crm_graph_t *transition = NULL;
-    enum transition_status graph_rc = -1;
+    enum transition_status graph_rc;
 
     crm_graph_functions_t exec_fns = {
         exec_pseudo_action,
@@ -825,43 +805,46 @@ run_simulation(pe_working_set_t * data_set, cib_t *cib, GListPtr op_fail_list, b
         exec_stonith_action,
     };
 
+    out = data_set->priv;
+
     fake_cib = cib;
-    fake_quiet = quiet;
     fake_op_fail_list = op_fail_list;
 
-    quiet_log("\nExecuting cluster transition:\n");
+    if (!out->is_quiet(out)) {
+        out->begin_list(out, NULL, NULL, "Executing Cluster Transition");
+    }
 
-    set_graph_functions(&exec_fns);
-    transition = unpack_graph(data_set->graph, crm_system_name);
-    print_graph(LOG_DEBUG, transition);
+    pcmk__set_graph_functions(&exec_fns);
+    transition = pcmk__unpack_graph(data_set->graph, crm_system_name);
+    pcmk__log_graph(LOG_DEBUG, transition);
 
     fake_resource_list = data_set->resources;
     do {
-        graph_rc = run_graph(transition);
+        graph_rc = pcmk__execute_graph(transition);
 
     } while (graph_rc == transition_active);
     fake_resource_list = NULL;
 
     if (graph_rc != transition_complete) {
-        fprintf(stdout, "Transition failed: %s\n", transition_status(graph_rc));
-        print_graph(LOG_ERR, transition);
+        out->err(out, "Transition failed: %s",
+                 pcmk__graph_status2text(graph_rc));
+        pcmk__log_graph(LOG_ERR, transition);
     }
-    destroy_graph(transition);
+    pcmk__free_graph(transition);
     if (graph_rc != transition_complete) {
-        fprintf(stdout, "An invalid transition was produced\n");
+        out->err(out, "An invalid transition was produced");
     }
 
-    if (quiet == FALSE) {
+    if (!out->is_quiet(out)) {
         xmlNode *cib_object = NULL;
         int rc = fake_cib->cmds->query(fake_cib, NULL, &cib_object, cib_sync_call | cib_scope_local);
 
         CRM_ASSERT(rc == pcmk_ok);
         pe_reset_working_set(data_set);
         data_set->input = cib_object;
+
+        out->end_list(out);
     }
 
-    if (graph_rc != transition_complete) {
-        return graph_rc;
-    }
-    return 0;
+    return graph_rc;
 }
