@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2020 the Pacemaker project contributors
+ * Copyright 2004-2021 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -11,55 +11,10 @@
 #include <crm/msg_xml.h>
 #include <crm/lrmd.h>       // lrmd_event_data_t
 #include <crm/common/xml_internal.h>
+#include <crm/lrmd_internal.h>
 #include <pacemaker-internal.h>
-
-pe__location_t *
-rsc2node_new(const char *id, pe_resource_t *rsc,
-             int node_weight, const char *discover_mode,
-             pe_node_t *foo_node, pe_working_set_t *data_set)
-{
-    pe__location_t *new_con = NULL;
-
-    if (rsc == NULL || id == NULL) {
-        pe_err("Invalid constraint %s for rsc=%p", crm_str(id), rsc);
-        return NULL;
-
-    } else if (foo_node == NULL) {
-        CRM_CHECK(node_weight == 0, return NULL);
-    }
-
-    new_con = calloc(1, sizeof(pe__location_t));
-    if (new_con != NULL) {
-        new_con->id = strdup(id);
-        new_con->rsc_lh = rsc;
-        new_con->node_list_rh = NULL;
-        new_con->role_filter = RSC_ROLE_UNKNOWN;
-
-
-        if (pcmk__str_eq(discover_mode, "always", pcmk__str_null_matches | pcmk__str_casei)) {
-            new_con->discover_mode = pe_discover_always;
-        } else if (pcmk__str_eq(discover_mode, "never", pcmk__str_casei)) {
-            new_con->discover_mode = pe_discover_never;
-        } else if (pcmk__str_eq(discover_mode, "exclusive", pcmk__str_casei)) {
-            new_con->discover_mode = pe_discover_exclusive;
-            rsc->exclusive_discover = TRUE;
-        } else {
-            pe_err("Invalid %s value %s in location constraint", XML_LOCATION_ATTR_DISCOVERY, discover_mode);
-        }
-
-        if (foo_node != NULL) {
-            pe_node_t *copy = pe__copy_node(foo_node);
-
-            copy->weight = node_weight;
-            new_con->node_list_rh = g_list_prepend(NULL, copy);
-        }
-
-        data_set->placement_constraints = g_list_prepend(data_set->placement_constraints, new_con);
-        rsc->rsc_location = g_list_prepend(rsc->rsc_location, new_con);
-    }
-
-    return new_con;
-}
+#include <pacemaker.h>
+#include "libpacemaker_private.h"
 
 gboolean
 can_run_resources(const pe_node_t * node)
@@ -102,7 +57,7 @@ pcmk__copy_node_table(GHashTable *nodes)
     if (nodes == NULL) {
         return NULL;
     }
-    new_table = g_hash_table_new_full(crm_str_hash, g_str_equal, NULL, free);
+    new_table = pcmk__strkey_table(NULL, free);
     g_hash_table_iter_init(&iter, nodes);
     while (g_hash_table_iter_next(&iter, NULL, (gpointer *) &node)) {
         pe_node_t *new_node = pe__copy_node(node);
@@ -268,8 +223,10 @@ native_deallocate(pe_resource_t * rsc)
 }
 
 gboolean
-native_assign_node(pe_resource_t * rsc, GListPtr nodes, pe_node_t * chosen, gboolean force)
+native_assign_node(pe_resource_t *rsc, pe_node_t *chosen, gboolean force)
 {
+    pcmk__output_t *out = rsc->cluster->priv;
+
     CRM_ASSERT(rsc->variant == pe_native);
 
     if (force == FALSE && chosen != NULL) {
@@ -287,7 +244,7 @@ native_assign_node(pe_resource_t * rsc, GListPtr nodes, pe_node_t * chosen, gboo
             crm_debug("All nodes for resource %s are unavailable"
                       ", unclean or shutting down (%s: %d, %d)",
                       rsc->id, chosen->details->uname, can_run_resources(chosen), chosen->weight);
-            rsc->next_role = RSC_ROLE_STOPPED;
+            pe__set_next_role(rsc, RSC_ROLE_STOPPED, "node availability");
             chosen = NULL;
         }
     }
@@ -300,11 +257,11 @@ native_assign_node(pe_resource_t * rsc, GListPtr nodes, pe_node_t * chosen, gboo
     pe__clear_resource_flags(rsc, pe_rsc_provisional);
 
     if (chosen == NULL) {
-        GListPtr gIter = NULL;
-        char *rc_inactive = crm_itoa(PCMK_OCF_NOT_RUNNING);
+        GList *gIter = NULL;
+        char *rc_inactive = pcmk__itoa(PCMK_OCF_NOT_RUNNING);
 
         crm_debug("Could not allocate a node for %s", rsc->id);
-        rsc->next_role = RSC_ROLE_STOPPED;
+        pe__set_next_role(rsc, RSC_ROLE_STOPPED, "unable to allocate");
 
         for (gIter = rsc->actions; gIter != NULL; gIter = gIter->next) {
             pe_action_t *op = (pe_action_t *) gIter->data;
@@ -312,12 +269,10 @@ native_assign_node(pe_resource_t * rsc, GListPtr nodes, pe_node_t * chosen, gboo
 
             crm_debug("Processing %s", op->uuid);
             if(pcmk__str_eq(RSC_STOP, op->task, pcmk__str_casei)) {
-                update_action_flags(op, pe_action_optional | pe_action_clear,
-                                    __func__, __LINE__);
+                pe__clear_action_flags(op, pe_action_optional);
 
             } else if(pcmk__str_eq(RSC_START, op->task, pcmk__str_casei)) {
-                update_action_flags(op, pe_action_runnable | pe_action_clear,
-                                    __func__, __LINE__);
+                pe__clear_action_flags(op, pe_action_runnable);
                 //pe__set_resource_flags(rsc, pe_rsc_block);
 
             } else if (interval_ms_s && !pcmk__str_eq(interval_ms_s, "0", pcmk__str_casei)) {
@@ -326,8 +281,7 @@ native_assign_node(pe_resource_t * rsc, GListPtr nodes, pe_node_t * chosen, gboo
 
                 } else {
                     /* Normal monitor operation, cancel it */
-                    update_action_flags(op, pe_action_runnable | pe_action_clear,
-                                        __func__, __LINE__);
+                    pe__clear_action_flags(op, pe_action_runnable);
                 }
             }
         }
@@ -343,8 +297,10 @@ native_assign_node(pe_resource_t * rsc, GListPtr nodes, pe_node_t * chosen, gboo
     chosen->details->num_resources++;
     chosen->count++;
     calculate_utilization(chosen->details->utilization, rsc->utilization, FALSE);
-    dump_rsc_utilization((show_utilization? LOG_STDOUT : LOG_TRACE),
-                         __func__, rsc, chosen);
+
+    if (pcmk_is_set(rsc->cluster->flags, pe_flag_show_utilization)) {
+        out->message(out, "resource-util", rsc, chosen, __func__);
+    }
 
     return TRUE;
 }
@@ -419,7 +375,7 @@ log_action(unsigned int log_level, const char *pre_text, pe_action_t * action, g
     }
 
     if (details) {
-        GListPtr gIter = NULL;
+        GList *gIter = NULL;
 
         crm_trace("\t\t====== Preceding Actions");
 
@@ -472,10 +428,10 @@ create_pseudo_resource_op(pe_resource_t * rsc, const char *task, bool optional,
 {
     pe_action_t *action = custom_action(rsc, pcmk__op_key(rsc->id, task, 0),
                                         task, NULL, optional, TRUE, data_set);
-    update_action_flags(action, pe_action_pseudo, __func__, __LINE__);
-    update_action_flags(action, pe_action_runnable, __func__, __LINE__);
+
+    pe__set_action_flags(action, pe_action_pseudo);
     if(runnable) {
-        update_action_flags(action, pe_action_runnable, __func__, __LINE__);
+        pe__set_action_flags(action, pe_action_runnable);
     }
     return action;
 }
@@ -536,8 +492,7 @@ sched_shutdown_op(pe_node_t *node, pe_working_set_t *data_set)
     pe_action_t *shutdown_op = custom_action(NULL, shutdown_id, CRM_OP_SHUTDOWN,
                                              node, FALSE, TRUE, data_set);
 
-    crm_notice("Scheduling shutdown of node %s", node->details->uname);
-    shutdown_constraints(node, shutdown_op, data_set);
+    pcmk__order_stops_before_shutdown(node, shutdown_op, data_set);
     add_hash_param(shutdown_op->meta, XML_ATTR_TE_NOWAIT, XML_BOOLEAN_TRUE);
     return shutdown_op;
 }
@@ -618,18 +573,25 @@ pcmk__create_history_xml(xmlNode *parent, lrmd_event_data_t *op,
 
     CRM_CHECK(op != NULL, return NULL);
     do_crm_log(level, "%s: Updating resource %s after %s op %s (interval=%u)",
-               origin, op->rsc_id, op->op_type, services_lrm_status_str(op->op_status),
-               op->interval_ms);
+               origin, op->rsc_id, op->op_type,
+               pcmk_exec_status_str(op->op_status), op->interval_ms);
 
     crm_trace("DC version: %s", caller_version);
 
     task = op->op_type;
 
-    /* Record a successful reload as a start, and a failed reload as a monitor,
-     * to make life easier for the scheduler when determining the current state.
+    /* Record a successful agent reload as a start, and a failed one as a
+     * monitor, to make life easier for the scheduler when determining the
+     * current state.
+     *
+     * @COMPAT We should check "reload" here only if the operation was for a
+     * pre-OCF-1.1 resource agent, but we don't know that here, and we should
+     * only ever get results for actions scheduled by us, so we can reasonably
+     * assume any "reload" is actually a pre-1.1 agent reload.
      */
-    if (pcmk__str_eq(task, "reload", pcmk__str_none)) {
-        if (op->op_status == PCMK_LRM_OP_DONE) {
+    if (pcmk__str_any_of(task, CRMD_ACTION_RELOAD, CRMD_ACTION_RELOAD_AGENT,
+                         NULL)) {
+        if (op->op_status == PCMK_EXEC_DONE) {
             task = CRMD_ACTION_START;
         } else {
             task = CRMD_ACTION_STATUS;
@@ -645,14 +607,13 @@ pcmk__create_history_xml(xmlNode *parent, lrmd_event_data_t *op,
         CRM_LOG_ASSERT(n_task != NULL);
         op_id = pcmk__notify_key(op->rsc_id, n_type, n_task);
 
-        if (op->op_status != PCMK_LRM_OP_PENDING) {
+        if (op->op_status != PCMK_EXEC_PENDING) {
             /* Ignore notify errors.
              *
              * @TODO It might be better to keep the correct result here, and
              * ignore it in process_graph_event().
              */
-            op->op_status = PCMK_LRM_OP_DONE;
-            op->rc = 0;
+            lrmd__set_result(op, PCMK_OCF_OK, PCMK_EXEC_DONE, NULL);
         }
 
     } else if (did_rsc_op_fail(op, target_rc)) {
@@ -711,21 +672,11 @@ pcmk__create_history_xml(xmlNode *parent, lrmd_event_data_t *op,
                       op->rsc_id, op->op_type, op->interval_ms,
                       op->t_run, op->t_rcchange, op->exec_time, op->queue_time);
 
-            if (op->interval_ms == 0) {
-                crm_xml_add_ll(xml_op, XML_RSC_OP_LAST_CHANGE,
-                               (long long) op->t_run);
-
-                // @COMPAT last-run is deprecated
-                crm_xml_add_ll(xml_op, XML_RSC_OP_LAST_RUN,
-                               (long long) op->t_run);
-
-            } else if(op->t_rcchange) {
-                /* last-run is not accurate for recurring ops */
+            if ((op->interval_ms != 0) && (op->t_rcchange != 0)) {
+                // Recurring ops may have changed rc after initial run
                 crm_xml_add_ll(xml_op, XML_RSC_OP_LAST_CHANGE,
                                (long long) op->t_rcchange);
-
             } else {
-                /* ...but is better than nothing otherwise */
                 crm_xml_add_ll(xml_op, XML_RSC_OP_LAST_CHANGE,
                                (long long) op->t_run);
             }
@@ -765,3 +716,113 @@ pcmk__create_history_xml(xmlNode *parent, lrmd_event_data_t *op,
     free(key);
     return xml_op;
 }
+
+pcmk__output_t *
+pcmk__new_logger(void)
+{
+    int rc = pcmk_rc_ok;
+    pcmk__output_t *out = NULL;
+    const char* argv[] = { "", NULL };
+    pcmk__supported_format_t formats[] = {
+        PCMK__SUPPORTED_FORMAT_LOG,
+        { NULL, NULL, NULL }
+    };
+
+    pcmk__register_formats(NULL, formats);
+    rc = pcmk__output_new(&out, "log", NULL, (char**)argv);
+    if ((rc != pcmk_rc_ok) || (out == NULL)) {
+        crm_err("Can't log resource details due to internal error: %s\n",
+                pcmk_rc_str(rc));
+        return NULL;
+    }
+
+    pe__register_messages(out);
+    pcmk__register_lib_messages(out);
+    return out;
+}
+
+/*!
+ * \internal
+ * \brief Check whether a resource has reached its migration threshold on a node
+ *
+ * \param[in]  rsc       Resource to check
+ * \param[in]  node      Node to check
+ * \param[in]  data_set  Cluster working set
+ * \param[out] failed    If the threshold has been reached, this will be set to
+ *                       the resource that failed (possibly a parent of \p rsc)
+ *
+ * \return true if the migration threshold has been reached, false otherwise
+ */
+bool
+pcmk__threshold_reached(pe_resource_t *rsc, pe_node_t *node,
+                        pe_working_set_t *data_set, pe_resource_t **failed)
+{
+    int fail_count, remaining_tries;
+    pe_resource_t *rsc_to_ban = rsc;
+
+    // Migration threshold of 0 means never force away
+    if (rsc->migration_threshold == 0) {
+        return false;
+    }
+
+    // If we're ignoring failures, also ignore the migration threshold
+    if (pcmk_is_set(rsc->flags, pe_rsc_failure_ignored)) {
+        return false;
+    }
+
+    // If there are no failures, there's no need to force away
+    fail_count = pe_get_failcount(node, rsc, NULL,
+                                  pe_fc_effective|pe_fc_fillers, NULL,
+                                  data_set);
+    if (fail_count <= 0) {
+        return false;
+    }
+
+    // If failed resource is anonymous clone instance, we'll force clone away
+    if (!pcmk_is_set(rsc->flags, pe_rsc_unique)) {
+        rsc_to_ban = uber_parent(rsc);
+    }
+
+    // How many more times recovery will be tried on this node
+    remaining_tries = rsc->migration_threshold - fail_count;
+
+    if (remaining_tries <= 0) {
+        crm_warn("%s cannot run on %s due to reaching migration threshold "
+                 "(clean up resource to allow again)"
+                 CRM_XS " failures=%d migration-threshold=%d",
+                 rsc_to_ban->id, node->details->uname, fail_count,
+                 rsc->migration_threshold);
+        if (failed != NULL) {
+            *failed = rsc_to_ban;
+        }
+        return true;
+    }
+
+    crm_info("%s can fail %d more time%s on "
+             "%s before reaching migration threshold (%d)",
+             rsc_to_ban->id, remaining_tries, pcmk__plural_s(remaining_tries),
+             node->details->uname, rsc->migration_threshold);
+    return false;
+}
+
+void
+pcmk_free_injections(pcmk_injections_t *injections)
+{
+    if (injections == NULL) {
+        return;
+    }
+
+    g_list_free_full(injections->node_up, g_free);
+    g_list_free_full(injections->node_down, g_free);
+    g_list_free_full(injections->node_fail, g_free);
+    g_list_free_full(injections->op_fail, g_free);
+    g_list_free_full(injections->op_inject, g_free);
+    g_list_free_full(injections->ticket_grant, g_free);
+    g_list_free_full(injections->ticket_revoke, g_free);
+    g_list_free_full(injections->ticket_standby, g_free);
+    g_list_free_full(injections->ticket_activate, g_free);
+    free(injections->quorum);
+    free(injections->watchdog);
+
+    free(injections);
+}
