@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2020 the Pacemaker project contributors
+ * Copyright 2004-2021 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -15,6 +15,7 @@
 #include <crm/pengine/status.h>
 #include <crm/pengine/internal.h>
 #include <crm/msg_xml.h>
+#include <crm/common/output.h>
 #include <crm/common/xml_internal.h>
 #include <pe_status_private.h>
 
@@ -226,7 +227,7 @@ create_docker_resource(pe_resource_t *parent, pe__bundle_variant_data_t *data,
             offset += snprintf(buffer+offset, max-offset, " -e PCMK_remote_port=%d", DEFAULT_REMOTE_PORT);
         }
 
-        for(GListPtr pIter = data->mounts; pIter != NULL; pIter = pIter->next) {
+        for(GList *pIter = data->mounts; pIter != NULL; pIter = pIter->next) {
             pe__bundle_mount_t *mount = pIter->data;
 
             if (pcmk_is_set(mount->flags, pe__bundle_mount_subdir)) {
@@ -248,7 +249,7 @@ create_docker_resource(pe_resource_t *parent, pe__bundle_variant_data_t *data,
             }
         }
 
-        for(GListPtr pIter = data->ports; pIter != NULL; pIter = pIter->next) {
+        for(GList *pIter = data->ports; pIter != NULL; pIter = pIter->next) {
             pe__bundle_port_t *port = pIter->data;
 
             if (replica->ipaddr) {
@@ -292,21 +293,19 @@ create_docker_resource(pe_resource_t *parent, pe__bundle_variant_data_t *data,
              * monitor the child independently
              */
             crm_create_nvpair_xml(xml_obj, NULL, "monitor_cmd", "/bin/true");
-        /* } else if(child && data->untrusted) {
-         * Support this use-case?
-         *
-         * The ability to have resources started/stopped by us, but
-         * unable to set attributes, etc.
-         *
-         * Arguably better to control API access this with ACLs like
-         * "normal" remote nodes
-         *
-         *     crm_create_nvpair_xml(xml_obj, NULL,
-         *                           "run_cmd",
-         *                           "/usr/libexec/pacemaker/pacemaker-execd");
-         *     crm_create_nvpair_xml(xml_obj, NULL, "monitor_cmd",
-         *         "/usr/libexec/pacemaker/lrmd_internal_ctl -c poke");
+#if 0
+        /* @TODO Consider supporting the use case where we can start and stop
+         * resources, but not proxy local commands (such as setting node
+         * attributes), by running the local executor in stand-alone mode.
+         * However, this would probably be better done via ACLs as with other
+         * Pacemaker Remote nodes.
          */
+        } else if ((child != NULL) && data->untrusted) {
+            crm_create_nvpair_xml(xml_obj, NULL, "run_cmd",
+                                  CRM_DAEMON_DIR "/pacemaker-execd");
+            crm_create_nvpair_xml(xml_obj, NULL, "monitor_cmd",
+                                  CRM_DAEMON_DIR "/pacemaker/cts-exec-helper -c poke");
+#endif
         } else {
             if (data->container_command) {
                 crm_create_nvpair_xml(xml_obj, NULL,
@@ -394,7 +393,7 @@ create_podman_resource(pe_resource_t *parent, pe__bundle_variant_data_t *data,
             offset += snprintf(buffer+offset, max-offset, " -e PCMK_remote_port=%d", DEFAULT_REMOTE_PORT);
         }
 
-        for(GListPtr pIter = data->mounts; pIter != NULL; pIter = pIter->next) {
+        for(GList *pIter = data->mounts; pIter != NULL; pIter = pIter->next) {
             pe__bundle_mount_t *mount = pIter->data;
 
             if (pcmk_is_set(mount->flags, pe__bundle_mount_subdir)) {
@@ -416,7 +415,7 @@ create_podman_resource(pe_resource_t *parent, pe__bundle_variant_data_t *data,
             }
         }
 
-        for(GListPtr pIter = data->ports; pIter != NULL; pIter = pIter->next) {
+        for(GList *pIter = data->ports; pIter != NULL; pIter = pIter->next) {
             pe__bundle_port_t *port = pIter->data;
 
             if (replica->ipaddr) {
@@ -460,21 +459,19 @@ create_podman_resource(pe_resource_t *parent, pe__bundle_variant_data_t *data,
              * monitor the child independently
              */
             crm_create_nvpair_xml(xml_obj, NULL, "monitor_cmd", "/bin/true");
-        /* } else if(child && data->untrusted) {
-         * Support this use-case?
-         *
-         * The ability to have resources started/stopped by us, but
-         * unable to set attributes, etc.
-         *
-         * Arguably better to control API access this with ACLs like
-         * "normal" remote nodes
-         *
-         *     crm_create_nvpair_xml(xml_obj, NULL,
-         *                           "run_cmd",
-         *                           "/usr/libexec/pacemaker/pacemaker-execd");
-         *     crm_create_nvpair_xml(xml_obj, NULL, "monitor_cmd",
-         *         "/usr/libexec/pacemaker/lrmd_internal_ctl -c poke");
+#if 0
+        /* @TODO Consider supporting the use case where we can start and stop
+         * resources, but not proxy local commands (such as setting node
+         * attributes), by running the local executor in stand-alone mode.
+         * However, this would probably be better done via ACLs as with other
+         * Pacemaker Remote nodes.
          */
+        } else if ((child != NULL) && data->untrusted) {
+            crm_create_nvpair_xml(xml_obj, NULL, "run_cmd",
+                                  CRM_DAEMON_DIR "/pacemaker-execd");
+            crm_create_nvpair_xml(xml_obj, NULL, "monitor_cmd",
+                                  CRM_DAEMON_DIR "/pacemaker/cts-exec-helper -c poke");
+#endif
         } else {
             if (data->container_command) {
                 crm_create_nvpair_xml(xml_obj, NULL,
@@ -560,7 +557,7 @@ create_rkt_resource(pe_resource_t *parent, pe__bundle_variant_data_t *data,
             offset += snprintf(buffer+offset, max-offset, " --environment=PCMK_remote_port=%d", DEFAULT_REMOTE_PORT);
         }
 
-        for(GListPtr pIter = data->mounts; pIter != NULL; pIter = pIter->next) {
+        for(GList *pIter = data->mounts; pIter != NULL; pIter = pIter->next) {
             pe__bundle_mount_t *mount = pIter->data;
 
             if (pcmk_is_set(mount->flags, pe__bundle_mount_subdir)) {
@@ -588,7 +585,7 @@ create_rkt_resource(pe_resource_t *parent, pe__bundle_variant_data_t *data,
             volid++;
         }
 
-        for(GListPtr pIter = data->ports; pIter != NULL; pIter = pIter->next) {
+        for(GList *pIter = data->ports; pIter != NULL; pIter = pIter->next) {
             pe__bundle_port_t *port = pIter->data;
 
             if (replica->ipaddr) {
@@ -631,21 +628,19 @@ create_rkt_resource(pe_resource_t *parent, pe__bundle_variant_data_t *data,
              * monitor the child independently
              */
             crm_create_nvpair_xml(xml_obj, NULL, "monitor_cmd", "/bin/true");
-        /* } else if(child && data->untrusted) {
-         * Support this use-case?
-         *
-         * The ability to have resources started/stopped by us, but
-         * unable to set attributes, etc.
-         *
-         * Arguably better to control API access this with ACLs like
-         * "normal" remote nodes
-         *
-         *     crm_create_nvpair_xml(xml_obj, NULL,
-         *                           "run_cmd",
-         *                           "/usr/libexec/pacemaker/pacemaker-execd");
-         *     crm_create_nvpair_xml(xml_obj, NULL, "monitor_cmd",
-         *         "/usr/libexec/pacemaker/lrmd_internal_ctl -c poke");
+#if 0
+        /* @TODO Consider supporting the use case where we can start and stop
+         * resources, but not proxy local commands (such as setting node
+         * attributes), by running the local executor in stand-alone mode.
+         * However, this would probably be better done via ACLs as with other
+         * Pacemaker Remote nodes.
          */
+        } else if ((child != NULL) && data->untrusted) {
+            crm_create_nvpair_xml(xml_obj, NULL, "run_cmd",
+                                  CRM_DAEMON_DIR "/pacemaker-execd");
+            crm_create_nvpair_xml(xml_obj, NULL, "monitor_cmd",
+                                  CRM_DAEMON_DIR "/pacemaker/cts-exec-helper -c poke");
+#endif
         } else {
             if (data->container_command) {
                 crm_create_nvpair_xml(xml_obj, NULL, "run_cmd",
@@ -689,11 +684,7 @@ disallow_node(pe_resource_t *rsc, const char *uname)
         ((pe_node_t *) match)->rsc_discover_mode = pe_discover_never;
     }
     if (rsc->children) {
-        GListPtr child;
-
-        for (child = rsc->children; child != NULL; child = child->next) {
-            disallow_node((pe_resource_t *) (child->data), uname);
-        }
+        g_list_foreach(rsc->children, (GFunc) disallow_node, (gpointer) uname);
     }
 }
 
@@ -704,7 +695,6 @@ create_remote_resource(pe_resource_t *parent, pe__bundle_variant_data_t *data,
 {
     if (replica->child && valid_network(data)) {
         GHashTableIter gIter;
-        GListPtr rsc_iter = NULL;
         pe_node_t *node = NULL;
         xmlNode *xml_remote = NULL;
         char *id = crm_strdup_printf("%s-%d", data->prefix, replica->offset);
@@ -728,7 +718,7 @@ create_remote_resource(pe_resource_t *parent, pe__bundle_variant_data_t *data,
         connect_name = (replica->ipaddr? replica->ipaddr : "#uname");
 
         if (data->control_port == NULL) {
-            port_s = crm_itoa(DEFAULT_REMOTE_PORT);
+            port_s = pcmk__itoa(DEFAULT_REMOTE_PORT);
         }
 
         /* This sets replica->container as replica->remote's container, which is
@@ -780,9 +770,7 @@ create_remote_resource(pe_resource_t *parent, pe__bundle_variant_data_t *data,
          * @TODO Possible alternative: ensure bundles are unpacked before other
          * resources, so the weight is correct before any copies are made.
          */
-        for (rsc_iter = data_set->resources; rsc_iter; rsc_iter = rsc_iter->next) {
-            disallow_node((pe_resource_t *) (rsc_iter->data), uname);
-        }
+        g_list_foreach(data_set->resources, (GFunc) disallow_node, (gpointer) uname);
 
         replica->node = pe__copy_node(node);
         replica->node->weight = 500;
@@ -792,9 +780,7 @@ create_remote_resource(pe_resource_t *parent, pe__bundle_variant_data_t *data,
         if (replica->child->allowed_nodes != NULL) {
             g_hash_table_destroy(replica->child->allowed_nodes);
         }
-        replica->child->allowed_nodes = g_hash_table_new_full(crm_str_hash,
-                                                              g_str_equal,
-                                                              NULL, free);
+        replica->child->allowed_nodes = pcmk__strkey_table(NULL, free);
         g_hash_table_insert(replica->child->allowed_nodes,
                             (gpointer) replica->node->details->id,
                             pe__copy_node(replica->node));
@@ -954,44 +940,33 @@ replica_for_remote(pe_resource_t *remote)
 }
 
 bool
-pe__bundle_needs_remote_name(pe_resource_t *rsc)
+pe__bundle_needs_remote_name(pe_resource_t *rsc, pe_working_set_t *data_set)
 {
     const char *value;
+    GHashTable *params = NULL;
 
     if (rsc == NULL) {
-        return FALSE;
+        return false;
     }
 
-    value = g_hash_table_lookup(rsc->parameters, XML_RSC_ATTR_REMOTE_RA_ADDR);
-    if (!pcmk__str_eq(value, "#uname", pcmk__str_casei)) {
-        return FALSE;
+    // Use NULL node since pcmk__bundle_expand() uses that to set value
+    params = pe_rsc_params(rsc, NULL, data_set);
+    value = g_hash_table_lookup(params, XML_RSC_ATTR_REMOTE_RA_ADDR);
 
-    } else {
-        const char *match[3][2] = {
-            { XML_ATTR_TYPE,           "remote"                },
-            { XML_AGENT_ATTR_CLASS,    PCMK_RESOURCE_CLASS_OCF },
-            { XML_AGENT_ATTR_PROVIDER, "pacemaker"             },
-        };
-
-        for (int m = 0; m < 3; m++) {
-            value = crm_element_value(rsc->xml, match[m][0]);
-            if (!pcmk__str_eq(value, match[m][1], pcmk__str_casei)) {
-                return FALSE;
-            }
-        }
-    }
-    return TRUE;
+    return pcmk__str_eq(value, "#uname", pcmk__str_casei)
+           && xml_contains_remote_node(rsc->xml);
 }
 
 const char *
-pe__add_bundle_remote_name(pe_resource_t *rsc, xmlNode *xml, const char *field)
+pe__add_bundle_remote_name(pe_resource_t *rsc, pe_working_set_t *data_set,
+                           xmlNode *xml, const char *field)
 {
     // REMOTE_CONTAINER_HACK: Allow remote nodes that start containers with pacemaker remote inside
 
     pe_node_t *node = NULL;
     pe__bundle_replica_t *replica = NULL;
 
-    if (!pe__bundle_needs_remote_name(rsc)) {
+    if (!pe__bundle_needs_remote_name(rsc, data_set)) {
         return NULL;
     }
 
@@ -1061,27 +1036,20 @@ pe__unpack_bundle(pe_resource_t *rsc, pe_working_set_t *data_set)
         }
     }
 
+    // Use 0 for default, minimum, and invalid promoted-max
     value = crm_element_value(xml_obj, XML_RSC_ATTR_PROMOTED_MAX);
     if (value == NULL) {
         // @COMPAT deprecated since 2.0.0
         value = crm_element_value(xml_obj, "masters");
     }
-    bundle_data->promoted_max = crm_parse_int(value, "0");
-    if (bundle_data->promoted_max < 0) {
-        pe_err("%s for %s must be nonnegative integer, using 0",
-               XML_RSC_ATTR_PROMOTED_MAX, rsc->id);
-        bundle_data->promoted_max = 0;
-    }
+    pcmk__scan_min_int(value, &bundle_data->promoted_max, 0);
 
+    // Default replicas to promoted-max if it was specified and 1 otherwise
     value = crm_element_value(xml_obj, "replicas");
-    if ((value == NULL) && bundle_data->promoted_max) {
+    if ((value == NULL) && (bundle_data->promoted_max > 0)) {
         bundle_data->nreplicas = bundle_data->promoted_max;
     } else {
-        bundle_data->nreplicas = crm_parse_int(value, "1");
-    }
-    if (bundle_data->nreplicas < 1) {
-        pe_err("'replicas' for %s must be positive integer, using 1", rsc->id);
-        bundle_data->nreplicas = 1;
+        pcmk__scan_min_int(value, &bundle_data->nreplicas, 1);
     }
 
     /*
@@ -1090,12 +1058,7 @@ pe__unpack_bundle(pe_resource_t *rsc, pe_working_set_t *data_set)
      *   --userland-proxy=false --ip-masq=false
      */
     value = crm_element_value(xml_obj, "replicas-per-host");
-    bundle_data->nreplicas_per_host = crm_parse_int(value, "1");
-    if (bundle_data->nreplicas_per_host < 1) {
-        pe_err("'replicas-per-host' for %s must be positive integer, using 1",
-               rsc->id);
-        bundle_data->nreplicas_per_host = 1;
-    }
+    pcmk__scan_min_int(value, &bundle_data->nreplicas_per_host, 1);
     if (bundle_data->nreplicas_per_host == 1) {
         pe__clear_resource_flags(rsc, pe_rsc_unique);
     }
@@ -1188,12 +1151,12 @@ pe__unpack_bundle(pe_resource_t *rsc, pe_working_set_t *data_set)
         crm_create_nvpair_xml(xml_set, NULL,
                               XML_RSC_ATTR_ORDERED, XML_BOOLEAN_TRUE);
 
-        value = crm_itoa(bundle_data->nreplicas);
+        value = pcmk__itoa(bundle_data->nreplicas);
         crm_create_nvpair_xml(xml_set, NULL,
                               XML_RSC_ATTR_INCARNATION_MAX, value);
         free(value);
 
-        value = crm_itoa(bundle_data->nreplicas_per_host);
+        value = pcmk__itoa(bundle_data->nreplicas_per_host);
         crm_create_nvpair_xml(xml_set, NULL,
                               XML_RSC_ATTR_INCARNATION_NODEMAX, value);
         free(value);
@@ -1205,7 +1168,7 @@ pe__unpack_bundle(pe_resource_t *rsc, pe_working_set_t *data_set)
             crm_create_nvpair_xml(xml_set, NULL,
                                   XML_RSC_ATTR_PROMOTABLE, XML_BOOLEAN_TRUE);
 
-            value = crm_itoa(bundle_data->promoted_max);
+            value = pcmk__itoa(bundle_data->promoted_max);
             crm_create_nvpair_xml(xml_set, NULL,
                                   XML_RSC_ATTR_PROMOTED_MAX, value);
             free(value);
@@ -1222,7 +1185,7 @@ pe__unpack_bundle(pe_resource_t *rsc, pe_working_set_t *data_set)
 
     if(xml_resource) {
         int lpc = 0;
-        GListPtr childIter = NULL;
+        GList *childIter = NULL;
         pe_resource_t *new_rsc = NULL;
         pe__bundle_port_t *port = NULL;
 
@@ -1280,7 +1243,7 @@ pe__unpack_bundle(pe_resource_t *rsc, pe_working_set_t *data_set)
              * environment and the connection resource parameters, and the user
              * can use a different port if desired by setting control-port.
              */
-            port->source = crm_itoa(DEFAULT_REMOTE_PORT);
+            port->source = pcmk__itoa(DEFAULT_REMOTE_PORT);
         }
         port->target = strdup(port->source);
         bundle_data->ports = g_list_append(bundle_data->ports, port);
@@ -1369,7 +1332,7 @@ gboolean
 pe__bundle_active(pe_resource_t *rsc, gboolean all)
 {
     pe__bundle_variant_data_t *bundle_data = NULL;
-    GListPtr iter = NULL;
+    GList *iter = NULL;
 
     get_bundle_variant_data(bundle_data, rsc);
     for (iter = bundle_data->replicas; iter != NULL; iter = iter->next) {
@@ -1500,14 +1463,14 @@ bundle_print_xml(pe_resource_t *rsc, const char *pre_text, long options,
     free(child_text);
 }
 
-PCMK__OUTPUT_ARGS("bundle", "unsigned int", "pe_resource_t *", "GListPtr", "GListPtr")
+PCMK__OUTPUT_ARGS("bundle", "unsigned int", "pe_resource_t *", "GList *", "GList *")
 int
 pe__bundle_xml(pcmk__output_t *out, va_list args)
 {
-    unsigned int options = va_arg(args, unsigned int);
+    unsigned int show_opts = va_arg(args, unsigned int);
     pe_resource_t *rsc = va_arg(args, pe_resource_t *);
-    GListPtr only_node = va_arg(args, GListPtr);
-    GListPtr only_rsc = va_arg(args, GListPtr);
+    GList *only_node = va_arg(args, GList *);
+    GList *only_rsc = va_arg(args, GList *);
 
     pe__bundle_variant_data_t *bundle_data = NULL;
     int rc = pcmk_rc_no_output;
@@ -1522,12 +1485,12 @@ pe__bundle_xml(pcmk__output_t *out, va_list args)
         return rc;
     }
 
-    print_everything = pcmk__str_in_list(only_rsc, rsc->id);
+    print_everything = pcmk__str_in_list(rsc->id, only_rsc, pcmk__str_star_matches);
 
     for (GList *gIter = bundle_data->replicas; gIter != NULL;
          gIter = gIter->next) {
         pe__bundle_replica_t *replica = gIter->data;
-        char *id = crm_itoa(replica->offset);
+        char *id = NULL;
         gboolean print_ip, print_child, print_ctnr, print_remote;
 
         CRM_ASSERT(replica);
@@ -1561,27 +1524,28 @@ pe__bundle_xml(pcmk__output_t *out, va_list args)
             CRM_ASSERT(rc == pcmk_rc_ok);
         }
 
+        id = pcmk__itoa(replica->offset);
         rc = pe__name_and_nvpairs_xml(out, true, "replica", 1, "id", id);
         free(id);
         CRM_ASSERT(rc == pcmk_rc_ok);
 
         if (print_ip) {
-            out->message(out, crm_map_element_name(replica->ip->xml), options,
+            out->message(out, crm_map_element_name(replica->ip->xml), show_opts,
                          replica->ip, only_node, only_rsc);
         }
 
         if (print_child) {
-            out->message(out, crm_map_element_name(replica->child->xml), options,
+            out->message(out, crm_map_element_name(replica->child->xml), show_opts,
                          replica->child, only_node, only_rsc);
         }
 
         if (print_ctnr) {
-            out->message(out, crm_map_element_name(replica->container->xml), options,
+            out->message(out, crm_map_element_name(replica->container->xml), show_opts,
                          replica->container, only_node, only_rsc);
         }
 
         if (print_remote) {
-            out->message(out, crm_map_element_name(replica->remote->xml), options,
+            out->message(out, crm_map_element_name(replica->remote->xml), show_opts,
                          replica->remote, only_node, only_rsc);
         }
 
@@ -1597,7 +1561,7 @@ pe__bundle_xml(pcmk__output_t *out, va_list args)
 
 static void
 pe__bundle_replica_output_html(pcmk__output_t *out, pe__bundle_replica_t *replica,
-                               pe_node_t *node, long options)
+                               pe_node_t *node, unsigned int show_opts)
 {
     pe_resource_t *rsc = replica->child;
 
@@ -1620,20 +1584,19 @@ pe__bundle_replica_output_html(pcmk__output_t *out, pe__bundle_replica_t *replic
                            replica->ipaddr);
     }
 
-    pe__common_output_html(out, rsc, buffer, node, options);
+    pe__common_output_html(out, rsc, buffer, node, show_opts);
 }
 
-PCMK__OUTPUT_ARGS("bundle", "unsigned int", "pe_resource_t *", "GListPtr", "GListPtr")
+PCMK__OUTPUT_ARGS("bundle", "unsigned int", "pe_resource_t *", "GList *", "GList *")
 int
 pe__bundle_html(pcmk__output_t *out, va_list args)
 {
-    unsigned int options = va_arg(args, unsigned int);
+    unsigned int show_opts = va_arg(args, unsigned int);
     pe_resource_t *rsc = va_arg(args, pe_resource_t *);
-    GListPtr only_node = va_arg(args, GListPtr);
-    GListPtr only_rsc = va_arg(args, GListPtr);
+    GList *only_node = va_arg(args, GList *);
+    GList *only_rsc = va_arg(args, GList *);
 
     pe__bundle_variant_data_t *bundle_data = NULL;
-    char buffer[LINE_MAX];
     int rc = pcmk_rc_no_output;
     gboolean print_everything = TRUE;
 
@@ -1645,7 +1608,7 @@ pe__bundle_html(pcmk__output_t *out, va_list args)
         return rc;
     }
 
-    print_everything = pcmk__str_in_list(only_rsc, rsc->id);
+    print_everything = pcmk__str_in_list(rsc->id, only_rsc, pcmk__str_star_matches);
 
     for (GList *gIter = bundle_data->replicas; gIter != NULL;
          gIter = gIter->next) {
@@ -1666,20 +1629,12 @@ pe__bundle_html(pcmk__output_t *out, va_list args)
         print_remote = replica->remote != NULL &&
                        !replica->remote->fns->is_filtered(replica->remote, only_rsc, print_everything);
 
-        if (pcmk_is_set(options, pe_print_implicit) ||
+        if (pcmk_is_set(show_opts, pcmk_show_implicit_rscs) ||
             (print_everything == FALSE && (print_ip || print_child || print_ctnr || print_remote))) {
             /* The text output messages used below require pe_print_implicit to
              * be set to do anything.
              */
-            unsigned int new_options = options;
-
-            if (!pcmk_is_set(options, pe_print_implicit)) {
-                new_options |= pe_print_implicit;
-            }
-
-            if (rc == pcmk_rc_no_output) {
-                pcmk__output_create_xml_node(out, "br");
-            }
+            unsigned int new_show_opts = show_opts | pcmk_show_implicit_rscs;
 
             PCMK__OUTPUT_LIST_HEADER(out, FALSE, rc, "Container bundle%s: %s [%s]%s%s",
                                      (bundle_data->nreplicas > 1)? " set" : "",
@@ -1687,36 +1642,33 @@ pe__bundle_html(pcmk__output_t *out, va_list args)
                                      pcmk_is_set(rsc->flags, pe_rsc_unique) ? " (unique)" : "",
                                      pcmk_is_set(rsc->flags, pe_rsc_managed) ? "" : " (unmanaged)");
 
-            pcmk__output_xml_create_parent(out, "li");
-
             if (pcmk__list_of_multiple(bundle_data->replicas)) {
-                snprintf(buffer, LINE_MAX, " Replica[%d]", replica->offset);
-                xmlNodeSetContent(pcmk__output_xml_peek_parent(out), (pcmkXmlStr) buffer);
+                out->begin_list(out, NULL, NULL, "Replica[%d]", replica->offset);
             }
-            pcmk__output_create_xml_node(out, "br");
-            out->begin_list(out, NULL, NULL, NULL);
 
             if (print_ip) {
                 out->message(out, crm_map_element_name(replica->ip->xml),
-                             new_options, replica->ip, only_node, only_rsc);
+                             new_show_opts, replica->ip, only_node, only_rsc);
             }
 
             if (print_child) {
                 out->message(out, crm_map_element_name(replica->child->xml),
-                             new_options, replica->child, only_node, only_rsc);
+                             new_show_opts, replica->child, only_node, only_rsc);
             }
 
             if (print_ctnr) {
                 out->message(out, crm_map_element_name(replica->container->xml),
-                             new_options, replica->container, only_node, only_rsc);
+                             new_show_opts, replica->container, only_node, only_rsc);
             }
 
             if (print_remote) {
                 out->message(out, crm_map_element_name(replica->remote->xml),
-                             new_options, replica->remote, only_node, only_rsc);
+                             new_show_opts, replica->remote, only_node, only_rsc);
             }
 
-            out->end_list(out);
+            if (pcmk__list_of_multiple(bundle_data->replicas)) {
+                out->end_list(out);
+            }
         } else if (print_everything == FALSE && !(print_ip || print_child || print_ctnr || print_remote)) {
             continue;
         } else {
@@ -1727,10 +1679,8 @@ pe__bundle_html(pcmk__output_t *out, va_list args)
                                      pcmk_is_set(rsc->flags, pe_rsc_managed) ? "" : " (unmanaged)");
 
             pe__bundle_replica_output_html(out, replica, pe__current_node(replica->container),
-                                           options);
+                                           show_opts);
         }
-
-        pcmk__output_xml_pop_parent(out);
     }
 
     PCMK__OUTPUT_LIST_FOOTER(out, rc);
@@ -1739,7 +1689,7 @@ pe__bundle_html(pcmk__output_t *out, va_list args)
 
 static void
 pe__bundle_replica_output_text(pcmk__output_t *out, pe__bundle_replica_t *replica,
-                               pe_node_t *node, long options)
+                               pe_node_t *node, unsigned int show_opts)
 {
     pe_resource_t *rsc = replica->child;
 
@@ -1762,17 +1712,17 @@ pe__bundle_replica_output_text(pcmk__output_t *out, pe__bundle_replica_t *replic
                            replica->ipaddr);
     }
 
-    pe__common_output_text(out, rsc, buffer, node, options);
+    pe__common_output_text(out, rsc, buffer, node, show_opts);
 }
 
-PCMK__OUTPUT_ARGS("bundle", "unsigned int", "pe_resource_t *", "GListPtr", "GListPtr")
+PCMK__OUTPUT_ARGS("bundle", "unsigned int", "pe_resource_t *", "GList *", "GList *")
 int
 pe__bundle_text(pcmk__output_t *out, va_list args)
 {
-    unsigned int options = va_arg(args, unsigned int);
+    unsigned int show_opts = va_arg(args, unsigned int);
     pe_resource_t *rsc = va_arg(args, pe_resource_t *);
-    GListPtr only_node = va_arg(args, GListPtr);
-    GListPtr only_rsc = va_arg(args, GListPtr);
+    GList *only_node = va_arg(args, GList *);
+    GList *only_rsc = va_arg(args, GList *);
 
     pe__bundle_variant_data_t *bundle_data = NULL;
     int rc = pcmk_rc_no_output;
@@ -1786,7 +1736,7 @@ pe__bundle_text(pcmk__output_t *out, va_list args)
         return rc;
     }
 
-    print_everything = pcmk__str_in_list(only_rsc, rsc->id);
+    print_everything = pcmk__str_in_list(rsc->id, only_rsc, pcmk__str_star_matches);
 
     for (GList *gIter = bundle_data->replicas; gIter != NULL;
          gIter = gIter->next) {
@@ -1807,16 +1757,13 @@ pe__bundle_text(pcmk__output_t *out, va_list args)
         print_remote = replica->remote != NULL &&
                        !replica->remote->fns->is_filtered(replica->remote, only_rsc, print_everything);
 
-        if (pcmk_is_set(options, pe_print_implicit) ||
+        if (pcmk_is_set(show_opts, pcmk_show_implicit_rscs) ||
             (print_everything == FALSE && (print_ip || print_child || print_ctnr || print_remote))) {
             /* The text output messages used below require pe_print_implicit to
              * be set to do anything.
              */
-            unsigned int new_options = options;
+            unsigned int new_show_opts = show_opts | pcmk_show_implicit_rscs;
 
-            if (!pcmk_is_set(options, pe_print_implicit)) {
-                new_options |= pe_print_implicit;
-            }
             PCMK__OUTPUT_LIST_HEADER(out, FALSE, rc, "Container bundle%s: %s [%s]%s%s",
                                      (bundle_data->nreplicas > 1)? " set" : "",
                                      rsc->id, bundle_data->image,
@@ -1831,22 +1778,22 @@ pe__bundle_text(pcmk__output_t *out, va_list args)
 
             if (print_ip) {
                 out->message(out, crm_map_element_name(replica->ip->xml),
-                             new_options, replica->ip, only_node, only_rsc);
+                             new_show_opts, replica->ip, only_node, only_rsc);
             }
 
             if (print_child) {
                 out->message(out, crm_map_element_name(replica->child->xml),
-                             new_options, replica->child, only_node, only_rsc);
+                             new_show_opts, replica->child, only_node, only_rsc);
             }
 
             if (print_ctnr) {
                 out->message(out, crm_map_element_name(replica->container->xml),
-                             new_options, replica->container, only_node, only_rsc);
+                             new_show_opts, replica->container, only_node, only_rsc);
             }
 
             if (print_remote) {
                 out->message(out, crm_map_element_name(replica->remote->xml),
-                             new_options, replica->remote, only_node, only_rsc);
+                             new_show_opts, replica->remote, only_node, only_rsc);
             }
 
             out->end_list(out);
@@ -1860,7 +1807,7 @@ pe__bundle_text(pcmk__output_t *out, va_list args)
                                      pcmk_is_set(rsc->flags, pe_rsc_managed) ? "" : " (unmanaged)");
 
             pe__bundle_replica_output_text(out, replica, pe__current_node(replica->container),
-                                           options);
+                                           show_opts);
         }
     }
 
@@ -2086,12 +2033,12 @@ pe__count_bundle(pe_resource_t *rsc)
 }
 
 gboolean
-pe__bundle_is_filtered(pe_resource_t *rsc, GListPtr only_rsc, gboolean check_parent)
+pe__bundle_is_filtered(pe_resource_t *rsc, GList *only_rsc, gboolean check_parent)
 {
     gboolean passes = FALSE;
     pe__bundle_variant_data_t *bundle_data = NULL;
 
-    if (pcmk__str_in_list(only_rsc, rsc_printable_id(rsc))) {
+    if (pcmk__str_in_list(rsc_printable_id(rsc), only_rsc, pcmk__str_star_matches)) {
         passes = TRUE;
     } else {
         get_bundle_variant_data(bundle_data, rsc);
