@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2020 the Pacemaker project contributors
+ * Copyright 2004-2021 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -14,11 +14,53 @@
 #include <crm/pengine/internal.h>
 #include <pe_status_private.h>
 #include <crm/msg_xml.h>
+#include <crm/common/output.h>
 #include <crm/common/xml_internal.h>
 
 #define VARIANT_CLONE 1
 #include "./variant.h"
 
+#ifdef PCMK__COMPAT_2_0
+#define PROMOTED_INSTANCES   RSC_ROLE_PROMOTED_LEGACY_S "s"
+#define UNPROMOTED_INSTANCES RSC_ROLE_UNPROMOTED_LEGACY_S "s"
+#else
+#define PROMOTED_INSTANCES   RSC_ROLE_PROMOTED_S
+#define UNPROMOTED_INSTANCES RSC_ROLE_UNPROMOTED_S
+#endif
+
+static void
+clone_header(pcmk__output_t *out, int *rc, pe_resource_t *rsc, clone_variant_data_t *clone_data)
+{
+    char *attrs = NULL;
+    size_t len = 0;
+
+    if (pcmk_is_set(rsc->flags, pe_rsc_promotable)) {
+        pcmk__add_separated_word(&attrs, &len, "promotable", ", ");
+    }
+
+    if (pcmk_is_set(rsc->flags, pe_rsc_unique)) {
+        pcmk__add_separated_word(&attrs, &len, "unique", ", ");
+    }
+
+    if (!pcmk_is_set(rsc->flags, pe_rsc_managed)) {
+        pcmk__add_separated_word(&attrs, &len, "unmanaged", ", ");
+    }
+
+    if (pe__resource_is_disabled(rsc)) {
+        pcmk__add_separated_word(&attrs, &len, "disabled", ", ");
+    }
+
+    if (attrs) {
+        PCMK__OUTPUT_LIST_HEADER(out, FALSE, *rc, "Clone Set: %s [%s] (%s)",
+                             rsc->id, ID(clone_data->xml_obj_child),
+                                 attrs);
+        free(attrs);
+    } else {
+        PCMK__OUTPUT_LIST_HEADER(out, FALSE, *rc, "Clone Set: %s [%s]",
+                                 rsc->id, ID(clone_data->xml_obj_child))
+    }
+}
+
 void
 pe__force_anon(const char *standard, pe_resource_t *rsc, const char *rid,
                pe_working_set_t *data_set)
@@ -76,8 +118,8 @@ pe__create_clone_child(pe_resource_t *rsc, pe_working_set_t *data_set)
     }
 
     // Allocate instance numbers in numerical order (starting at 0)
-    inc_num = crm_itoa(clone_data->total_clones);
-    inc_max = crm_itoa(clone_data->clone_max);
+    inc_num = pcmk__itoa(clone_data->total_clones);
+    inc_max = pcmk__itoa(clone_data->clone_max);
 
     child_copy = copy_xml(clone_data->xml_obj_child);
 
@@ -134,7 +176,7 @@ clone_unpack(pe_resource_t * rsc, pe_working_set_t * data_set)
         if (promoted_max == NULL) {
             // @COMPAT deprecated since 2.0.0
             promoted_max = g_hash_table_lookup(rsc->meta,
-                                               XML_RSC_ATTR_MASTER_MAX);
+                                               PCMK_XE_PROMOTED_MAX_LEGACY);
         }
 
         promoted_node_max = g_hash_table_lookup(rsc->meta,
@@ -142,26 +184,42 @@ clone_unpack(pe_resource_t * rsc, pe_working_set_t * data_set)
         if (promoted_node_max == NULL) {
             // @COMPAT deprecated since 2.0.0
             promoted_node_max = g_hash_table_lookup(rsc->meta,
-                                                    XML_RSC_ATTR_MASTER_NODEMAX);
+                                                    PCMK_XE_PROMOTED_NODE_MAX_LEGACY);
         }
 
-        clone_data->promoted_max = crm_parse_int(promoted_max, "1");
-        clone_data->promoted_node_max = crm_parse_int(promoted_node_max, "1");
+        // Use 1 as default but 0 for minimum and invalid
+        if (promoted_max == NULL) {
+            clone_data->promoted_max = 1;
+        } else {
+            pcmk__scan_min_int(promoted_max, &(clone_data->promoted_max), 0);
+        }
+
+        // Use 1 as default but 0 for minimum and invalid
+        if (promoted_node_max == NULL) {
+            clone_data->promoted_node_max = 1;
+        } else {
+            pcmk__scan_min_int(promoted_node_max,
+                               &(clone_data->promoted_node_max), 0);
+        }
     }
 
     // Implied by calloc()
     /* clone_data->xml_obj_child = NULL; */
 
-    clone_data->clone_node_max = crm_parse_int(max_clones_node, "1");
-
-    if (max_clones) {
-        clone_data->clone_max = crm_parse_int(max_clones, "1");
-
-    } else if (pcmk__list_of_multiple(data_set->nodes)) {
-        clone_data->clone_max = g_list_length(data_set->nodes);
-
+    // Use 1 as default but 0 for minimum and invalid
+    if (max_clones_node == NULL) {
+        clone_data->clone_node_max = 1;
     } else {
-        clone_data->clone_max = 1;      /* Handy during crm_verify */
+        pcmk__scan_min_int(max_clones_node, &(clone_data->clone_node_max), 0);
+    }
+
+    /* Use number of nodes (but always at least 1, which is handy for crm_verify
+     * for a CIB without nodes) as default, but 0 for minimum and invalid
+     */
+    if (max_clones == NULL) {
+        clone_data->clone_max = QB_MAX(1, g_list_length(data_set->nodes));
+    } else {
+        pcmk__scan_min_int(max_clones, &(clone_data->clone_max), 0);
     }
 
     clone_data->ordered = crm_is_true(ordered);
@@ -236,7 +294,7 @@ clone_unpack(pe_resource_t * rsc, pe_working_set_t * data_set)
 gboolean
 clone_active(pe_resource_t * rsc, gboolean all)
 {
-    GListPtr gIter = rsc->children;
+    GList *gIter = rsc->children;
 
     for (; gIter != NULL; gIter = gIter->next) {
         pe_resource_t *child_rsc = (pe_resource_t *) gIter->data;
@@ -310,7 +368,7 @@ clone_print_xml(pe_resource_t * rsc, const char *pre_text, long options, void *p
 {
     char *child_text = crm_strdup_printf("%s    ", pre_text);
     const char *target_role = configured_role_str(rsc);
-    GListPtr gIter = rsc->children;
+    GList *gIter = rsc->children;
 
     status_print("%s<clone ", pre_text);
     status_print("id=\"%s\" ", rsc->id);
@@ -338,7 +396,7 @@ clone_print_xml(pe_resource_t * rsc, const char *pre_text, long options, void *p
 
 bool is_set_recursive(pe_resource_t * rsc, long long flag, bool any)
 {
-    GListPtr gIter;
+    GList *gIter;
     bool all = !any;
 
     if (pcmk_is_set(rsc->flags, flag)) {
@@ -375,9 +433,9 @@ clone_print(pe_resource_t * rsc, const char *pre_text, long options, void *print
     size_t list_text_len = 0;
     size_t stopped_list_len = 0;
 
-    GListPtr master_list = NULL;
-    GListPtr started_list = NULL;
-    GListPtr gIter = rsc->children;
+    GList *promoted_list = NULL;
+    GList *started_list = NULL;
+    GList *gIter = rsc->children;
 
     clone_variant_data_t *clone_data = NULL;
     int active_instances = 0;
@@ -458,8 +516,8 @@ clone_print(pe_resource_t * rsc, const char *pre_text, long options, void *print
                 if (location->details->online == FALSE && location->details->unclean) {
                     print_full = TRUE;
 
-                } else if (a_role > RSC_ROLE_SLAVE) {
-                    master_list = g_list_append(master_list, location);
+                } else if (a_role > RSC_ROLE_UNPROMOTED) {
+                    promoted_list = g_list_append(promoted_list, location);
 
                 } else {
                     started_list = g_list_append(started_list, location);
@@ -486,22 +544,23 @@ clone_print(pe_resource_t * rsc, const char *pre_text, long options, void *print
         }
     }
 
-    /* Masters */
-    master_list = g_list_sort(master_list, sort_node_uname);
-    for (gIter = master_list; gIter; gIter = gIter->next) {
+    /* Promoted */
+    promoted_list = g_list_sort(promoted_list, sort_node_uname);
+    for (gIter = promoted_list; gIter; gIter = gIter->next) {
         pe_node_t *host = gIter->data;
 
         pcmk__add_word(&list_text, &list_text_len, host->details->uname);
 	active_instances++;
     }
 
-    short_print(list_text, child_text, "Masters", NULL, options, print_data);
-    g_list_free(master_list);
+    short_print(list_text, child_text, PROMOTED_INSTANCES, NULL, options,
+                print_data);
+    g_list_free(promoted_list);
     free(list_text);
     list_text = NULL;
     list_text_len = 0;
 
-    /* Started/Slaves */
+    /* Started/Unpromoted */
     started_list = g_list_sort(started_list, sort_node_uname);
     for (gIter = started_list; gIter; gIter = gIter->next) {
         pe_node_t *host = gIter->data;
@@ -513,10 +572,13 @@ clone_print(pe_resource_t * rsc, const char *pre_text, long options, void *print
     if (pcmk_is_set(rsc->flags, pe_rsc_promotable)) {
         enum rsc_role_e role = configured_role(rsc);
 
-        if(role == RSC_ROLE_SLAVE) {
-            short_print(list_text, child_text, "Slaves (target-role)", NULL, options, print_data);
+        if (role == RSC_ROLE_UNPROMOTED) {
+            short_print(list_text, child_text,
+                        UNPROMOTED_INSTANCES " (target-role)", NULL, options,
+                        print_data);
         } else {
-            short_print(list_text, child_text, "Slaves", NULL, options, print_data);
+            short_print(list_text, child_text, UNPROMOTED_INSTANCES, NULL,
+                        options, print_data);
         }
 
     } else {
@@ -539,8 +601,8 @@ clone_print(pe_resource_t * rsc, const char *pre_text, long options, void *print
         if (!pcmk_is_set(rsc->flags, pe_rsc_unique)
             && (clone_data->clone_max > active_instances)) {
 
-            GListPtr nIter;
-            GListPtr list = g_hash_table_get_values(rsc->allowed_nodes);
+            GList *nIter;
+            GList *list = g_hash_table_get_values(rsc->allowed_nodes);
 
             /* Custom stopped list for non-unique clones */
             free(stopped_list);
@@ -577,16 +639,17 @@ clone_print(pe_resource_t * rsc, const char *pre_text, long options, void *print
     free(child_text);
 }
 
-PCMK__OUTPUT_ARGS("clone", "unsigned int", "pe_resource_t *", "GListPtr", "GListPtr")
+PCMK__OUTPUT_ARGS("clone", "unsigned int", "pe_resource_t *", "GList *", "GList *")
 int
 pe__clone_xml(pcmk__output_t *out, va_list args)
 {
-    unsigned int options = va_arg(args, unsigned int);
+    unsigned int show_opts = va_arg(args, unsigned int);
     pe_resource_t *rsc = va_arg(args, pe_resource_t *);
-    GListPtr only_node = va_arg(args, GListPtr);
-    GListPtr only_rsc = va_arg(args, GListPtr);
+    GList *only_node = va_arg(args, GList *);
+    GList *only_rsc = va_arg(args, GList *);
 
-    GListPtr gIter = rsc->children;
+    GList *gIter = rsc->children;
+    GList *all = NULL;
     int rc = pcmk_rc_no_output;
     gboolean printed_header = FALSE;
     gboolean print_everything = TRUE;
@@ -595,8 +658,10 @@ pe__clone_xml(pcmk__output_t *out, va_list args)
         return rc;
     }
 
-    print_everything = pcmk__str_in_list(only_rsc, rsc_printable_id(rsc)) ||
-                       (strstr(rsc->id, ":") != NULL && pcmk__str_in_list(only_rsc, rsc->id));
+    print_everything = pcmk__str_in_list(rsc_printable_id(rsc), only_rsc, pcmk__str_star_matches) ||
+                       (strstr(rsc->id, ":") != NULL && pcmk__str_in_list(rsc->id, only_rsc, pcmk__str_star_matches));
+
+    all = g_list_prepend(all, (gpointer) "*");
 
     for (; gIter != NULL; gIter = gIter->next) {
         pe_resource_t *child_rsc = (pe_resource_t *) gIter->data;
@@ -624,34 +689,35 @@ pe__clone_xml(pcmk__output_t *out, va_list args)
             CRM_ASSERT(rc == pcmk_rc_ok);
         }
 
-        out->message(out, crm_map_element_name(child_rsc->xml), options,
-                     child_rsc, only_node, only_rsc);
+        out->message(out, crm_map_element_name(child_rsc->xml), show_opts,
+                     child_rsc, only_node, all);
     }
 
     if (printed_header) {
         pcmk__output_xml_pop_parent(out);
     }
 
+    g_list_free(all);
     return rc;
 }
 
-PCMK__OUTPUT_ARGS("clone", "unsigned int", "pe_resource_t *", "GListPtr", "GListPtr")
+PCMK__OUTPUT_ARGS("clone", "unsigned int", "pe_resource_t *", "GList *", "GList *")
 int
-pe__clone_html(pcmk__output_t *out, va_list args)
+pe__clone_default(pcmk__output_t *out, va_list args)
 {
-    unsigned int options = va_arg(args, unsigned int);
+    unsigned int show_opts = va_arg(args, unsigned int);
     pe_resource_t *rsc = va_arg(args, pe_resource_t *);
-    GListPtr only_node = va_arg(args, GListPtr);
-    GListPtr only_rsc = va_arg(args, GListPtr);
+    GList *only_node = va_arg(args, GList *);
+    GList *only_rsc = va_arg(args, GList *);
 
     char *list_text = NULL;
     char *stopped_list = NULL;
     size_t list_text_len = 0;
     size_t stopped_list_len = 0;
 
-    GListPtr master_list = NULL;
-    GListPtr started_list = NULL;
-    GListPtr gIter = rsc->children;
+    GList *promoted_list = NULL;
+    GList *started_list = NULL;
+    GList *gIter = rsc->children;
 
     clone_variant_data_t *clone_data = NULL;
     int active_instances = 0;
@@ -664,16 +730,8 @@ pe__clone_html(pcmk__output_t *out, va_list args)
         return rc;
     }
 
-    print_everything = pcmk__str_in_list(only_rsc, rsc_printable_id(rsc)) ||
-                       (strstr(rsc->id, ":") != NULL && pcmk__str_in_list(only_rsc, rsc->id));
-
-    out->begin_list(out, NULL, NULL, "Clone Set: %s [%s]%s%s%s%s",
-                    rsc->id, ID(clone_data->xml_obj_child),
-                    pcmk_is_set(rsc->flags, pe_rsc_promotable) ? " (promotable)" : "",
-                    pcmk_is_set(rsc->flags, pe_rsc_unique) ? " (unique)" : "",
-                    pcmk_is_set(rsc->flags, pe_rsc_managed) ? "" : " (unmanaged)",
-                    pe__resource_is_disabled(rsc) ? " (disabled)" : "");
-    rc = pcmk_rc_ok;
+    print_everything = pcmk__str_in_list(rsc_printable_id(rsc), only_rsc, pcmk__str_star_matches) ||
+                       (strstr(rsc->id, ":") != NULL && pcmk__str_in_list(rsc->id, only_rsc, pcmk__str_star_matches));
 
     for (; gIter != NULL; gIter = gIter->next) {
         gboolean print_full = FALSE;
@@ -688,7 +746,7 @@ pe__clone_html(pcmk__output_t *out, va_list args)
             continue;
         }
 
-        if (options & pe_print_clone_details) {
+        if (pcmk_is_set(show_opts, pcmk_show_clone_detail)) {
             print_full = TRUE;
         }
 
@@ -700,7 +758,7 @@ pe__clone_html(pcmk__output_t *out, va_list args)
 
         // Everything else in this block is for anonymous clones
 
-        } else if (pcmk_is_set(options, pe_print_pending)
+        } else if (pcmk_is_set(show_opts, pcmk_show_pending)
                    && (child_rsc->pending_task != NULL)
                    && strcmp(child_rsc->pending_task, "probe")) {
             // Print individual instance when non-probe action is pending
@@ -709,7 +767,7 @@ pe__clone_html(pcmk__output_t *out, va_list args)
         } else if (partially_active == FALSE) {
             // List stopped instances when requested (except orphans)
             if (!pcmk_is_set(child_rsc->flags, pe_rsc_orphan)
-                && !pcmk_is_set(options, pe_print_clone_active)) {
+                && pcmk_is_set(show_opts, pcmk_show_inactive_rscs)) {
                 pcmk__add_word(&stopped_list, &stopped_list_len, child_rsc->id);
             }
 
@@ -733,8 +791,8 @@ pe__clone_html(pcmk__output_t *out, va_list args)
                 if (location->details->online == FALSE && location->details->unclean) {
                     print_full = TRUE;
 
-                } else if (a_role > RSC_ROLE_SLAVE) {
-                    master_list = g_list_append(master_list, location);
+                } else if (a_role > RSC_ROLE_UNPROMOTED) {
+                    promoted_list = g_list_append(promoted_list, location);
 
                 } else {
                     started_list = g_list_append(started_list, location);
@@ -751,77 +809,87 @@ pe__clone_html(pcmk__output_t *out, va_list args)
         }
 
         if (print_full) {
-            GListPtr all = NULL;
+            GList *all = NULL;
+
+            clone_header(out, &rc, rsc, clone_data);
 
             /* Print every resource that's a child of this clone. */
-            all = g_list_prepend(all, strdup("*"));
-            out->message(out, crm_map_element_name(child_rsc->xml), options,
+            all = g_list_prepend(all, (gpointer) "*");
+            out->message(out, crm_map_element_name(child_rsc->xml), show_opts,
                          child_rsc, only_node, all);
-            g_list_free_full(all, free);
+            g_list_free(all);
         }
     }
 
-    if (pcmk_is_set(options, pe_print_clone_details)) {
+    if (pcmk_is_set(show_opts, pcmk_show_clone_detail)) {
         free(stopped_list);
-        out->end_list(out);
+        PCMK__OUTPUT_LIST_FOOTER(out, rc);
         return pcmk_rc_ok;
     }
 
-    /* Masters */
-    master_list = g_list_sort(master_list, sort_node_uname);
-    for (gIter = master_list; gIter; gIter = gIter->next) {
+    /* Promoted */
+    promoted_list = g_list_sort(promoted_list, sort_node_uname);
+    for (gIter = promoted_list; gIter; gIter = gIter->next) {
         pe_node_t *host = gIter->data;
 
-        if (!pcmk__str_in_list(only_node, host->details->uname)) {
+        if (!pcmk__str_in_list(host->details->uname, only_node,
+                               pcmk__str_star_matches|pcmk__str_casei)) {
             continue;
         }
 
         pcmk__add_word(&list_text, &list_text_len, host->details->uname);
         active_instances++;
     }
+    g_list_free(promoted_list);
 
     if (list_text != NULL) {
-        out->list_item(out, NULL, " Masters: [ %s ]", list_text);
-        g_list_free(master_list);
+        clone_header(out, &rc, rsc, clone_data);
+
+        out->list_item(out, NULL, PROMOTED_INSTANCES ": [ %s ]", list_text);
         free(list_text);
         list_text = NULL;
         list_text_len = 0;
     }
 
-    /* Started/Slaves */
+    /* Started/Unpromoted */
     started_list = g_list_sort(started_list, sort_node_uname);
     for (gIter = started_list; gIter; gIter = gIter->next) {
         pe_node_t *host = gIter->data;
 
-        if (!pcmk__str_in_list(only_node, host->details->uname)) {
+        if (!pcmk__str_in_list(host->details->uname, only_node,
+                               pcmk__str_star_matches|pcmk__str_casei)) {
             continue;
         }
 
         pcmk__add_word(&list_text, &list_text_len, host->details->uname);
         active_instances++;
     }
+    g_list_free(started_list);
 
     if (list_text != NULL) {
+        clone_header(out, &rc, rsc, clone_data);
+
         if (pcmk_is_set(rsc->flags, pe_rsc_promotable)) {
             enum rsc_role_e role = configured_role(rsc);
 
-            if(role == RSC_ROLE_SLAVE) {
-                out->list_item(out, NULL, " Slaves (target-role): [ %s ]", list_text);
+            if (role == RSC_ROLE_UNPROMOTED) {
+                out->list_item(out, NULL,
+                               UNPROMOTED_INSTANCES " (target-role): [ %s ]",
+                               list_text);
             } else {
-                out->list_item(out, NULL, " Slaves: [ %s ]", list_text);
+                out->list_item(out, NULL, UNPROMOTED_INSTANCES ": [ %s ]",
+                               list_text);
             }
 
         } else {
-            out->list_item(out, NULL, " Started: [ %s ]", list_text);
+            out->list_item(out, NULL, "Started: [ %s ]", list_text);
         }
-
-        g_list_free(started_list);
         free(list_text);
         list_text = NULL;
         list_text_len = 0;
     }
 
-    if (!pcmk_is_set(options, pe_print_clone_active)) {
+    if (pcmk_is_set(show_opts, pcmk_show_inactive_rscs)) {
         const char *state = "Stopped";
         enum rsc_role_e role = configured_role(rsc);
 
@@ -832,8 +900,8 @@ pe__clone_html(pcmk__output_t *out, va_list args)
         if (!pcmk_is_set(rsc->flags, pe_rsc_unique)
             && (clone_data->clone_max > active_instances)) {
 
-            GListPtr nIter;
-            GListPtr list = g_hash_table_get_values(rsc->allowed_nodes);
+            GList *nIter;
+            GList *list = g_hash_table_get_values(rsc->allowed_nodes);
 
             /* Custom stopped list for non-unique clones */
             free(stopped_list);
@@ -852,7 +920,8 @@ pe__clone_html(pcmk__output_t *out, va_list args)
                 pe_node_t *node = (pe_node_t *)nIter->data;
 
                 if (pe_find_node(rsc->running_on, node->details->uname) == NULL &&
-                    pcmk__str_in_list(only_node, node->details->uname)) {
+                    pcmk__str_in_list(node->details->uname, only_node,
+                                      pcmk__str_star_matches|pcmk__str_casei)) {
                     pcmk__add_word(&stopped_list, &stopped_list_len,
                                    node->details->uname);
                 }
@@ -861,248 +930,24 @@ pe__clone_html(pcmk__output_t *out, va_list args)
         }
 
         if (stopped_list != NULL) {
-            out->list_item(out, NULL, " %s: [ %s ]", state, stopped_list);
+            clone_header(out, &rc, rsc, clone_data);
+
+            out->list_item(out, NULL, "%s: [ %s ]", state, stopped_list);
             free(stopped_list);
             stopped_list_len = 0;
+
+        /* If there are no instances of this clone (perhaps because there are no
+         * nodes configured), simply output the clone header by itself.  This can
+         * come up in PCS testing.
+         */
+        } else if (active_instances == 0) {
+            clone_header(out, &rc, rsc, clone_data);
+            PCMK__OUTPUT_LIST_FOOTER(out, rc);
+            return rc;
         }
     }
 
-    out->end_list(out);
-
-    return rc;
-}
-
-PCMK__OUTPUT_ARGS("clone", "unsigned int", "pe_resource_t *", "GListPtr", "GListPtr")
-int
-pe__clone_text(pcmk__output_t *out, va_list args)
-{
-    unsigned int options = va_arg(args, unsigned int);
-    pe_resource_t *rsc = va_arg(args, pe_resource_t *);
-    GListPtr only_node = va_arg(args, GListPtr);
-    GListPtr only_rsc = va_arg(args, GListPtr);
-
-    char *list_text = NULL;
-    char *stopped_list = NULL;
-    size_t list_text_len = 0;
-    size_t stopped_list_len = 0;
-
-    GListPtr master_list = NULL;
-    GListPtr started_list = NULL;
-    GListPtr gIter = rsc->children;
-
-    clone_variant_data_t *clone_data = NULL;
-    int active_instances = 0;
-    int rc = pcmk_rc_no_output;
-    gboolean print_everything = TRUE;
-
-    get_clone_variant_data(clone_data, rsc);
-
-    if (rsc->fns->is_filtered(rsc, only_rsc, TRUE)) {
-        return rc;
-    }
-
-    print_everything = pcmk__str_in_list(only_rsc, rsc_printable_id(rsc)) ||
-                       (strstr(rsc->id, ":") != NULL && pcmk__str_in_list(only_rsc, rsc->id));
-
-    out->begin_list(out, NULL, NULL, "Clone Set: %s [%s]%s%s%s%s",
-                    rsc->id, ID(clone_data->xml_obj_child),
-                    pcmk_is_set(rsc->flags, pe_rsc_promotable) ? " (promotable)" : "",
-                    pcmk_is_set(rsc->flags, pe_rsc_unique) ? " (unique)" : "",
-                    pcmk_is_set(rsc->flags, pe_rsc_managed) ? "" : " (unmanaged)",
-                    pe__resource_is_disabled(rsc) ? " (disabled)" : "");
-    rc = pcmk_rc_ok;
-
-    for (; gIter != NULL; gIter = gIter->next) {
-        gboolean print_full = FALSE;
-        pe_resource_t *child_rsc = (pe_resource_t *) gIter->data;
-        gboolean partially_active = child_rsc->fns->active(child_rsc, FALSE);
-
-        if (pcmk__rsc_filtered_by_node(child_rsc, only_node)) {
-            continue;
-        }
-
-        if (child_rsc->fns->is_filtered(child_rsc, only_rsc, print_everything)) {
-            continue;
-        }
-
-        if (options & pe_print_clone_details) {
-            print_full = TRUE;
-        }
-
-        if (pcmk_is_set(rsc->flags, pe_rsc_unique)) {
-            // Print individual instance when unique (except stopped orphans)
-            if (partially_active || !pcmk_is_set(rsc->flags, pe_rsc_orphan)) {
-                print_full = TRUE;
-            }
-
-        // Everything else in this block is for anonymous clones
-
-        } else if (pcmk_is_set(options, pe_print_pending)
-                   && (child_rsc->pending_task != NULL)
-                   && strcmp(child_rsc->pending_task, "probe")) {
-            // Print individual instance when non-probe action is pending
-            print_full = TRUE;
-
-        } else if (partially_active == FALSE) {
-            // List stopped instances when requested (except orphans)
-            if (!pcmk_is_set(child_rsc->flags, pe_rsc_orphan)
-                && !pcmk_is_set(options, pe_print_clone_active)) {
-                pcmk__add_word(&stopped_list, &stopped_list_len, child_rsc->id);
-            }
-
-        } else if (is_set_recursive(child_rsc, pe_rsc_orphan, TRUE)
-                   || is_set_recursive(child_rsc, pe_rsc_managed, FALSE) == FALSE
-                   || is_set_recursive(child_rsc, pe_rsc_failed, TRUE)) {
-
-            // Print individual instance when active orphaned/unmanaged/failed
-            print_full = TRUE;
-
-        } else if (child_rsc->fns->active(child_rsc, TRUE)) {
-            // Instance of fully active anonymous clone
-
-            pe_node_t *location = child_rsc->fns->location(child_rsc, NULL, TRUE);
-
-            if (location) {
-                // Instance is active on a single node
-
-                enum rsc_role_e a_role = child_rsc->fns->state(child_rsc, TRUE);
-
-                if (location->details->online == FALSE && location->details->unclean) {
-                    print_full = TRUE;
-
-                } else if (a_role > RSC_ROLE_SLAVE) {
-                    master_list = g_list_append(master_list, location);
-
-                } else {
-                    started_list = g_list_append(started_list, location);
-                }
-
-            } else {
-                /* uncolocated group - bleh */
-                print_full = TRUE;
-            }
-
-        } else {
-            // Instance of partially active anonymous clone
-            print_full = TRUE;
-        }
-
-        if (print_full) {
-            GListPtr all = NULL;
-
-            /* Print every resource that's a child of this clone. */
-            all = g_list_prepend(all, strdup("*"));
-            out->message(out, crm_map_element_name(child_rsc->xml), options,
-                         child_rsc, only_node, all);
-            g_list_free_full(all, free);
-        }
-    }
-
-    if (pcmk_is_set(options, pe_print_clone_details)) {
-        free(stopped_list);
-        out->end_list(out);
-        return pcmk_rc_ok;
-    }
-
-    /* Masters */
-    master_list = g_list_sort(master_list, sort_node_uname);
-    for (gIter = master_list; gIter; gIter = gIter->next) {
-        pe_node_t *host = gIter->data;
-
-        if (!pcmk__str_in_list(only_node, host->details->uname)) {
-            continue;
-        }
-
-        pcmk__add_word(&list_text, &list_text_len, host->details->uname);
-        active_instances++;
-    }
-
-    if (list_text != NULL) {
-        out->list_item(out, "Masters", "[ %s ]", list_text);
-        g_list_free(master_list);
-        free(list_text);
-        list_text = NULL;
-        list_text_len = 0;
-    }
-
-    /* Started/Slaves */
-    started_list = g_list_sort(started_list, sort_node_uname);
-    for (gIter = started_list; gIter; gIter = gIter->next) {
-        pe_node_t *host = gIter->data;
-
-        if (!pcmk__str_in_list(only_node, host->details->uname)) {
-            continue;
-        }
-
-        pcmk__add_word(&list_text, &list_text_len, host->details->uname);
-        active_instances++;
-    }
-
-    if (list_text != NULL) {
-        if (pcmk_is_set(rsc->flags, pe_rsc_promotable)) {
-            enum rsc_role_e role = configured_role(rsc);
-
-            if(role == RSC_ROLE_SLAVE) {
-                out->list_item(out, "Slaves (target-role)", "[ %s ]", list_text);
-            } else {
-                out->list_item(out, "Slaves", "[ %s ]", list_text);
-            }
-        } else {
-            out->list_item(out, "Started", "[ %s ]", list_text);
-        }
-
-        g_list_free(started_list);
-        free(list_text);
-        list_text = NULL;
-    }
-
-    if (!pcmk_is_set(options, pe_print_clone_active)) {
-        const char *state = "Stopped";
-        enum rsc_role_e role = configured_role(rsc);
-
-        if (role == RSC_ROLE_STOPPED) {
-            state = "Stopped (disabled)";
-        }
-
-        if (!pcmk_is_set(rsc->flags, pe_rsc_unique)
-            && (clone_data->clone_max > active_instances)) {
-
-            GListPtr nIter;
-            GListPtr list = g_hash_table_get_values(rsc->allowed_nodes);
-
-            /* Custom stopped list for non-unique clones */
-            free(stopped_list);
-            stopped_list = NULL;
-            stopped_list_len = 0;
-
-            if (list == NULL) {
-                /* Clusters with symmetrical=false haven't calculated allowed_nodes yet
-                 * If we've not probed for them yet, the Stopped list will be empty
-                 */
-                list = g_hash_table_get_values(rsc->known_on);
-            }
-
-            list = g_list_sort(list, sort_node_uname);
-            for (nIter = list; nIter != NULL; nIter = nIter->next) {
-                pe_node_t *node = (pe_node_t *)nIter->data;
-
-                if (pe_find_node(rsc->running_on, node->details->uname) == NULL &&
-                    pcmk__str_in_list(only_node, node->details->uname)) {
-                    pcmk__add_word(&stopped_list, &stopped_list_len,
-                                   node->details->uname);
-                }
-            }
-            g_list_free(list);
-        }
-
-        if (stopped_list != NULL) {
-            out->list_item(out, state, "[ %s ]", stopped_list);
-            free(stopped_list);
-        }
-    }
-
-    out->end_list(out);
-
+    PCMK__OUTPUT_LIST_FOOTER(out, rc);
     return rc;
 }
 
@@ -1115,7 +960,7 @@ clone_free(pe_resource_t * rsc)
 
     pe_rsc_trace(rsc, "Freeing %s", rsc->id);
 
-    for (GListPtr gIter = rsc->children; gIter != NULL; gIter = gIter->next) {
+    for (GList *gIter = rsc->children; gIter != NULL; gIter = gIter->next) {
         pe_resource_t *child_rsc = (pe_resource_t *) gIter->data;
 
         CRM_ASSERT(child_rsc);
@@ -1144,7 +989,7 @@ enum rsc_role_e
 clone_resource_state(const pe_resource_t * rsc, gboolean current)
 {
     enum rsc_role_e clone_role = RSC_ROLE_UNKNOWN;
-    GListPtr gIter = rsc->children;
+    GList *gIter = rsc->children;
 
     for (; gIter != NULL; gIter = gIter->next) {
         pe_resource_t *child_rsc = (pe_resource_t *) gIter->data;
@@ -1182,19 +1027,19 @@ pe__is_universal_clone(pe_resource_t *rsc,
 }
 
 gboolean
-pe__clone_is_filtered(pe_resource_t *rsc, GListPtr only_rsc, gboolean check_parent)
+pe__clone_is_filtered(pe_resource_t *rsc, GList *only_rsc, gboolean check_parent)
 {
     gboolean passes = FALSE;
     clone_variant_data_t *clone_data = NULL;
 
-    if (pcmk__str_in_list(only_rsc, rsc_printable_id(rsc))) {
+    if (pcmk__str_in_list(rsc_printable_id(rsc), only_rsc, pcmk__str_star_matches)) {
         passes = TRUE;
     } else {
         get_clone_variant_data(clone_data, rsc);
-        passes = pcmk__str_in_list(only_rsc, ID(clone_data->xml_obj_child));
+        passes = pcmk__str_in_list(ID(clone_data->xml_obj_child), only_rsc, pcmk__str_star_matches);
 
         if (!passes) {
-            for (GListPtr gIter = rsc->children; gIter != NULL; gIter = gIter->next) {
+            for (GList *gIter = rsc->children; gIter != NULL; gIter = gIter->next) {
                 pe_resource_t *child_rsc = (pe_resource_t *) gIter->data;
 
                 if (!child_rsc->fns->is_filtered(child_rsc, only_rsc, FALSE)) {
