@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2020 the Pacemaker project contributors
+ * Copyright 2004-2021 the Pacemaker project contributors
  *
  * The version control history for this file may have further details.
  *
@@ -140,37 +140,6 @@ find_expression_type(xmlNode * expr)
     return attr_expr;
 }
 
-gboolean
-pe_test_role_expression(xmlNode *expr, enum rsc_role_e role, crm_time_t *now)
-{
-    pe_rule_eval_data_t rule_data = {
-        .node_hash = NULL,
-        .role = role,
-        .now = now,
-        .match_data = NULL,
-        .rsc_data = NULL,
-        .op_data = NULL
-    };
-
-    return pe__eval_role_expr(expr, &rule_data);
-}
-
-gboolean
-pe_test_attr_expression(xmlNode *expr, GHashTable *hash, crm_time_t *now,
-                        pe_match_data_t *match_data)
-{
-    pe_rule_eval_data_t rule_data = {
-        .node_hash = hash,
-        .role = RSC_ROLE_UNKNOWN,
-        .now = now,
-        .match_data = match_data,
-        .rsc_data = NULL,
-        .op_data = NULL
-    };
-
-    return pe__eval_attr_expr(expr, &rule_data);
-}
-
 /* As per the nethack rules:
  *
  * moon period = 29.53058 days ~= 30, year = 365.2422 days
@@ -304,65 +273,36 @@ pe_cron_range_satisfied(crm_time_t * now, xmlNode * cron_spec)
     return pcmk_rc_ok;
 }
 
-#define update_field(xml_field, time_fn)			\
-    value = crm_element_value(duration_spec, xml_field);	\
-    if(value != NULL) {						\
-	int value_i = crm_parse_int(value, "0");		\
-	time_fn(end, value_i);					\
+static void
+update_field(crm_time_t *t, xmlNode *xml, const char *attr,
+            void (*time_fn)(crm_time_t *, int))
+{
+    long long value;
+
+    if ((pcmk__scan_ll(crm_element_value(xml, attr), &value, 0LL) == pcmk_rc_ok)
+        && (value != 0LL) && (value >= INT_MIN) && (value <= INT_MAX)) {
+        time_fn(t, (int) value);
     }
+}
 
 crm_time_t *
 pe_parse_xml_duration(crm_time_t * start, xmlNode * duration_spec)
 {
-    crm_time_t *end = NULL;
-    const char *value = NULL;
+    crm_time_t *end = crm_time_new_undefined();
 
-    end = crm_time_new(NULL);
     crm_time_set(end, start);
 
-    update_field("years", crm_time_add_years);
-    update_field("months", crm_time_add_months);
-    update_field("weeks", crm_time_add_weeks);
-    update_field("days", crm_time_add_days);
-    update_field("hours", crm_time_add_hours);
-    update_field("minutes", crm_time_add_minutes);
-    update_field("seconds", crm_time_add_seconds);
+    update_field(end, duration_spec, "years", crm_time_add_years);
+    update_field(end, duration_spec, "months", crm_time_add_months);
+    update_field(end, duration_spec, "weeks", crm_time_add_weeks);
+    update_field(end, duration_spec, "days", crm_time_add_days);
+    update_field(end, duration_spec, "hours", crm_time_add_hours);
+    update_field(end, duration_spec, "minutes", crm_time_add_minutes);
+    update_field(end, duration_spec, "seconds", crm_time_add_seconds);
 
     return end;
 }
 
-/*!
- * \internal
- * \brief Test a date expression (pass/fail) for a specific time
- *
- * \param[in]  time_expr    date_expression XML
- * \param[in]  now          Time for which to evaluate expression
- * \param[out] next_change  If not NULL, set to when evaluation will change
- *
- * \return TRUE if date expression is in effect at given time, FALSE otherwise
- */
-gboolean
-pe_test_date_expression(xmlNode *expr, crm_time_t *now, crm_time_t *next_change)
-{
-    pe_rule_eval_data_t rule_data = {
-        .node_hash = NULL,
-        .role = RSC_ROLE_UNKNOWN,
-        .now = now,
-        .match_data = NULL,
-        .rsc_data = NULL,
-        .op_data = NULL
-    };
-
-    switch (pe__eval_date_expr(expr, &rule_data, next_change)) {
-        case pcmk_rc_within_range:
-        case pcmk_rc_ok:
-            return TRUE;
-
-        default:
-            return FALSE;
-    }
-}
-
 // Set next_change to t if t is earlier
 static void
 crm_time_set_if_earlier(crm_time_t *next_change, crm_time_t *t)
@@ -375,31 +315,6 @@ crm_time_set_if_earlier(crm_time_t *next_change, crm_time_t *t)
     }
 }
 
-/*!
- * \internal
- * \brief Evaluate a date expression for a specific time
- *
- * \param[in]  time_expr    date_expression XML
- * \param[in]  now          Time for which to evaluate expression
- * \param[out] next_change  If not NULL, set to when evaluation will change
- *
- * \return Standard Pacemaker return code
- */
-int
-pe_eval_date_expression(xmlNode *expr, crm_time_t *now, crm_time_t *next_change)
-{
-    pe_rule_eval_data_t rule_data = {
-        .node_hash = NULL,
-        .role = RSC_ROLE_UNKNOWN,
-        .now = now,
-        .match_data = NULL,
-        .rsc_data = NULL,
-        .op_data = NULL
-    };
-
-    return pe__eval_date_expr(expr, &rule_data, next_change);
-}
-
 // Information about a block of nvpair elements
 typedef struct sorted_set_s {
     int score;                  // This block's score for sorting
@@ -617,31 +532,31 @@ static GList *
 make_pairs(xmlNode *top, xmlNode *xml_obj, const char *set_name,
            const char *always_first)
 {
-    GListPtr unsorted = NULL;
-    const char *score = NULL;
-    sorted_set_t *pair = NULL;
-    xmlNode *attr_set = NULL;
+    GList *unsorted = NULL;
 
     if (xml_obj == NULL) {
         return NULL;
     }
-    for (attr_set = pcmk__xe_first_child(xml_obj); attr_set != NULL;
+    for (xmlNode *attr_set = pcmk__xe_first_child(xml_obj); attr_set != NULL;
          attr_set = pcmk__xe_next(attr_set)) {
 
-        /* Uncertain if set_name == NULL check is strictly necessary here */
-        if (pcmk__str_eq(set_name, (const char *)attr_set->name, pcmk__str_null_matches)) {
-            pair = NULL;
-            attr_set = expand_idref(attr_set, top);
-            if (attr_set == NULL) {
+        if (pcmk__str_eq(set_name, (const char *) attr_set->name,
+                         pcmk__str_null_matches)) {
+            const char *score = NULL;
+            sorted_set_t *pair = NULL;
+            xmlNode *expanded_attr_set = expand_idref(attr_set, top);
+
+            if (expanded_attr_set == NULL) {
+                // Schema (if not "none") prevents this
                 continue;
             }
 
             pair = calloc(1, sizeof(sorted_set_t));
-            pair->name = ID(attr_set);
+            pair->name = ID(expanded_attr_set);
             pair->special_name = always_first;
-            pair->attr_set = attr_set;
+            pair->attr_set = expanded_attr_set;
 
-            score = crm_element_value(attr_set, XML_RULE_ATTR_SCORE);
+            score = crm_element_value(expanded_attr_set, XML_RULE_ATTR_SCORE);
             pair->score = char2score(score);
 
             unsorted = g_list_prepend(unsorted, pair);
@@ -800,10 +715,10 @@ pe_expand_re_matches(const char *string, pe_re_match_data_t *match_data)
 GHashTable*
 pe_unpack_versioned_parameters(xmlNode *versioned_params, const char *ra_version)
 {
-    GHashTable *hash = crm_str_table_new();
+    GHashTable *hash = pcmk__strkey_table(free, free);
 
     if (versioned_params && ra_version) {
-        GHashTable *node_hash = crm_str_table_new();
+        GHashTable *node_hash = pcmk__strkey_table(free, free);
         xmlNode *attr_set = pcmk__xe_first_child(versioned_params);
 
         if (attr_set) {
@@ -908,7 +823,16 @@ pe_eval_subexpr(xmlNode *expr, pe_rule_eval_data_t *rule_data, crm_time_t *next_
             break;
 
         case time_expr:
-            accept = pe_test_date_expression(expr, rule_data->now, next_change);
+            switch (pe__eval_date_expr(expr, rule_data, next_change)) {
+                case pcmk_rc_within_range:
+                case pcmk_rc_ok:
+                    accept = TRUE;
+                    break;
+
+                default:
+                    accept = FALSE;
+                    break;
+            }
             break;
 
         case role_expr:
@@ -989,13 +913,13 @@ compare_attr_expr_vals(const char *l_val, const char *r_val, const char *type,
             cmp = strcasecmp(l_val, r_val);
 
         } else if (pcmk__str_eq(type, "integer", pcmk__str_casei)) {
-            long long l_val_num = crm_parse_ll(l_val, NULL);
-            int rc1 = errno;
+            long long l_val_num;
+            int rc1 = pcmk__scan_ll(l_val, &l_val_num, 0LL);
 
-            long long r_val_num = crm_parse_ll(r_val, NULL);
-            int rc2 = errno;
+            long long r_val_num;
+            int rc2 = pcmk__scan_ll(r_val, &r_val_num, 0LL);
 
-            if (rc1 == 0 && rc2 == 0) {
+            if ((rc1 == pcmk_rc_ok) && (rc2 == pcmk_rc_ok)) {
                 if (l_val_num < r_val_num) {
                     cmp = -1;
                 } else if (l_val_num > r_val_num) {
@@ -1104,12 +1028,54 @@ accept_attr_expr(const char *l_val, const char *r_val, const char *type,
     return false;   // Should never reach this point
 }
 
+/*!
+ * \internal
+ * \brief Get correct value according to value-source
+ *
+ * \param[in] value         value given in rule expression
+ * \param[in] value_source  value-source given in rule expressions
+ * \param[in] match_data    If not NULL, resource back-references and params
+ */
+static const char *
+expand_value_source(const char *value, const char *value_source,
+                    pe_match_data_t *match_data)
+{
+    GHashTable *table = NULL;
+
+    if (pcmk__str_empty(value)) {
+        return NULL; // value_source is irrelevant
+
+    } else if (pcmk__str_eq(value_source, "param", pcmk__str_casei)) {
+        table = match_data->params;
+
+    } else if (pcmk__str_eq(value_source, "meta", pcmk__str_casei)) {
+        table = match_data->meta;
+
+    } else { // literal
+        return value;
+    }
+
+    if (table == NULL) {
+        return NULL;
+    }
+    return (const char *) g_hash_table_lookup(table, value);
+}
+
+/*!
+ * \internal
+ * \brief Evaluate a node attribute expression based on #uname, #id, #kind,
+ *        or a generic node attribute
+ *
+ * \param[in] expr       XML of rule expression
+ * \param[in] rule_data  The match_data and node_hash members are used
+ *
+ * \return TRUE if rule_data satisfies the expression, FALSE otherwise
+ */
 gboolean
 pe__eval_attr_expr(xmlNodePtr expr, pe_rule_eval_data_t *rule_data)
 {
     gboolean attr_allocated = FALSE;
     const char *h_val = NULL;
-    GHashTable *table = NULL;
 
     const char *op = NULL;
     const char *type = NULL;
@@ -1129,32 +1095,19 @@ pe__eval_attr_expr(xmlNodePtr expr, pe_rule_eval_data_t *rule_data)
         return FALSE;
     }
 
-    if (rule_data->match_data) {
-        if (rule_data->match_data->re) {
+    if (rule_data->match_data != NULL) {
+        // Expand any regular expression submatches (%0-%9) in attribute name
+        if (rule_data->match_data->re != NULL) {
             char *resolved_attr = pe_expand_re_matches(attr, rule_data->match_data->re);
 
-            if (resolved_attr) {
+            if (resolved_attr != NULL) {
                 attr = (const char *) resolved_attr;
                 attr_allocated = TRUE;
             }
         }
 
-        if (pcmk__str_eq(value_source, "param", pcmk__str_casei)) {
-            table = rule_data->match_data->params;
-        } else if (pcmk__str_eq(value_source, "meta", pcmk__str_casei)) {
-            table = rule_data->match_data->meta;
-        }
-    }
-
-    if (table) {
-        const char *param_name = value;
-        const char *param_value = NULL;
-
-        if (param_name && param_name[0]) {
-            if ((param_value = (const char *)g_hash_table_lookup(table, param_name))) {
-                value = param_value;
-            }
-        }
+        // Get value appropriate to value-source
+        value = expand_value_source(value, value_source, rule_data->match_data);
     }
 
     if (rule_data->node_hash != NULL) {
@@ -1169,8 +1122,16 @@ pe__eval_attr_expr(xmlNodePtr expr, pe_rule_eval_data_t *rule_data)
     return accept_attr_expr(h_val, value, type, op);
 }
 
-
-
+/*!
+ * \internal
+ * \brief Evaluate a date_expression
+ *
+ * \param[in]  expr         XML of rule expression
+ * \param[in]  rule_data    Only the now member is used
+ * \param[out] next_change  If not NULL, set to when evaluation will change
+ *
+ * \return Standard Pacemaker return code
+ */
 int
 pe__eval_date_expr(xmlNodePtr expr, pe_rule_eval_data_t *rule_data, crm_time_t *next_change)
 {
@@ -1285,6 +1246,15 @@ pe__eval_op_expr(xmlNodePtr expr, pe_rule_eval_data_t *rule_data) {
     return TRUE;
 }
 
+/*!
+ * \internal
+ * \brief Evaluate a node attribute expression based on #role
+ *
+ * \param[in] expr       XML of rule expression
+ * \param[in] rule_data  Only the role member is used
+ *
+ * \return TRUE if rule_data->role satisfies the expression, FALSE otherwise
+ */
 gboolean
 pe__eval_role_expr(xmlNodePtr expr, pe_rule_eval_data_t *rule_data)
 {
@@ -1305,7 +1275,8 @@ pe__eval_role_expr(xmlNodePtr expr, pe_rule_eval_data_t *rule_data)
         }
 
     } else if (pcmk__str_eq(op, "not_defined", pcmk__str_casei)) {
-        if (rule_data->role < RSC_ROLE_SLAVE && rule_data->role > RSC_ROLE_UNKNOWN) {
+        if ((rule_data->role > RSC_ROLE_UNKNOWN)
+            && (rule_data->role < RSC_ROLE_UNPROMOTED)) {
             accept = TRUE;
         }
 
@@ -1316,7 +1287,8 @@ pe__eval_role_expr(xmlNodePtr expr, pe_rule_eval_data_t *rule_data)
 
     } else if (pcmk__str_eq(op, "ne", pcmk__str_casei)) {
         // Test "ne" only with promotable clone roles
-        if (rule_data->role < RSC_ROLE_SLAVE && rule_data->role > RSC_ROLE_UNKNOWN) {
+        if ((rule_data->role > RSC_ROLE_UNKNOWN)
+            && (rule_data->role < RSC_ROLE_UNPROMOTED)) {
             accept = FALSE;
 
         } else if (text2role(value) != rule_data->role) {
@@ -1363,27 +1335,9 @@ pe__eval_rsc_expr(xmlNodePtr expr, pe_rule_eval_data_t *rule_data)
 }
 
 // Deprecated functions kept only for backward API compatibility
-gboolean test_ruleset(xmlNode *ruleset, GHashTable *node_hash, crm_time_t *now);
-gboolean test_rule(xmlNode *rule, GHashTable *node_hash, enum rsc_role_e role,
-                   crm_time_t *now);
-gboolean pe_test_rule_re(xmlNode *rule, GHashTable *node_hash,
-                         enum rsc_role_e role, crm_time_t *now,
-                         pe_re_match_data_t *re_match_data);
-gboolean pe_test_rule_full(xmlNode *rule, GHashTable *node_hash,
-                           enum rsc_role_e role, crm_time_t *now,
-                           pe_match_data_t *match_data);
-gboolean test_expression(xmlNode *expr, GHashTable *node_hash,
-                         enum rsc_role_e role, crm_time_t *now);
-gboolean pe_test_expression_re(xmlNode *expr, GHashTable *node_hash,
-                               enum rsc_role_e role, crm_time_t *now,
-                               pe_re_match_data_t *re_match_data);
-gboolean pe_test_expression_full(xmlNode *expr, GHashTable *node_hash,
-                                 enum rsc_role_e role, crm_time_t *now,
-                                 pe_match_data_t *match_data);
-void unpack_instance_attributes(xmlNode *top, xmlNode *xml_obj,
-                                const char *set_name, GHashTable *node_hash,
-                                GHashTable *hash, const char *always_first,
-                                gboolean overwrite, crm_time_t *now);
+// LCOV_EXCL_START
+
+#include <crm/pengine/rules_compat.h>
 
 gboolean
 test_ruleset(xmlNode *ruleset, GHashTable *node_hash, crm_time_t *now)
@@ -1458,3 +1412,6 @@ unpack_instance_attributes(xmlNode *top, xmlNode *xml_obj, const char *set_name,
     unpack_nvpair_blocks(top, xml_obj, set_name, hash, always_first,
                          overwrite, &rule_data, NULL, unpack_attr_set);
 }
+
+// LCOV_EXCL_STOP
+// End deprecated API
